dojo.provide("websheet.ColumnHelper");

websheet.ColumnHelper = {

	translateFromColCharacters: function(column) {
		var ret = -1;
		if (column) {
			var i = 0;
			ret = 0;
			var Z = this.columnCharToInt('Z');
			var tcolumn = column.toUpperCase();
			for (var o = (tcolumn.length - 1); o >= 0; o--) {
				ret += this.columnCharToInt(tcolumn.charAt(o)) * Math.pow(Z, i++);
			}
		} 
		return ret;
	},
	
	translateFromColNum: function(num) {
		var tempColNum = num;
		var Z = this.columnCharToInt('Z');
		var sb = "";
		
		while (tempColNum >= 1) {
			var mod = tempColNum % Z;
			var div = tempColNum / Z;
			if (mod === 0) {
				mod = Z;
				div = div - 1;
			}
			sb = this.intToColumnChar(mod) + sb;
			tempColNum = div;
		}
		return sb;
	},
	
	findBoundaries: function(sheetObj) {
		var minRow = 0;
		var minColumnNumber = 0;
		var maxRow = 0;
		var maxColumnNumber = 0;
		if (sheetObj && sheetObj.rows) {
			for (var row in sheetObj.rows) {
				if (row != '_id') {
					// dont max row here - gets wrong results on empty sheets
					for (var column in sheetObj.rows[row].cells) {
						if (column != '_id') {
							// max row only if there are columns in it...
							var curRow = parseInt(row, 10);
							var curCol = this.translateFromColCharacters(column);
							maxRow = Math.max(maxRow, curRow);
							maxColumnNumber = Math.max(maxColumnNumber, curCol);
							
							if(minRow === 0) {
								minRow = curRow;
							}
							// min row only if there are columns in it...
							minRow = Math.min(minRow, curRow);
							minColumnNumber = Math.min(minColumnNumber, curCol);
						}
					}
				}
			}
		}
		
		return {
			maxRow: maxRow,
			maxColumnNumber: maxColumnNumber,
			maxColumn: this.translateFromColNum(maxColumnNumber),
			minRow: minRow,
			minColumnNumber: minColumnNumber,
			minColumn: this.translateFromColNum(minColumnNumber)
		};
	},
	
	columnCharToInt: function(c) {
		return 1 + (c.toUpperCase().charCodeAt(0) - 'A'.charCodeAt(0));
	},
	
	
	intToColumnChar: function(i) {
		var ti = (i - 1) % this.columnCharToInt('Z');
		return String.fromCharCode('A'.charCodeAt(0) + ti);
	},
	
	/**
	 * returns 1 if a > b, -1 if a < b, 0 if equal.
	 *
	 * Doesn't work with \0 values
	 *
	 * @param {String} a
	 * @param {String} b
	 */
	comparator: function(a, b) {
		var left = this.translateFromColCharacters(a);
		var right = this.translateFromColCharacters(b);
		
		return left - right;
	},
	
	/**
	 * analyze result the parser get,then return the real result.Maybe it is a websheet.parse.tokenList,or some other type,
	 * we only want to get the calculative result to join the next calculation.
	 * @param {Object} result
	 */
	analyzeResult:function(result){
		if(result instanceof websheet.parse.tokenList || result instanceof websheet.parse.token){
			var showValue = result.getCalcuteValue();
			if(result.getTokenType() == websheet.parse.tokenType.NEGATIVE_TOKEN){
				showValue = this.negativeCalculate(result,0);
			}
			return showValue;
		}else{
			return result;
		}
	},
	negativeCalculate:function(token,count){
		var value;
		if(token.getTokenType() == websheet.parse.tokenType.NEGATIVE_TOKEN){
			value = dojo.hitch(this,"negativeCalculate")(token.getTokenList()[0],count + 1);
		}else{
			if(token.getTokenType() == websheet.parse.tokenType.CELLREF_TOKEN){
				value = token.getCalcuteValue().getRows()[0];
				if(count % 2 != 0){
					value = this.transForm2Negative(value);
				}
			}else if(token.getTokenType() == websheet.parse.tokenType.RANGEREF_TOKEN){
				value = token.getCalcuteValue();
				dojo.forEach(value.getRows(),function(row,index){
					if(dojo.isArray(row)){
						dojo.forEach(row,function(cell,cellIndex){
							if(count % 2 != 0){
								row[cellIndex] = this.transForm2Negative(cell);
							}
						},this);
					}else if(count % 2 != 0){
						value.getRows()[index] = this.transForm2Negative(row);
					}
				},this);
				dojo.forEach(value.getCols(),function(col,index){
					if(dojo.isArray(col)){
						dojo.forEach(col,function(cell,cellIndex){
							if(count % 2 != 0){
								col[cellIndex] = this.transForm2Negative(cell);
							}
						},this);
					}else if(count % 2 != 0){
						value.getCols()[index] = this.transForm2Negative(row);
					}
				},this);
			}else{
				value = token.getCalcuteValue();
				if(count % 2 != 0){
					value = this.transForm2Negative(value);
				}
			}
		}
		return value;
	},
	transForm2Negative:function(value){
		if(value == null || value == undefined){return null;} 
		if(value != 0){
			if(value.toString().charAt(0) == "-"){
			   if(websheet.Helper.isNumeric(value.toString().substring(1))){ 
				   value = -value;
			   }else{
				   value = "-" + value;
			   }
		   }else{
	    	   if(websheet.Helper.isNumeric(value)){
	    		   value = -value;
			   }else{
				   value = "-" + value;
			   }
	       }
	    }
		return value;
	},
	/**
	 * generate the token-list tree to UI.
	 */
	serialize2String:function(cell,isShowValue){
		if(cell.getTokenList().length != 0){
			if(isShowValue){
				var divNode = dojo.doc.createElement("div");
				divNode.innerHTML = "=" + cell.getTokenList().serialize2String(isShowValue);
				dojo.doc.body.appendChild(divNode);
				//return "=" + cell.getTokenList().serialize2String(isShowValue);
			}else{var tableNode = dojo.doc.createElement("table");
				var trNode = dojo.doc.createElement("tr");
				trNode.innerHTML = cell._rawValue;
				tableNode.appendChild(trNode);
				var trNode2 = cell.getTokenList().serialize2String(isShowValue);
				if(typeof trNode2 != "object"){
					trNode2 = dojo.doc.createElement("tr");
					trNode2.innerHTML = cell.getTokenList().serialize2String(isShowValue);
				}
				tableNode.appendChild(trNode2);
				dojo.doc.body.appendChild(tableNode);
				dojo.doc.body.appendChild(dojo.doc.createElement("br"));
				dojo.doc.body.appendChild(dojo.doc.createElement("br"));
				dojo.doc.body.appendChild(dojo.doc.createElement("br"));
			}
		}
	},
	
	/**
	 * 
	 * @param:relativeCellRef	E.g:B1,$B1,B$1,$B$1,if current cell is A7 , the relative formats are R[6]C[-1],R6C[-1],R[6]C-1,R6C-1 
	 * 
	 */
	parse2RelativePosition: function(curRow,curCol,relativeCellRef){
		var relativePos = "";
		var sheetStart = relativeCellRef.indexOf(".");
		if(sheetStart != -1){
			relativePos = relativeCellRef.substring(0,sheetStart + 1);
			relativeCellRef = relativeCellRef.substring(sheetStart + 1);
		}
		var relativeCell = this.parse2RACDueRef(relativeCellRef);
		//row whether is absolute
		if(relativeCell[3]){
			relativePos = "R" +  (relativeCell[1] - curRow);
		}else{
			relativePos = "R[" +  (relativeCell[1] - curRow) + "]";
		}	
		//column whether is absolute
		if(relativeCell[2]){
			relativePos += "C" +  (this.translateFromColCharacters(relativeCell[0]) - this.translateFromColCharacters(curCol));
		}else{
			relativePos += "C[" +  (this.translateFromColCharacters(relativeCell[0]) - this.translateFromColCharacters(curCol)) + "]";
		}	
		return relativePos;
	},
	
	parse2RACDueRef:function(cellRef){
		var RAC = new Array();
		var rowStart = 0;
		var colStart = 0;
		var isColAbsolute = false;
		var isRowAbsolute = false;
		if(cellRef[0]=="$"){
			isColAbsolute = true;
			colStart = 1;
		}
		for(var i = colStart;i <= cellRef.length -1;i++){
			if(!this.isLetter(cellRef[i])){
				if(cellRef[i] == "$"){
					isRowAbsolute = true;
				}
				rowStart = i;
				break;
			}
		}
		RAC.push(cellRef.substring(colStart,rowStart));
		if(isRowAbsolute){
			RAC.push(cellRef.substring(rowStart + 1));
		}else{
			RAC.push(cellRef.substring(rowStart));
		}	
		RAC.push(isColAbsolute);
		RAC.push(isRowAbsolute);
		return RAC;
	},
	
	isLetter:function(character) {  
		return (character >= "a" && character <= "z") || (character >= "A" && character <= "Z");  
	} 
};