/********************************************************/
/* created by Rui SuYing, ruisuy@cn.ibm.com				*/
/*  2010/6/28											*/
/* for range object in a spreadsheet document			*/
/********************************************************/

dojo.provide("websheet.model.ColumnModel");

dojo.declare("websheet.model.ColumnModel",null,{
	_id : null,
	
	index:null,
	
	_dependencedList:null,			//cells list which is dependent on cells in this sheet. 
	
	isDelete:false,
	
	property: null, 				// sheet property, now for hide or show or filtered,
	
	width:null,
	
	styleCode:null,
	
	styleID:null,					//reserved for future support of style
	
	repeatednum:null,
	
	_parent: null,
	
	_document: null,
	
	constructor:function(colName,sheet,colId,width,property,repeatednum){
		this._id = colId;
		this.index = colName;
		this._parent = sheet;
		this._document = sheet._parent;
		if(undefined != property)
			this.property = property;
		this.count = 0;
		this._dependencedList = null;
		if(undefined != repeatednum)
			this.repeatednum = repeatednum;
		if(undefined != width)
			this.width = width;
	},
	
	/*
	 * construct column json for event, no cells in it
	 * null : if this column is empty
	 */
	toEventJson: function()
	{
	    var col = {};
	    var flag = 0;
	   if(this.width!= null)
	   {
	       col.width = this.width;
	       flag++;
	   }
	   if(this.property!= null)
	   {
	       col.property = this.property;
	       flag++;
	   }
	   if(this.repeatednum != null)
	   {
	       col.repeatednum = this.repeatednum;
	       flag++;
	   }
	   if( this.styleCode != null )
	   {
			var strStyleCode = JSON.stringify(this.styleCode);
			var newStyleCodeJson = JSON.parse(strStyleCode);
			col.style = newStyleCodeJson;
			flag++;
	   }
	   if(flag > 0 ) return col;
	   else return null;
	},
	
	toEventJsonWithCells: function()
	{
		var colJson = this.toEventJson();
		colJson = (null == colJson)? new Object(): colJson;
		var cells = {};;
		var sheetModel = this._parent;
		for(var rowIndex in sheetModel.rows)
		{
			var rowModel = sheetModel.rows[rowIndex];
			var repeateNum = (rowModel.repeatednum == null || rowModel.repeatednum == undefined)? 0 : parseInt(rowModel.repeatednum);
			var cellModel = null;
			cellModel = rowModel.getRepeatedCell(this.index);
			if(null == cellModel || undefined == cellModel)
			{
				cellModel = rowModel.getCell(this.index);
			}
			if(null == cellModel || undefined == cellModel)
			{
				continue;
			}
			var cellJson = cellModel.toEventJson();
			if(!cellJson)
			{
				continue;
			}
			cellJson.repeatednum = undefined;
			rowIndex = parseInt(rowIndex);
			for(var index = rowIndex; index <= sheetModel.count && index <= rowIndex + repeateNum; index++)
			{
				cells[index] = cellJson;
			}
		}
		colJson.cells = cells;
		return colJson;
	},
	getIndex:function(){return this.index;},
	
	
	addDependenceList:function(cell){
		var isLoad = false;
		dojo.forEach(this.getDependencedList(),function(item){
			if(item == cell){
				isLoad = true;
				return;
			}
		});
		if(!isLoad){
			this.getDependencedList().push(cell);
		}
	},
	
	getDependencedList:function(){
		if(!this._dependencedList){
			this._dependencedList = new Array();
		}
		return this._dependencedList;	
	},
	
	
	/* call the dependence cell to update */
	dependenceAutoUpdate: function(){
		dojo.forEach(this.getDependencedList(),function(cell){
			cell.referenceAutoUpdate(this.index,1);
		},this); 
	},
	
	getProperty:function(){
		return this.property;
	},
	
	setProperty:function(property){
		this.property = property;
	},
	
	
	getWidth:function(){
		return this.width;
	},
	
	setWidth:function(width){
		this.width = width;
	},
	
	getStyle:function(){
		return this.styleCode;
	},
	getParent:function(){
		return this._parent;
	},
	setStyle:function(newStyleCode){
		if (this.styleCode)
			delete this.styleCode;
		if (newStyleCode!=null){
			this.styleCode = new websheet.model.StyleCode("FROMOBJ", newStyleCode);
			this.getParent().addStyleColumn(this);
		}else{ this.styleCode = null;this.getParent().deleteStyleColumn(this);}
	},
	
	changeStyle: function(styleChange) {
		if (styleChange == null)
		{	
			return;
		}
		websheet.RepeatHelper.splitColumn(this.getParent(),this,this.getIndex());
		if (this.styleCode == null)
		{
			this.styleCode = new websheet.model.StyleCode("FROMOBJ", styleChange);
			this.styleCode.changeStyle(styleChange);
			this.getParent().addStyleColumn(this);
		}
		else
		{
			this.styleCode.changeStyle(styleChange);
		}
		if (this.styleCode.checkDefault())
		{
			delete this.styleCode;
			this.styleCode = null;
			this.getParent().deleteStyleColumn(this);
		}
	},
	copy2Next:function(columnModel){
		if(!columnModel){return;}
		if(this.repeatednum > 1){
			columnModel.repeatednum = this.repeatednum;
		}else{
			delete this.repeatednum;
		}
		columnModel.setStyle(this.getStyle());
	},
	getCellsValueInDependencyList: function()
	{
		var arraylist = this.getDependencedList();
		var cells = {};
		for(var index in arraylist)
		{
			var cell = arraylist[index];
			var cellObj = {};
			var cellRef = cell.getCellRef();
			cellObj.value = cell._rawValue;
			cellObj.showvalue = cell._showValue;
			cells[cellRef] = cellObj; 
		}
		return cells;
	},
	deleteDependence:function(cell){
		var dependenceList = this.getDependencedList(); 
		if(dependenceList.length != 0){
			dojo.every(dependenceList,function(item,index){
				if(item == cell){
					dependenceList.splice(index,1);
					return;
				}
			},this);
		}	
	},
	
	reGenerateFormula:function(deleteType,notSend){
		dojo.forEach(this.getDependencedList(),function(item){item.reGenerateFormula(deleteType,notSend)});
	},
	
	getCellStyleCodeInColumn: function()
	{
		var cells = {};
		var sheet = this.getParent();
		var curColIndex = this.getIndex();
		var flag = false;
		var styleRows = sheet.getStyleRows();
		for(var rowIndex in styleRows)
		{
			var rowModel = styleRows[rowIndex];
			var cell = rowModel.getRepeatedCell(curColIndex);
			if(cell)
			{
//				var cellRef = cell.getSheetName() + "." + curColIndex + rowIndex;
				cells[rowIndex] =  new websheet.model.StyleCode("FROMOBJ",cell.styleCode);
				flag = true;
			}
		}
		if(flag == false)
		{
			return null;
		}
		else
		{
			return cells;
		}
	},
	setColumnStyle:function(style)
	{
		this.changeStyle(style);
		var sheet = this.getParent();
		var curRowIndex = -1;
		var curColIndex = this.getIndex();
		for(var attr in sheet.rows)
		{
			var row = sheet.rows[attr];
			var cell = row.getRepeatedCell(curColIndex);
			if(cell)
			{
				row.getCell(curColIndex,true).changeStyle(style,true);
			}
		}
	},
	is2ColCanMerge: function(pCol, lCol)
	{
		if(null == pCol || null == lCol)
		{
			return false;
		}
		var pColIndex = pCol.index;
		var pRepeateNum = (pCol.repeatednum == null || pCol.repeatednum == undefined) ? 0 : parseInt(pCol.repeatednum);
		var lColIndex = lCol.index;
		if(pColIndex + pRepeateNum + 1 != lColIndex)
		{
			return false;
		}
		if(pCol.width == undefined)
		{
			pCol.width = null;
		}
		if(lCol.width == undefined)
		{
			lCol.width = null;
		}
		if(pCol.width != lCol.width)
		{
			return false;
		}
		return pCol.getStyle().compareWithme(lCol.getStyle()) ;
	}
});