dojo.provide('websheet.functions.Complete');

dojo.require('websheet.functions.XPath');
dojo.require('websheet.parse.tokenType');
//dojo.require('dijit.form.Button');
dojo.require('websheet.ColumnHelper');
dojo.require('websheet.model.RangeModel');
dojo.require("websheet.parse.parseHelper");

websheet.functions.Complete = {

	/**
	 * Holds the iWidget iContext.io object, if available
	 * if not available falls back to native with some restrictions (no external websites, FF only)
	 *
	 * @param {Object} name
	 */
	io: null,
	EN_TURE: true,
	EN_FALSE: false,
	
	ERRORCODE:
	{	
		501: "#Err501!", //Invalid character
		502: "#Err502!", //Invalid argument
		503: "#Err503!", //Invalid floating point operation
		504: "#NUM!", //Parameter list error",
		508: "#Err508!", //Error: Pair missing",
		509: "#Err509!", //Missing operator
		510: "#Err510!", //Missing variable
		511: "#Err511!", //Missing variable
		512: "#Err512!", //Formula overflow
		513: "#Err513!", //String overflow
		514: "#Err514!", //Internal overflow
		516: "#Err516!", //Internal syntax error
		517: "#Err517!", //Internal syntax error
		518: "#Err518!", //Internal syntax error
		519: "#VALUE!", //No result (#VALUE is in the cell rather than Err:519!)
		520: "#Err520!", //Internal syntax error
		521: "#Err521!", //Internal syntax error
		522: "Circular reference!",
		523: "#Err523!", //The calculation procedure does not converge
		524:  "#REF!", //invalid references (instead of Err:524 cell contains)
		525: "#NAME?", //invalid names (instead of Err:525 cell contains #NAME?)
		526: "#Err526!", //Internal syntax error
		527: "#Err527!", //Internal overflow
		532: "#DIV/0!" // Division by zero"
	},
	/* changed by Rui SuYing */
	/* error type
	 * 	100: unsupported formula, e.message will be formula name
	 *  ...
	 */
	__get: function(name) {
		var method = name.toUpperCase();
		if (!this[method]) {
			e = {};
			e.errorType = 100;
			e.message = "" + method;
			throw ( e );
		}
		return dojo.hitch(this, this[method]);
	},
	
	
	
	
	/**
	 * counts a color based on a percentage and a given starting and ending color
	 * 
	 * @param int percentage a percentage to use. Must be an integer between 0 and 100.
	 * @param string from	a base RGB color 'R' for red, 'G' for green and 'B' for blue
	 * @param string to	a base RGB color 'R' for red, 'G' for green and 'B' for blue, should not be the same as from color
	 * 
	 * @return array an array holding the new color: array(R,G,B);
	 */
	_getColor: function(percentage,from,to) {
		from = from || 'R';
		to = to || 'G';
		
		//~ make sure they are given uppercase
		from 	= from.toUpperCase();
		to 	= to.toUpperCase();
		
		//~ form to percentage
		if (percentage > 1) {
			percentage /= 100;
		}
		
		//~ initialize array with black
		var temp = {};
		temp.R = 0;
		temp.G = 0;
		temp.B = 0;
		
		if(percentage < 0) {
			//~ percentage is smaller than 0 - return black
			return temp;
		}
		
		//~ maximum RGB value
		var max = 255;
		
		//~	R: FF 00 00
		//~	Y: FF FF 00
		//~	G: 00 FF 00
		//~	255,0,0 -> 255,128,0 -> 255,255,0 -> 128,255,0 -> 0,255,0
		var picentage = Math.round(max*Math.sin(percentage*Math.PI));
		temp[from] = picentage;
		temp[to] = picentage;
			
		if(percentage < 1/3) {
			temp[from] = max;
		} else if(percentage > 2/3) {
			temp[to] = max;	
		}
		
		return temp;
	},
	
	__booleanize: function(what) {
		if (what === 'no' || what === 'false' || what === '0' || (isNaN(what) === false && what == 0)) {
			return false;
		}
		return new Boolean(what) == true;
	},
	
	UI_BUTTON: function(parts) {
		if(!parts || !parts[0]) {
			throw new Exception("Missing caption parameter");
		}
		var caption = parts[0];
		var onclick = parts[1] || '';
		var disabled = this.__booleanize(parts[2]);
		
		
		if(disabled) {
			disabled = 'disabled="disabled"';
		} else {
			disabled = '';
		}
		
		return {
			toString: function() {
				return '<button '+disabled+' onclick="'+onclick+'">'+caption+'</button>';
			}
		}
//		return new dijit.form.Button({
//			label: 		caption,
//			onclick: 	function() {
//				dojo.eval(onclick);
//			},
//			disabled: disabled
//		});
		
	},
	
	__percentaganize: function(percentage,min, max) {
		min = min || 0;
		max = max || 100;
		
		percentage = parseFloat(percentage);
		
		if(percentage < 1) {
			percentage *= 100;
		}
		percentage = Math.min(Math.max(min,percentage),max);
		return percentage;
	},
	
	UI_PROGRESS: function(percentage) {
		percentage = this.__percentaganize(percentage);
		var color = this._getColor(percentage);
		return {
			toString: function() {
				return '<div class="progress-container">' +
				'<div style="width: ' +
				percentage +
				'%; background-color: rgb('+color.R+','+color.G+','+color.B+');"></div>' +
				'</div>';
			}
		}
	},
	
	IF: function(values) {
		if(values == undefined || values == null){throw "Err:511";}
		if(this.MustHaveParamCount(values.length,3,3)){
			var criteria = websheet.ColumnHelper.analyzeResult(values[0]);
			var condition = false;
			if(this.isRangStore(criteria)){
				criteria = criteria.getCols()[0];
			}
			condition = criteria;
			var result;
			if(condition){
				result =  websheet.ColumnHelper.analyzeResult(values[1]);
			}else{
				result =  websheet.ColumnHelper.analyzeResult(values[2]);
			}
			if(this.isRangStore(result)){
				if(result.getCols().length > 1){throw "#VALUE!";}
				result = result.getCols()[0];
			}
			return result;
		}
	},
	MATH: function(type, left, right) {
		left = this.fetchScalaResult(left);
		right = this.fetchScalaResult(right);
		left = parseFloat(this.Convert2Num(left));
		right = parseFloat(this.Convert2Num(right));
		
		switch (type) {
			case '+':
				return (left + right);
			case '-':
				return (left - right);
			case '*':
				return (left * right);
			case '/':
				return (right !== 0.0) ? (left / right) : '###Division by zero###';
			case '=':
				return left == right;
			case '>':
				return left > right;
			case '>=':
				return left >= right;
			case '<':
				return left < right;
			case '<=':
				return left <= right;
			case '<>':
			case '!=':
				return left != right;
			case '&&':
				return left && right;
			case '||':
				return left || right;
			case '!':
				return !left;
			case '^':
				return Math.pow(left,right);
			case '+=':
				return left+=right;
			case '-=':
				return left-=right;
			case '*=':
				return left*=right;
			case '/=':
				return left/=right;
			case '%=':
				return left%=right;
			case '%':
				return left%right;
			default:
				return NaN;
		}
	},
	/**
	 *
	 * @param {Array} parts
	 */
	CONCAT: function(parts) {
		var ret = "";
		for (var part in parts) {
			ret += parts[part];
		}
		return ret;
	},
	
//	ROUND: function(value) {
//		return Math.round(value);
//	},
	
	/**
	 *
	 * @param {Number} value
	 */
	A: function(value) {
		var numOfAteams = parseInt(value, 10);
		if (numOfAteams <= 0) {
			return "No A-Team in da house :-(";
		} else if (numOfAteams > 1) {
			return numOfAteams + " A-Teams in da house yaa!";
		} else {
			return "A-Team in da house yaa!";
		}
	},
	
	/**
	 *
	 * @param {String} url
	 * @param {String} xpath
	 */
	INGEST: function(url, xpath) {
		try {
			var ret = [];
			
			if (this.io) {
				// we got the MM enabler io tools :-D
				url = this.io.rewriteURI(url);
			}
			
			var resp = dojo.xhrGet({
				url: url,
				handleAs: "xml",
				sync: true,
				preventCache: !!this.io, // may be removed if/when proxy is configured correctly to allow caching headers
				timeout: 5000,
				load: function(response, ioArgs) {
				
				},
				error: function(response, ioArgs) {
				
				}
			});
			if (resp.ioArgs.xhr.status == 200) {
			
				var tmpArr = websheet.functions.XPath.evaluate(xpath, resp.ioArgs.xhr.responseXML);
				for (var x in tmpArr) {
					ret.push(tmpArr[x].textContent);
				}
				
			} else {
				return '###' + resp.ioArgs.xhr.statusText + '###';
			}
			
			return ret.join(';');
		} 
		catch (e) {
			return '###' + e.message + '###';
		}
	},
	
/************************************************************************************************************
 *																											*
 *	FORMULA FUNCTION																									    *
 ***********************************************************************************************************/
	
	/**
	 *
	 * @param {Array} values
	 */
	AVERAGE: function(values) {
		if(values == undefined || values == null){throw "Err:511";}
		if(this.MustHaveParamCount(values.length,1,32)){
			var obj = {};
			obj.count = 0;
			obj.sum = 0;
			this.IterateInArray(values[0], obj, dojo.hitch(this,"avgOperate"));
			return obj.sum / obj.count;
		}
	},
	
	TRUE: function(values)
	{
		if(values == undefined || values == null)
		{
			return this.EN_TURE;
		}
		else
		{
			throw this.ERRORCODE["508"];//;"#Err508";
		}
	},
	
	FALSE: function(values)
	{
		if(values == undefined || values == null)
		{
			return this.EN_FALSE;
		}
		else
		{
			throw this.ERRORCODE["508"];//"Err:508";
		}
	},
	
	NA: function(values)
	{
		if(values == undefined || values == null)
		{
			return "#N/A";
		}
		else
		{
			throw this.ERRORCODE["508"];//return "#Err508" ;
		}
	},
	/*
	 * RATE(Nper;Pmt;Pv;Fv;Type;Guess)
	 */
/*	RATE: function(values)
	{
		if(values == undefined || values == null){throw "Err:511";}
		var fPv = 0;
		var fPayment = 0;
		var fGuess = 0.1;
		var bValid = true;
		var nParamCount = values.length;
		if(!this.MustHaveParamCount(nParamCount,3,6))
		{
			throw "Err:511";
		}
		if( nParamCount == 6 )
		{
			if(!this.isCellStore(values[5]))
			{
				throw "Err:511";	
			}
		}
		if( nParamCount >= 5)
		{
			if(!this.isCellStore(values[4]))
			{
				throw "Err:511";	
			}
		}
		if( nParamCount >= 4)
		{
			if(!this.isCellStore(values[3]))
			{
				throw "Err:511";	
			}
		}
		for(var index = 0 ; index <nParamCount ; index++)
		{
			var item = values[index];
		}
	},*/
	
	/**
	 *
	 * @param {Array} values
	 * @param {Boolean} isArrayFormula	judge whether is a array formula
	 */
	SUM: function(values,isArrayFormula) {
		if(values == undefined || values == null){return 0;}
		if(this.MustHaveParamCount(values.length,0,32)){
			return this.GetSumValueInArray(values,isArrayFormula);
		}
	},
	PRODUCT:function(values,isArrayFormular){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,30)){
			var total = 1;
			return this.IterateInArray(values,total,dojo.hitch(this,"ProductOperate"),isArrayFormular);
		}
	},
	/**
	 *
	 * @param {Array} values
	 */
	SUMIF: function(values) {
		if(values == undefined || values == null){throw "Err:511";}
		if(this.MustHaveParamCount(values.length,3,3)){
			if(!this.isRangStore(values[0]) || !this.isRangStore(values[2])){
				throw "Err:511";
			}
			var satisfyCriteria = new Array();
			var sumSatisfy = new Array();
			var criteria = websheet.ColumnHelper.analyzeResult(values[1]);
			if(this.isRangStore(criteria)){
				criteria = criteria.getRows()[0];
			}
			satisfyCriteria.push(criteria);
			var compareOperate;
			if(this.isEquation(criteria.charAt(0))){
				compareOperate = dojo.hitch(this,"CompareOperate1");
			}else{
				compareOperate = dojo.hitch(this,"CompareOperate2");
			}
			this.IterateInArray(values[0], satisfyCriteria, compareOperate , false,true);
			var compareValue = websheet.ColumnHelper.analyzeResult(values[2]);
			dojo.forEach(satisfyCriteria,function(item,index){
				if(index != 0 && item.row < compareValue.getRows().length && item.col < compareValue.getRows()[item.row].length){
					sumSatisfy.push(compareValue.getRows()[item.row][item.col]);
				}
			});
			return this.GetSumValueInArray(sumSatisfy, false);
		}
	},
	
	/**
	 *
	 * @param {Array} values
	 */
	AND: function(values) {
		if(values == undefined || values == null){throw "Err:511";}
		if(this.MustHaveParamCount(values.length,1,30)){
			var result = true;
			var andOperate = dojo.hitch(this,"andOperate");
			return this.IterateInArray(values, result, andOperate ,false,true);
		}
	},
	andOperate:function(oriObj,curObj){
		if(!oriObj){return oriObj;}
		var ret = true;
		if(!this.isNumberOperator(ret,curObj)){throw this.ERRORCODE["519"];}
		if(this.Convert2Num(curObj) < 1){
			return false;
		}
		return true;
	},
	/**
	 *
	 * @param {Array} values	values[0]:row  values[1]:column values[2]:absolute values[3]:sheet name   
	 */
	ADDRESS:function(values){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,2,4)){
			var address = "";
//			var sheetName = websheet.ColumnHelper.analyzeResult(values[3]);
//			var type = websheet.ColumnHelper.analyzeResult(values[2]);
//			var column = websheet.ColumnHelper.analyzeResult(values[1]);
//			var row = websheet.ColumnHelper.analyzeResult(values[0]);
			var sheetName = this.fetchScalaResult(values[3]);
			var type = this.fetchScalaResult(values[2]);
			var column = this.fetchScalaResult(values[1]);
			var row = this.fetchScalaResult(values[0]);
			sheetName = this.Convert2Num4NOT(sheetName);
			type = parseInt(this.Convert2Num4NOT(type));
			column = parseInt(this.Convert2Num4NOT(column));
			row = parseInt(this.Convert2Num4NOT(row));
			if(( column > 1024) || (column  < 1)
				|| ( row > 65536) || ( row < 1))
				throw this.ERRORCODE["502"];
			//sheet name
			if(values.length == 4){
				address += sheetName + ".";
			}
			//if column absolute
			if(type == 1 || type == 3 ){
				address += "$";
			}
			address += websheet.ColumnHelper.translateFromColNum(column);
			//if row absolute
			if(type == 1 || type == 2 ){
				address += "$";
			}
			address += row;
			return address;
		}
	},
	
	/**
	 * the number value is transformed to date base on this date 12/31/1899. 
	 */
	MONTH:function(values){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length, 1, 1)){
			var milio = websheet.ColumnHelper.analyzeResult(values[0]);
			if(this.isRangStore(milio)){
				milio = milio.getCols()[0];
			}
			if(dojo.trim(milio) == ""){return "12"};
			var interval = Date.parse(new Date("12/31/1899"));
			if(websheet.Helper.isNumeric(milio)){
				return new Date(interval + (86400000 * this.Convert2Num(milio))).getMonth() + 1;
			}else{
				var dateObj = new Date(milio.split(" ")[0].replace(new RegExp("\-","ig"),"/"));
				return new Date(Date.parse(dateObj) + interval).getMonth() + 1;
			}
		}
	},
	
	/**
	 * get the current time.
	 */
	NOW:function(){ 
		var curDate = new Date();
		return curDate.getFullYear() + "-" + (parseInt(curDate.getMonth()) + 1) + "-" + curDate.getDate() + " " + curDate.getHours() + ":" + curDate.getMinutes();
	},
	/**
	 *
	 * @param {Array} values
	 */
	VLOOKUP: function(values) {
		return this.CALCULATELOOKUP(values,false);
	},
	
	/**
	 *
	 * @param {Array} values
	 */
	HLOOKUP: function(values) {
		return this.CALCULATELOOKUP(values,true);
	},
	
	/**
	 *
	 * @param {Array} values
	 * @param {Boolean} isHLOOKUP 
	 */
	CALCULATELOOKUP: function(values,isHLOOKUP) {
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		var vParamCount = values.length;
		if(! this.isRangStore(websheet.ColumnHelper.analyzeResult(values[1]))){
			throw this.ERRORCODE["511"];
		}
		if(this.MustHaveParamCount(vParamCount, 3, 4)){
			//determine whether the array sort by ascend.
			var isSorted = false;
			if(vParamCount == 4){
				isSorted = this.Convert2Num(this.fetchScalaResult(values[3]))?true:false;
			}
			var criterion = this.fetchScalaResult(values[0]);
			var pos = this.fetchScalaResult(values[2]);
			var returnValue = 0;
			var rows = websheet.ColumnHelper.analyzeResult(values[1]).getRows();
			var cols = websheet.ColumnHelper.analyzeResult(values[1]).getCols();
			if(isSorted){
//				dojo.forEach(rows,function(item,index){
//					rows[index] = item.sort();
//				});
//				dojo.forEach(cols,function(item,index){
//					cols[index] = item.sort();
//				});
			}
			if(isHLOOKUP){
				if(pos > cols[0].length){
					throw this.ERRORCODE["502"];
				}
				dojo.forEach(cols,function(item){
					if(item[0].search(criterion) != -1){
						returnValue = item[pos-1];
					}
				});
			}else{
				if(pos > rows[0].length){
					throw this.ERRORCODE["502"];
				}
				dojo.forEach(rows,function(item){
					if(item[0].search(criterion)!= -1){
						returnValue = item[pos-1];
					}
				});
			}
			return returnValue;
		}
		
	},
	ISNUMBER:function(values){
		var ret=false;
		if(this.MustHaveParamCount(values.length,1,1)){
			ret=this.IterateInArray(values,ret,dojo.hitch(this,"isNumberOperator"),null,true);
		}else{
			throw "illegal ref";
		}
		return ret;
	},
	ISBLANK:function(values){
		var ret=false;
		if(this.MustHaveParamCount(values.length,1,1)){
			ret=this.IterateInArray(values,ret,dojo.hitch(this,"isBlankOperator"),null,true);
		}else{
			throw "illegal ref";
		}
		return ret;
	},
	ISREF:function(values){
		
		var ret=false;
		if(this.MustHaveParamCount(values.length,1,1)){
			//ret=this.IterateInArray(values,ret,dojo.hitch(this,"ifREFOperator"));
			if(this.isRangStore(values[0])){
				ret=true;
			}
		}else{
			throw "illegal ref";
		}
		return ret;
	},
	NOT:function(values){
		var ret=false;
		if(this.MustHaveParamCount(values.length,1,1)){
			ret=this.IterateInArray(values,ret,dojo.hitch(this,"notOperator"));
		}else{
			throw "illegal ref";
		}
		return ret;
	},
	OR:function(values){
		var ret=false;
		if(this.MustHaveParamCount(values.length,1,32)){
			ret=this.IterateInArray(values,ret,dojo.hitch(this,"orOpeartor"));
		}else{
			throw "illegal ref";
		}
		return ret;
	},
	COUNTBLANK:function(values){
		var ret=0;
		if(this.MustHaveParamCount(values.length,1,1)){
			ret=this.IterateInArray(values,ret,dojo.hitch(this,"countBlankOperator"),null,true);
		}else{
			throw "illegal ref";
		}
		return ret;
	},
	CEILING:function(values,args){
		var ret=0.0;
		if(this.MustHaveParamCount(values.length,2,3)){
			var temparray=new Array();
			temparray.push(values[0]);
			var rawValue=0.00;
			rawValue=this.Convert2Num(this.IterateInArray(temparray,rawValue,dojo.hitch(this,"ceilingOperator")));
			temparray.pop();
			var step=0.0;
			temparray.push(values[1]);
			step=this.Convert2Num(this.IterateInArray(temparray,step,dojo.hitch(this,"ceilingOperator")));
			
			var mode=null;
			if(values.length==3){
				mode=0.0;
				temparray.pop();
				temparray.push(values[2]);
				mode=this.Convert2Num(this.IterateInArray(temparray,mode,dojo.hitch(this,"ceilingOperator")));
			}
			if(rawValue*step==0){
				return 0;
			}
			if(rawValue*step>0){
				var count=0.00;
				if(step<0&&(mode==null||mode==0)){
					step=0-step;
					count=count-Math.ceil(rawValue/step);
					step=0-step;
					
				}else{
					count=Math.ceil(rawValue/step);
				}
				ret=step*count;
			}else{
				throw "Err:502";
			}
		}else{
			throw "Err:502";
		}
		return ret;
	},
	COUNTIF:function(values){
		if(this.MustHaveParamCount(values.length,2,2)){
			if(!this.isRangStore(websheet.ColumnHelper.analyzeResult(values[0])) ){
				throw "Err:511";
			}
			var satisfyCriteria = new Array();
			var sumSatisfy = new Array();
			var criteria = websheet.ColumnHelper.analyzeResult(values[1]);
			satisfyCriteria.push(criteria);
			var compareOperate;
			if(this.isEquation(criteria.charAt(0))){
				compareOperate = dojo.hitch(this,"CompareOperate1");
			}else{
				compareOperate = dojo.hitch(this,"CompareOperate2");
			}
			this.IterateInArray(values[0], satisfyCriteria, compareOperate , false);
			return satisfyCriteria.length-1;
		}
	},
	FLOOR:function(values){
		var ret=0.0;
		var backStep=0;
		if(this.MustHaveParamCount(values.length,2,3)){
			var temparray=new Array();
			temparray.push(values[0]);
			var rawValue=0.00;
			rawValue=this.Convert2Num(this.IterateInArray(temparray,rawValue,dojo.hitch(this,"ceilingOperator")));
			temparray.pop();
			var step=0.0;
			temparray.push(values[1]);
			step=this.Convert2Num(this.IterateInArray(temparray,step,dojo.hitch(this,"ceilingOperator")));
			
			if(rawValue*step==0){
				return 0;
			}
			if(step<0&&values[2]!=null){
			temparray.pop();
			temparray.push(values[2]);
			var mode=0;
			mode=this.Convert2Num(this.IterateInArray(temparray,mode,dojo.hitch(this,"ceilingOperator")));
			if(mode!=0){
				backStep=0-step;
				}
			}
			if(rawValue*step>0){
				var count=0.00;
				if(step>0){
					step=0-step;
				}
				count=Math.ceil(rawValue/step);
				ret=step*count;
			}else{
				throw "Err: 502";
			}
		}else{
			throw "Err: 502";
		}
		
		return ret+backStep;
	},
	RANDBETWEEN:function(values){
		var ret=0.0;
		if(this.MustHaveParamCount(values.length,2,2)){
			var temparray=new Array();
			temparray.push(values[0]);
			var button=0.00;
			button=this.Convert2Num(this.IterateInArray(temparray,button,dojo.hitch(this,"ceilingOperator")));
			temparray.pop();
			var top=0.0;
			temparray.push(values[1]);
			top=this.Convert2Num(this.IterateInArray(temparray,top,dojo.hitch(this,"ceilingOperator")));
			button=Math.ceil(button);
			if(top<button){
				throw "Err 520"
			}
			
			top=Math.ceil(top);
			ret=(Math.random())*(top-button)+button;
			ret=Math.ceil(ret);
		}
		return ret;
	},
	COUNT:function(values){
		var ret=0;
		if(this.MustHaveParamCount(values.length,1,32)){
			ret=this.IterateInArray(values,ret,dojo.hitch(this,"countOperator"));
		}else{
			throw "illegal ref";
		}
		return ret;
	},
	MAX:function(values){
		var ret=Number.MIN_VALUE;
		if(this.MustHaveParamCount(values.length,1,32)){
			ret=this.IterateInArray(values,ret,dojo.hitch(this,"maxOperator"));
		}
		if(ret==Number.MIN_VALUE){
			ret=0;
		}
		return ret;
	},
	MIN:function(values){
		var ret=Number.MAX_VALUE;
		if(this.MustHaveParamCount(values.length,1,32)){
			ret=this.IterateInArray(values,ret,dojo.hitch(this,"minOperator"));
		}
		if(ret==Number.MAX_VALUE){
			ret=0;
		}
		return ret;
	},
	EXACT:function(values){
		var ret=true;
		var array1=new Array();
		var array2=new Array();
		var temp=new Array();
		if(this.MustHaveParamCount(values.length,2,2)){
			temp.push(values[0]);
			array1=this.IterateInArray(temp,array1,dojo.hitch(this,"exactOperator"));
			temp.pop();
			temp.push(values[1]);
			array2=this.IterateInArray(temp,array2,dojo.hitch(this,"exactOperator"));
			if(array1.length==array2.length){
				for(var i=0;i<array1.length;i++){
					if(array1[i]!=array2[i]){
						ret=false;
						break;
					}
				}
			}else{
				throw "Err 520";
			}
		}
		return ret;
	},
	CONCATENATE:function(values){
		if(this.MustHaveParamCount(values.length,1,32)){
			var concateStr = "";
			concateStr = this.IterateInArray(values, concateStr, dojo.hitch(this,"concateNameOperator"));
			return concateStr;
		}
	},
	ABS:function(values){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var numValue = this.fetchScalaResult(values[0]);
			numValue = this.Convert2Num4NOT(numValue);
			var ret = false;
			if(this.isNumberOperator(ret,numValue)){
				if(parseFloat(numValue) >= 0){
					return numValue;
				}else{
					return -numValue;
				}
			}else{
				throw this.ERRORCODE["519"];
			}
		}
	},
	EVEN:function(values){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var numValue = this.fetchScalaResult(values[0]);
			numValue = this.Convert2Num4NOT(numValue);
			var ret = false;
			if(this.isNumberOperator(ret,numValue)){
				if(numValue == 0){return 0;}
				var isNegative = false;
				if(numValue < 0){
					isNegative = true;
					numValue =  -numValue;
				}
				var intValue =  parseInt(numValue);
				if(intValue % 2 == 0){
					if(numValue > intValue){ 
						intValue = intValue + 2;
					}else{
						intValue = numValue;
					}
				}else{
					intValue = intValue + 1;
				}
				if(isNegative){
					return -intValue;
				}
				return intValue;
			}else{
				throw this.ERRORCODE["519"];
			}
		}
	},
	ROUND:function(values){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		var len = values.length;
		if(this.MustHaveParamCount(len,1,2)){
			var count = 0;
			var roundValue = this.fetchScalaResult(values[0]);
			roundValue = this.Convert2Num4NOT(roundValue);
			var ret = false;
			if(!this.isNumberOperator(ret,roundValue)){
				throw this.ERRORCODE["519"];
			}
			if(len == 2){
				count = this.fetchScalaResult(values[1]);
				count = this.Convert2Num(count);
				if(!this.isNumberOperator(ret,count)){
					throw this.ERRORCODE["519"];
				}
				count = parseInt(count);
			}
			if(count == 0){return Math.round(roundValue);}
			if(count > 0){
				roundValue = new String(roundValue);
				var dotIndex = roundValue.indexOf(".");
				if(dotIndex != -1){
					var len = roundValue.length;
					var decimalCount = len - dotIndex - 1;
					if(count > decimalCount){
						return roundValue;
					}else{
						var temp = count;
						while(temp > 0){
							roundValue *= 10;
							temp--;
						}
						roundValue = Math.round(roundValue);
						while(count > 0){
							roundValue /= 10;
							count--;
						}
						return roundValue;
					}
				}
			}else{
				count = -count;
				roundValue = new String(roundValue);
				var intFraction = roundValue.indexOf(".");
				if(count >= intFraction){
					return 0;
				}else{
					var temp = count;
					while(temp > 0){
						roundValue /= 10;
						temp--;
					}
					roundValue = Math.round(roundValue);
					while(count > 0){
						roundValue *= 10;
						count--;
					}
				}
				return roundValue;
			}
			return roundValue;
		}
	},
	FACT:function(values){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var ret = false;
			var value = 1;
			var fact = this.fetchScalaResult(values[0]);
			fact = this.Convert2Num4NOT(fact);
			if(!this.isNumberOperator(ret,fact)){
				throw this.ERRORCODE["519"];
			}
			fact = parseInt(fact);
			if(fact > 170){throw this.ERRORCODE["519"];}
			if(fact == 0){return 1;}
			if(fact <0){throw this.ERRORCODE["502"];}
			while(fact > 1){
				value *= fact;
				fact--;
			}
			return value;
		}
	},
	TRUNC:function(values){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		var len = values.length;
		if(this.MustHaveParamCount(len,1,2)){
			var ret = false;
			var value = 1;
			var decimalCount = 0;
			var fact = this.fetchScalaResult(values[0]);
			fact = this.Convert2Num4NOT(fact);
			if(!this.isNumberOperator(ret,fact)){
				throw this.ERRORCODE["519"];
			}
			if(len == 2){
				decimalCount = this.fetchScalaResult(values[1]);
				decimalCount = this.Convert2Num4NOT(decimalCount);
				if(!this.isNumberOperator(ret,fact)){
					throw this.ERRORCODE["519"];
				}
				decimalCount = parseInt(decimalCount);
			}
			if(decimalCount == 0){
//				this.INT(values[0]);
				return parseInt(fact);
			}else if(decimalCount >0){
				fact = new String(fact);
				var dotIndex = fact.indexOf(".");
				if(dotIndex != -1){
					var len = fact.length;
					var fractionalCount = len - dotIndex - 1;
					if(decimalCount > fractionalCount){
						return fact;
					}else{
						return fact.substring(0, dotIndex + 1 + parseInt(decimalCount));
					}
				}
				return fact;
			}else if(decimalCount < 0){
				decimalCount = -decimalCount;
				fact = new String(fact);
				var intFraction = fact.indexOf(".");
				if(decimalCount >= intFraction){
					return 0;
				}else{
					var del = intFraction - decimalCount;
					fact = fact.substring(0, del);
					while(decimalCount > 0){
						fact += "0";
						decimalCount--;
					}
				}
				return fact;
			}
		}
	},
	
	/**
	 *
	 * @param {Array} values
	 * @param {Boolean} isArrayFormula	judge whether is a array formula
	 */
	LOWER: function(values,isArrayFormula) {
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var text = this.fetchScalaResult(values[0]);
			var lowerText = "";
			if(text)
				lowerText = text.toLocaleLowerCase();
			return lowerText;
		}
	},
	/**
	 *
	 * @param {Array} values
	 * @param {Boolean} isArrayFormula	judge whether is a array formula
	 */
	UPPER: function(values,isArrayFormula) {
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var text = this.fetchScalaResult(values[0]);
			var upperText = "";
			if(text)
				upperText = text.toLocaleUpperCase();
			return upperText;
		}
	},
	/**
	 *
	 * @param {Array} values
	 * @param {Boolean} isArrayFormula	judge whether is a array formula
	 */
	/*VALUE: function(values,isArrayFormula) {
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var text = this.fetchScalaResult(values[0]);
			if(text){
//				var numberRegEx = /^[1-9][0-9]*(\.[0-9]*)?$/;
				var numberRegExs = {};
				numberRegExs[0] = /^[+|-]?\$?([0-9]+(,[0-9]{3})*)?(\.[0-9]+)?(([eE][+-]?[0-9]+)|%)?$/;
				numberRegExs[1] = /^[+|-]?[0-9]+\ [0-9]+\/[1-9][0-9]?$/;
				//performing a trim
				text = text.toString().replace(/^(\s|&nbsp;)+|(\s|&nbsp;)+$/g,"");
				for(var i in numberRegExs){
					var expression = numberRegExs[i];
					var matcher = text.match(expression);
					if(matcher != null && (matcher[0].length == text.length)){
						var value = parseFloat(matcher);
						var parseValue = dojo.number.parse(matcher[0]);
						if( i == 0){
							
							//TODO: for Currency, locale
							//TODO: for thousand seperator, locale
							var intStr = text.split(".")[0];
							var floatStr = text.split(".")[1];
							var thousandArray = intStr.split(",");
							var digitValue = 0;
							for(var n=0; n < thousandArray.length; n++){
								if(n != 0)
									digitValue *= 1000;
								digitValue += parseFloat(thousandArray[n]);
							}
							if(floatStr){
//								if( ((scientificIndex = floatStr.indexOf("e")) > 0) || 
//									((scientificIndex = floatStr.indexOf("E") )> 0)){
//									floatStr = parseFloat(text.substring(0,scientificIndex - 1));
//								}else 
//									floatStr = "";
								var tempStr = (digitValue.toString() + "." + floatStr);
								digitValue = parseFloat(tempStr);
							}
							//for percentage
							var hasPercentage = false;
							if(text.indexOf("%") > -1){
								digitValue /= 100;
							}
//							//for scientific
//							var scientificNumber = 0;
//							var scientificIndex = -1;
//							if( ((scientificIndex = text.indexOf("e")) > -1) || 
//									((scientificIndex = text.indexOf("E") )> -1)){
//								scientificNumber = parseFloat(text.substring(scientificIndex + 1, text.length));
//								digitValue *= Math.pow(10, scientificNumber);
//							}
							return digitValue;
						}else if( i == 1){
							var fractionArray = text.split(" ");
							if(fractionArray.length == 2){
								var intValue = parseFloat(fractionArray[0]);
								var decValue = parseFloat(fractionArray[1].split("/")[0]);
								var nomValue = parseFloat(fractionArray[1].split("/")[1]);
								var bNegative = false;
								if(intValue < 0)
									bNegative = true;
								var value = Math.abs(intValue) + (decValue/nomValue);
								if(bNegative)
									value = 0 - value;
								return value;
							}
						}
					}
				}

			}
			throw this.ERRORCODE["502"];
		}
	},*/
	/**
	 *
	 * @param {Array} values
	 * @param {Boolean} isArrayFormula	judge whether is a array formula
	 */
	ISEVEN: function(values,isArrayFormula) {
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var value = this.fetchScalaResult(values[0]);
			if(typeof value == "string"){
				if(dojo.trim(value.toLowerCase()) == "true" || dojo.trim(value.toLowerCase()) == "false")
					throw this.ERRORCODE["504"];
			}
			value = this.Convert2Num4NOT(value);
			var result = value % 2;
			if(result == 0){
				return true;
			}else if(result == 1)
				return false;
		}
		throw this.ERRORCODE["502"];
	},
	ISFORMULA:function(values){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var result = websheet.ColumnHelper.analyzeResult(values[0]);
			if(this.isRangStore(result)){
				if(result.endAddress && result.startAddress != result.endAddress){
					return false;
				}else{
					var cell = result.getCells()[0].getCellVieType();
					if(cell)
						return cell.isFormula;
					else
						throw this.ERRORCODE["502"];
				}
			}else
				return false;
		}else{
			throw this.ERRORCODE["502"];
		}
	},
	/**
	 *
	 * @param {Array} values
	 * @param {Boolean} isArrayFormula	judge whether is a array formula
	 */
	SHEET: function(values,isArrayFormula) {
		var sheetName = null;
		if(values == undefined || values == null){
			//TODO: should be the sheet contains this formula, rather than current sheet
			sheetName = websheet.parse.parseHelper.getCurrentParseSheetName();
		}
		else if(this.MustHaveParamCount(values.length,1,1)){
			var result = websheet.ColumnHelper.analyzeResult(values[0]);
			if(this.isRangStore(result)){
				var address = result.startAddress;
				if(result.endAddress)
					address += ":" + result.endAddress
				var ref = websheet.Helper.parseRef(address); 
				sheetName = ref.sheet;
				if(sheetName == null)
					sheetName = websheet.parse.parseHelper.getCurrentParseSheetName();
			}else if(typeof result == "string"){
				sheetName = result;
			}
		}
		if(sheetName != null){
			var curSheet = pe.base.getDataConcentrator().getDocumentObj().getSheet(sheetName);
			if(curSheet){
				var sheetIndex = curSheet.getIndex();
				if(sheetIndex > 0)
					return sheetIndex;
			}
		}
		throw this.ERRORCODE["502"];
	},
	
	/**
	 *
	 * @param {Array} values
	 * @param {Boolean} isArrayFormula	judge whether is a array formula
	 */
	POWER: function(values,isArrayFormula) {
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,2,2)){
			var base = this.fetchScalaResult(values[0]);
			var exp = this.fetchScalaResult(values[1]);
			base = this.Convert2Num4NOT(base);
			exp = this.Convert2Num4NOT(exp);
			var result = Math.pow(base, exp);
			if(result == Number.POSITIVE_INFINITY || result == Number.NEGATIVE_INFINITY)
				throw this.ERRORCODE["504"];
			return result;
		}
		throw this.ERRORCODE["502"];
	},
	
	YEAR: function( values ) {
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var result = websheet.ColumnHelper.analyzeResult(values[0]);
				
			var realValue = this.getInnerString(this.fetchScalaResult(result));
			if ((typeof realValue == "string")
				&& ((dojo.trim(realValue.toLowerCase())=="true")||(dojo.trim(realValue.toLowerCase())=="false")))
				realValue = 0;
			else
				realValue = parseFloat(this.Convert2Num(realValue));
			
			if ((realValue>100000000)||(realValue<-10000000))
				throw this.ERRORCODE["519"];
			
			var dtTmp = new Date("12/31/1899");
    		targetDate = new Date(Date.parse(dtTmp) + (86400000 * realValue));
			
    		return targetDate.getFullYear();
		}
	},
	 /** Return the starting position of a given text
	 */
	SEARCH: function(values) {
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		this.MustHaveParamCount(values.length,2,3);
		
		// not case sensitive
		var find_text = this.fetchScalaResult(values[0]).toUpperCase();
		var text = this.fetchScalaResult(values[1]).toUpperCase();
		var start = 1;

		if (values.length == 3)
			start = parseInt(this.Convert2Num(this.fetchScalaResult(values[2])));
		if (start < 1) throw this.ERRORCODE["519"];
		if (start >= text.length) throw this.ERRORCODE["519"]; // 525 instead?
		
		if (start != 1)
			// dojo string's index starts with 0
			text = text.substr(start-1);
		var pos = -1;
		if (text != "" && find_text != "")
			pos = text.search(find_text);
		if (pos == -1) throw this.ERRORCODE["519"];
		
		return start + pos;
	},
	
	REPLACE: function(values) {
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		this.MustHaveParamCount(values.length,4,4);
		
		var text = this.fetchScalaResult(values[0]);
		var start = parseInt(this.Convert2Num4NOT(this.fetchScalaResult(values[1])));
		var len = parseInt(this.Convert2Num4NOT(this.fetchScalaResult(values[2])));
		if (start < 1 || len < 0) throw this.ERRORCODE["519"];
		var newText = this.fetchScalaResult(values[3]);

		var pre = text.substr(0, start-1);
		var post = "";
		if (start + len < text.length)
			post = text.substr(start - 1 + len);
		
		return pre + newText + post;
	},
	
	DAY: function( values ) {
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var result = websheet.ColumnHelper.analyzeResult(values[0]);

			var realValue = this.getInnerString(this.fetchScalaResult(result));
			if ((typeof realValue == "string")
				&& ((dojo.trim(realValue).toLowerCase()=="true")||(dojo.trim(realValue).toLowerCase()=="false")))
				realValue = 0;
			else
				realValue = parseFloat(this.Convert2Num(realValue));
			
			if ((realValue>100000000)||(realValue<-10000000))
				throw this.ERRORCODE["519"];
			
			var dtTmp = new Date("12/31/1899");
    		targetDate = new Date(Date.parse(dtTmp) + (86400000 * realValue));
			
    		return targetDate.getDate();
		}
	},
	
	FORMULA:function(values){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var result = websheet.ColumnHelper.analyzeResult(values[0]);
			if(this.isRangStore(result)){
				if(result.endAddress && result.startAddress != result.endAddress){
					var rows = result.getRows();
					if(rows.length == 1){
						var cells = result.getCells();
						for(var i = 0; i < cells.length; i++){
							var cell = cells[i].getCellVieType();
							if(cell.getRawValue() == "" || cell.isFormula)
								throw this.ERRORCODE["519"]; //"#VALUE"
						}
						return "#N/A";
					}else
						throw this.ERRORCODE["519"]; //"#VALUE"
				}else{
					var cell = result.getCells()[0].getCellVieType();
					if(cell.isFormula)
						return cell.getRawValue();
					else
						return "#N/A";
				}				
			}else{
				if((values[0].getTokenType() == websheet.parse.tokenType.STRING_TOKEN) ||
						(values[0].getTokenType() == websheet.parse.tokenType.INT_TOKEN))
					return "#N/A";
				else
					throw this.ERRORCODE["525"]; //"#NAME?"
			}
		}else{
			throw this.ERRORCODE["502"];
		}
	},
	ISERROR:function(values){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var result = websheet.ColumnHelper.analyzeResult(values[0]);
			if(this.isRangStore(result)){
				if(result.endAddress && result.startAddress != result.endAddress){
					var rows = result.getRows();
					if(rows.length == 1){
						var cell = result.getCells()[0].getCellVieType();
						if(cell.getRawValue() == "" || cell.isFormula)
							return true;
						else
							return false;
					}else
						return true;
				}else{
					var cell = result.getCells()[0].getCellVieType();
					if(cell)
						return cell.isError;
					else
						throw this.ERRORCODE["502"];
				}
			}else{
				if((values[0].getTokenType() == websheet.parse.tokenType.STRING_TOKEN))
					return false;
				else
					return true;
			}
		}else{
			throw this.ERRORCODE["502"];
		}
	},
	INDEX:function(values){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		var len = values.length;
		if(this.MustHaveParamCount(len,1,4)){
			var result = websheet.ColumnHelper.analyzeResult(values[0]);
			if(this.isRangStore(result)){
				if(result.endAddress && result.startAddress != result.endAddress){
					var rows = result.getRows();
					var cols = result.getCols();
					switch(len){
						case 1:	
							throw this.ERRORCODE["519"];
						break;
						case 2:{
							var rowResult = websheet.ColumnHelper.analyzeResult(values[1]);
							var rowIndex = this.Convert2Num4NOT(rowResult);
							if( 0 <= rowIndex && rowIndex < (rows.length + 1))
								throw this.ERRORCODE["519"];
							else
								throw this.ERRORCODE["502"];
						}
						break;
						case 3:{
							var rowResult = websheet.ColumnHelper.analyzeResult(values[1]);
							var rowIndex = this.Convert2Num4NOT(rowResult);
							if( 0 == rowIndex )
								throw this.ERRORCODE["519"];
							else if( rowIndex < 0 || rowIndex >= (rows.length + 1) )
								throw this.ERRORCODE["502"];
							
							var colResult = websheet.ColumnHelper.analyzeResult(values[2]);
							var colIndex = this.Convert2Num4NOT(colResult);
							if( 0 == colIndex )
								throw this.ERRORCODE["519"];
							else if( colIndex < 0 || colIndex >= (cols.length + 1))
								throw this.ERRORCODE["502"];
							
							rowIndex = parseInt(rowIndex);
							colIndex = parseInt(colIndex);
							var cellShowValue = rows[rowIndex-1][colIndex-1];
							if(cellShowValue == "")
								cellShowValue = 0;
							return cellShowValue;
						}
						break;
						case 4:{
							var rangeIndexResult = websheet.ColumnHelper.analyzeResult(values[3]);
							var rangeIndex = this.Convert2Num4NOT(rangeIndexResult);
							if(0 <= rangeIndex && rangeIndex < 1)
								throw this.ERRORCODE["502"];
							else if(rangeIndex < 0 || rangeIndex >= 2 )
								throw this.ERRORCODE["524"];
							
							var rowResult = websheet.ColumnHelper.analyzeResult(values[1]);
							var rowIndex = this.Convert2Num4NOT(rowResult);
							if( 0 == rowIndex )
								throw this.ERRORCODE["519"];
							else if( rowIndex < 0 || rowIndex >= (rows.length + 1) )
								throw this.ERRORCODE["502"];
							
							var colResult = websheet.ColumnHelper.analyzeResult(values[2]);
							var colIndex = this.Convert2Num4NOT(colResult);
							if( 0 == colIndex )
								throw this.ERRORCODE["519"];
							else if( colIndex < 0 || colIndex >= (cols.length + 1))
								throw this.ERRORCODE["502"];
							
							rowIndex = parseInt(rowIndex);
							colIndex = parseInt(colIndex);
							var cellShowValue = rows[rowIndex-1][colIndex-1];
							if(cellShowValue == "")
								cellShowValue = 0;
							return cellShowValue;
						}
						break;
					}
				}else{
					var cell = result.getCells()[0].getCellVieType();
					var actValue = cell.getShowValue();
					if(actValue == "")
						actValue = 0;
					switch(len){
						case 1:				
						break;
						case 2:{
							var rowResult = websheet.ColumnHelper.analyzeResult(values[1]);
							var rowIndex = this.Convert2Num4NOT(rowResult);
							if( rowIndex < 0 || rowIndex >= 2 )
								throw this.ERRORCODE["502"];
						}
						break;
						case 3:{
							var rowResult = websheet.ColumnHelper.analyzeResult(values[1]);
							var rowIndex = this.Convert2Num4NOT(rowResult);
							if( rowIndex < 0 || rowIndex >= 2 )
								throw this.ERRORCODE["502"];
							
							var colResult = websheet.ColumnHelper.analyzeResult(values[2]);
							var colIndex = this.Convert2Num4NOT(colResult);
							if( colIndex < 0 || colIndex >= 2 )
								throw this.ERRORCODE["502"];
						}
						break;
						case 4:{
							var rangeIndexResult = websheet.ColumnHelper.analyzeResult(values[3]);
							var rangeIndex = this.Convert2Num4NOT(rangeIndexResult);
							if(0 <= rangeIndex && rangeIndex < 1)
								throw this.ERRORCODE["502"];
							else if(rangeIndex < 0 || rangeIndex >= 2 )
								throw this.ERRORCODE["524"];
							
							var rowResult = websheet.ColumnHelper.analyzeResult(values[1]);
							var rowIndex = this.Convert2Num4NOT(rowResult);
							if( rowIndex < 0 || rowIndex >= 2 )
								throw this.ERRORCODE["502"];
							
							var colResult = websheet.ColumnHelper.analyzeResult(values[2]);
							var colIndex = this.Convert2Num4NOT(colResult);
							if( colIndex < 0 || colIndex >= 2 )
								throw this.ERRORCODE["502"];
						}
						break;
					}
					return actValue;
				}				
			}else{
				throw this.ERRORCODE["525"];
			}
		}else{
			throw this.ERRORCODE["502"];
		}
	},

	isLeapYear:function(iYear) {
		 if (iYear % 4 == 0 && iYear % 100 != 0) 
		 {
			 return true;
		 } 
		 else 
		 {
			 if (iYear % 400 == 0) 
			 {
				 return true;
			 } 
			 else 
			 {
				 return false;
			 }
		 }
	},

	/*
	 * if oldString composed of "xxxx", the return value is xxxx
	 * if oldString composed of 'xxxx', the return value is xxxx
	 * else return old string
	 */
	getInnerString: function(oldString){
		if ((oldString==null)||(oldString==undefined))
			return oldString;
		var length = oldString.length;
		if (((oldString.charAt(0)=='"')&&(oldString.charAt(length-1)=='"'))
				||((oldString.charAt(0)=='\'')&&(oldString.charAt(length-1)=='\'')))
			return oldString.slice(1, length-1);
		
		return oldString;
	},

	fetchScalaResult:function(value){
		var result = websheet.ColumnHelper.analyzeResult(value);
		if(this.isRangStore(result)){
			var cols = result.getCols();
			if(cols.length > 1){
				throw this.ERRORCODE["519"];
			}
			var column = cols[0];
			if(dojo.isArray(column)) {
				if (column.length >= 2)
					throw this.ERRORCODE["519"];
				else
					result = column[0];
			}else{
				result = column;
			}
		}
		return result
	},
	
	ROW:function(values,isArrayFormular){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var c = websheet.ColumnHelper.analyzeResult(values[0]);
			if(this.isRangStore(c)){
				c = c.getCells();
				if(!this.isInSheetsArray(c))
					throw this.ERRORCODE["502"];
				if(c.length==1){
					var rowIdx = c[0].getCellVieType().getRow();
					return rowIdx;
				}else{
					if(isArrayFormular){
						var ret = null;
						for(var i=c[0].getCellVieType().getRow();i<=c[c.length-1].getCellVieType().getRow();i++)
						{
							var comma = null;
							if(ret!=null)
							{
								comma = ',';
								ret += comma+i;
							}else
							{
								comma = '';
								ret = i;
							}
						}
						return ret;
					}else{
						return c[0].getCellVieType().getRow();
					}
				}
			}else{
				throw this.ERRORCODE["502"];
			}
		}
		
	},
	COLUMN:function(values,isArrayFormular){
		if(values == undefined || values == null){throw this.ERRORCODE["511"];}
		if(this.MustHaveParamCount(values.length,1,1)){
			var c = websheet.ColumnHelper.analyzeResult(values[0]);
			if(this.isRangStore(c)){
				c = c.getCells();
				if(!this.isInSheetsArray(c))
					throw this.ERRORCODE["502"];
				var s = websheet.ColumnHelper.translateFromColCharacters(c[0].getCellVieType().getCol());
				if(c.length==1){
					return s;
				}else{
					if(isArrayFormular){
						var ret = null;
						var e = websheet.ColumnHelper.translateFromColCharacters(c[c.length-1].getCellVieType().getCol());
						for(var i=s;i<=e;i++)
						{
							var comma = null;
							if(ret!=null)
							{
								comma = ',';
								ret += comma+i;
							}else
							{
								comma = '';
								ret = i;
							}
						}
						return ret;
					}else{
						return websheet.ColumnHelper.translateFromColCharacters(c[0].getCellVieType().getCol());
					}
				}
			}else{
				throw this.ERRORCODE["502"];
			}
		}
		
	},
	concateNameOperator:function(ret,value){
		ret=ret+value;
		return ret;
	},
	exactOperator:function(ret,oriobj){
		ret.push(oriobj);
		return ret;
	},
	minOperator:function(ret,oriobj){
		var strP=/^(\-|\+)?\d+(\.\d+)?$/; 
		if (strP.test(oriobj)) {
			if(ret>this.Convert2Num(oriobj)){
				ret=this.Convert2Num(oriobj);
			}
		}
		return ret;
	},
	maxOperator:function(ret,oriobj){
		var strP=/^(\-|\+)?\d+(\.\d+)?$/; 
		if (strP.test(oriobj)) {
			if(ret<this.Convert2Num(oriobj)){
				ret=this.Convert2Num(oriobj);
			}
		}
		return ret;
	},
	countOperator:function(ret,oriobj){
		var strP=/^(\-|\+)?\d+(\.\d+)?$/; 
		if(strP.test(oriobj)){
			ret=ret+1;
		}
		return ret;
	},
	countIfOperator:function(ret,oriobj){
		
	},
	ceilingOperator:function(ret,oriobj){
		ret=oriobj;
		return ret;
	},
	countBlankOperator:function(ret,cell,index,cellIndex,rowIndex){
		if(cell==null||cell==""){
			ret=ret+1;
		}
		return ret;
	},
	orOpeartor:function(ret,oriObj){
		if(ret){return ret;}
		if(dojo.trim(oriObj) == "true"){return true;}
		if(dojo.trim(oriObj) == "false"){return false;}
		if(!this.isNumberOperator(ret,oriObj)){throw "#VALUE!";}
		if(this.Convert2Num(oriObj)>0){
			ret = true;
		}
		return ret;
	},
	notOperator:function(ret,oriobj){
		var num=this.Convert2Num4NOT(oriobj);
		if( num<0){
			ret = false;
		}
		if(num==0){
			ret=true;
		}
		if(num>0){
			ret=false;
		}	
		return ret;
		
	},
	
	ifREFOperator:function(ret,oriobj){
		if(this.isRangStore(oriobj)){
			ret=true;
		}else{
			ret=false;
		}
		
		return ret;
	},
	isBlankOperator:function(oriObj,curObj){
		if(curObj == ""||curObj==null){
			oriObj = true;
		}else{
			oriObj = false;			
		}
		return oriObj;
	},
	isNumberOperator:function(ret,value){
		
		var strP=/^(\-|\+)?\d+(\.\d+)?$/; 
		if(strP.test(value)){
			ret=true;
		}else{
			ret=false;
		}
		return ret;
	},
	
	_countArgsOperate: function(value) {
		var count = 0;
		var result = websheet.ColumnHelper.analyzeResult(value);
		if(!this.isRangStore(result))  return ++count;
		
		// range token list
		var cols = result.getCols();
		for (var i=0; i<cols.length; ++i) {
			var col = cols[i];
					
			if (!dojo.isArray(col)) {
				if (col != "") ++count;
			} else {
				for (var j=0; j<col.length; ++j)
					if (col[j] != "") ++count;
			}
		}
		
		return count;
	},
	
	
/************************************************************************************************************
 *																											*
 *	BASE FUNCTION																									    *
 ***********************************************************************************************************/
	/**
	 *
	 * @param {Integer} vAct		current function has parameters count
	 * @param {Integer} vMust 		function must have parameters count
	 * @param {Integer} vMax 		function has the max parameters count
	 * @return {Integer} true/false
	 */
	MustHaveParamCount:function(vAct,vMust,vMax){		
		if(vAct >= vMust && vAct <= vMax){
			return true;
		}else{
			//511 Parameter list error
			throw "Err:511";
		}
	},
	
	/**
	 *
	 * @param {any} convert		the value which would be transformed to Float.
	 * @return {Float} 
	 */
	Convert2Num:function(convert,isProduct){
		if(convert == null||convert=="")return 0;
		switch (typeof convert) {
			case "string":
				convert = convert.replace(new RegExp('\"',"ig"),"");
				var ret=false;
				ret=this.isNumberOperator(ret,convert);
				if(ret){
					return convert;
				}else{
					if(convert.toLowerCase == "true"){
						return 1;
					}else{
						if(isProduct)
							return 1;
						return 0;
					}
				}
			case "number":
				return convert;
			case "boolean":
				if(convert == true){
					return 1;
				}else{
					return 0;
				}
			case "undefined":
			case "null":
				return 0;
			case "object":
				if(dojo.isArray(convert)){return dojo.hitch(this,"Convert2Num")(convert[0])}else{return 0;};
			default:
				return 0;
		}
	},
	Convert2Num4NOT:function(convert){
		if(convert == null||convert=="")return 0;
		switch (typeof convert) {
			case "string":
				convert = convert.replace(new RegExp('\"',"ig"),"");
				var ret=false;
				ret=this.isNumberOperator(ret,convert);
				if(ret){
					return convert;
				}else{
					if(dojo.trim(convert.toLowerCase()) == "true"){
						return 1;
					}else if(dojo.trim(convert.toLowerCase()) == "false"){
						return 0;
					}
					throw this.ERRORCODE["519"];
				}
			case "number":
				return convert;
			case "boolean":
				if(convert == true){
					return 1;
				}else{
					return 0;
				}
			default:
				throw this.ERRORCODE["502"];
		}
	},
	/**
	 *
	 * @param {Array} arrays		the values arrays which to be added.
	 * @param {Boolean} isArrayFormula	judge whether is a array formula.
	 * @return {Float} the total value of numbers in the arrays.
	 */
	GetSumValueInArray:function(arrays,isArrayFormula){
		var total = 0;
		return this.IterateInArray(arrays,total,dojo.hitch(this,"SumOperate"),isArrayFormula);
	},	
	SumOperate:function(oriObj,curObj){
		oriObj += parseFloat(dojo.hitch(this,"Convert2Num")(curObj));
		return oriObj;
	},
	ProductOperate:function(oriObj,curObj){
		oriObj *= parseFloat(dojo.hitch(this,"Convert2Num")(curObj,true));
		return oriObj;
	},
	avgOperate:function(obj,value){
		obj.sum += parseFloat(this.Convert2Num(value));
		obj.count += 1;
		return obj;
	},
	CompareOperate1:function(oriObj,curObj,index,colIndex,rowIndex){
		//oriObj[0] store the comparison condition.
		var comparison = false;
		if(dojo.hitch(this,"isEquation")(oriObj[0].charAt(1))){
			comparison = dojo.hitch(this,"MATH")(oriObj[0].substring(0,2),curObj,oriObj[0].substring(2));
		}else{
			comparison = dojo.hitch(this,"MATH")(oriObj[0].substring(0,1),curObj,oriObj[0].substring(1));
		}
		if(comparison){
			var pos = {row:0,col:0};
			pos.row = rowIndex;
			pos.col = colIndex;
			oriObj.push(pos);
		}
		return oriObj;
	},
	CompareOperate2:function(oriObj,curObj,index,colIndex,rowIndex){
		//oriObj[0] store the comparison condition.
		if(oriObj[0] == curObj){
			var pos = {row:0,col:0};
			pos.row = rowIndex;
			pos.col = colIndex;
			oriObj.push(pos);
		}else {
			var s1=new String(oriObj[0]);
			var s2=new String(curObj);
			var ret=s2.search(s1);
			if(s2.search(s1)!=-1){
				oriObj.push(curObj);
			}
		}
		return oriObj;
	},
	/**
	 *
	 * @param {Array} arrays		the values arrays which to be added.
	 * @param {any} cache		do the operator,then store the value in cache.
	 * @param {function} operator		do the operation when iterate the array.
	 * @param {Boolean} isArrayFormula	judge whether is a array formula.
	 * @return 
	 */
	IterateInArray:function(arrays,cache,operator,isArrayFormula,ignoreError){
		if(!dojo.isArray(arrays) && arrays.length == 1 && !this.isRangStore(arrays[0]))return arrays[0];
		var temp = arrays;
		if(this.isRangStore(arrays))
		{
			var temp = new Array();
			temp.push(arrays);
		}
		if(this.isTokenList(arrays))
		{
			temp = arrays.getTokenList();
		}
		if(!dojo.isArray(arrays))
		{
			var temp = new Array();
			temp.push(arrays);
		}
		dojo.forEach(temp,function(item,index){
			if(dojo.isArray(item)){
				cache = this.IterateInArray(item,cache,operator,isArrayFormula,ignoreError);
			}else{
				try{
					if(this.isRangStore(item)){
						//error cell check
						if(!ignoreError && !(item instanceof websheet.model.RangeModel)){
							dojo.forEach(item.getCalcuteValue().getCells(),function(item){
								if(item.getCell() && item.getCell().isError){
									throw item.getCell().getShowValue();
								}
							});
						}
						//calculate check
						var rangRows;
						try{
							var rangRows = item.getCalcuteValue().getRows();
						}catch(e){
							rangRows = item.getRows();
						}
						dojo.forEach(rangRows,function(rows,rowIndex){
							var row = rows;
							if(!dojo.isArray(rows)){row = new Array();row.push(rows);}
							dojo.forEach(row,function(cell,cellIndex){
								cache = operator(cache,cell,index,cellIndex,rowIndex);
							},this);
						},this);
					}else if(item instanceof websheet.parse.token || item instanceof websheet.parse.tokenList){
						if(dojo.isArray(item.getCalcuteValue())){
							cache = this.IterateInArray(item.getCalcuteValue(),cache,operator,isArrayFormula,ignoreError);
						}else{
							if(item.getTokenType() == websheet.parse.tokenType.NEGATIVE_TOKEN){
								cache = this.IterateInArray(item.getCalcuteValue(),cache,operator,isArrayFormula,ignoreError);
							}else{
								var value = item.getCalcuteValue();
								cache = operator(cache,value,index);
							}
						}
					}else{
						cache = operator(cache,item,index);
					}
				}catch(e){
					throw (e.message || e.toString() || "Error");
				}
			}
		},this);
		return cache;
	},
	
	isCellStore: function(store)
	{
		if(store instanceof websheet.model.CellModel
		  || store instanceof websheet.parse.token
		  || store.getTokenType() == websheet.parse.tokenType.CELLREF_TOKEN )
		{
			return true;
		}
	},
	
	isEquation:function(ch){
		var equation = "= > < <= >= <> !=";
		return equation.indexOf(ch) >=0?true:false;
	},
	
	isTokenList:function(store){
		if(store instanceof websheet.parse.tokenList)return true;
		return false;
	},
	
	isRangStore:function(store){
		if(store instanceof websheet.model.RangeModel)return true;
		if(!(store instanceof websheet.parse.token))return false;
		if(store.getTokenType() == websheet.parse.tokenType.CELLREF_TOKEN || store.getTokenType() == websheet.parse.tokenType.RANGEREF_TOKEN)return true;
		return false;
	},
	isInSheetsArray:function(cells){
		var sRowIdx = cells[0].getCellVieType().getRow();
		var sColIdx = websheet.ColumnHelper.translateFromColCharacters(cells[0].getCellVieType().getCol());
		var minRowIdx = pe.base.getWorksheetContainer().getMinRowIdx();
		var minColIdx = pe.base.getWorksheetContainer().getMinColIdx();
		if(cells.length!=1){
			var eRowIdx = cells[cells.length-1].getCellVieType().getRow();
			var eColIdx = websheet.ColumnHelper.translateFromColCharacters(cells[cells.length-1].getCellVieType().getCol());
			if(sRowIdx>eRowIdx){
			if(sRowIdx>minRowIdx)
				return false;
			}else{
				if(eRowIdx>minRowIdx)
					return false;
			}
			if(sColIdx>eColIdx){
				if(sColIdx>minRowIdx)
					return false;
			}else{
				if(eColIdx>minRowIdx)
					return false;
			}
		}else{
			if(sRowIdx>minRowIdx)
				return false;
			if(sColIdx>minRowIdx)
				return false;
		}
		return true;
	}
};

