dojo.provide("websheet.model.DocumentModel");
dojo.require("websheet.model.SheetModel");
dojo.require("websheet.Helper");
dojo.require("websheet.model.RangeModel");
dojo.require("websheet.model.ROAModel");
dojo.require("websheet.model.StyleList");
dojo.require("dojox.collections.Stack");

dojo.declare("websheet.model.DocumentModel",null,{
	type: null,
	url: null,
	sheets: {},									//document object
	count: 0,									//sheet count
	base:null,
	styleList:null,								//store style object.to get it by style id
	nameRangList:null,								//store name rang
	unnameRangeList:null,						//store unnamed range
	_memory: null,								//escape cyclic reference
	isRecycle:null,					
	_recycle:null,								//store recycle cell
	_rowUpdate:null,							//mark the row update
	status:null,								//whether it is loading,help to resolve the recycle
	commentsList:null,              				//Map comments and cells binding info.
	constructor: function(args) {
		dojo.mixin(this, args);
	},
	
	//initialize the document model
	init: function(doc){
		this.status = true;
		this.loadDocument(doc);
		this._parse();
		this.status = false;
	},
	oldLoad:function(doc){
		if(doc == undefined || doc == null)return null;
		for(var sheet in doc.sheets){
			this.createSheet(sheet,doc.sheets[sheet].id);
			this.sheets[sheet].loadSheet(doc.sheets[sheet]);
			this.sheets[sheet].index = this.count;
			this.count += 1;
		}
	},
	//load document json data to generate document object.
	loadDocument: function(doc){ 
		if(doc == undefined || doc == null)return null;
		/** load content.js */
		this.styleList = new websheet.model.StyleList(doc.content.styles);
		
		/** load meta.js */
		this.url = doc.meta.url;
		var sheets = doc.meta.sheetsIdArray;
		var obj = {};
		obj.self = this;
		obj.document = doc;
		dojo.forEach(sheets,function(sheetId,index){
			if(sheetId && sheetId.replace(/(^\s*)|(\s*$)/g,"") != ""){
				var sheetName = this.document.meta.sheets[sheetId].sheetname;
				var sheet = this.self.createSheet(sheetName,sheetId,this.self);
				
				sheet.index = parseInt(index) + 1 ;
				if (this.document.meta.sheetsArray!=undefined)
				{
					sheet.loadSheet(this.document.meta.sheetsArray[sheetId],this.document);
				}
				this.self.sheets[sheetName] = sheet;
			}
		},obj);
		
		/** load content.js */
		this.loadCellRang(doc.content.names, true);
		this.loadCellRang(doc.content.unnames, false);
		this.styleList.deleteAll();
		delete this.styleList;
	},
	
	//only the valid range can be put in the range list
	loadCellRang:function(rang, isNameRange){
		var rangeModel = null;
		var rangData = null;
		for(var id in rang){
			rangData = rang[id];
			if(rangData){
				rangeModel = new websheet.model.RangeModel(null,null,this);
				var result = rangeModel.transform(rangData.sheetid,rangData.startrowid,rangData.startcolid,rangData.endrowid,rangData.endcolid);
				if(result){
					rangeModel.getCellsFromAddress();
					rangeModel.rangeId = id;
					if(isNameRange){
						this.getNameRangList()[id]=rangeModel;
					}else{
						rangeModel.setUsage(rangData.usage);
						this.getUnNameRangeList()[id]=rangeModel;
					}
				}
//				else{
//					//if transform return wrong, then the address must contains "#REF!"
//					range.address
//				}
			}
		}
	},
	
	//if cell has formula,then parse it.
	_parse:function(){
		for(var sheet in this.sheets){
			this.sheets[sheet]._parse();
		}
	},

	//create a new sheet in document.
	createSheet: function(sheetName,sheetId) {
		var id = sheetId;
		if(id == undefined || id == null){
			id = websheet.Helper.newGuid();
		}
		var sheet = new websheet.model.SheetModel(id,sheetName,this);
		this.sheets[sheetName] = sheet;
		this.count += 1;
		sheet.index = this.count;
		return sheet;
	},
	
	/* delete the sheet with specified name, */
	/* the reference will be adjusted accordingly */
	/* success -- return true, failed -- return false */
	deleteSheet: function(sheetName) {
		var sheet = this.getSheet(sheetName);
		var curIndex = sheet.getIndex();
		sheet.isDelete = true;
		sheet.dependenceAutoUpdate(true);
		for(var row in sheet.rows){
			if(row != undefined){
				sheet.deleteRow(row,true,true);
			}
		}
		delete this.sheets[sheetName];
		this.count -= 1;
		//update sheet index
		for(var s in this.sheets){
			var so = this.sheets[s];
			var i = parseInt(so.getIndex());
			if(i > curIndex){
				so.setIndex(i - 1);
			}
		}
		this.updateRanges(2,sheetName);
		return this.getRowUpdate();
	},
	
	/* to check sheet exist or not */
	/* return true or false */
	_isSheetExist: function(sheetName) {
		return !!this.sheets[sheetName];
	},
	
	getSheet:function(sheetName){
		if(!this._isSheetExist(sheetName)){
			throw "#REF!";
		}
		return this.sheets[sheetName];
	},
	
	getSheetById:function(sheetId){
		for(var sheet in this.sheets){
			if(this.sheets[sheet]._id == sheetId){
				return this.sheets[sheet];
			}
		}
		return null;
	},
	
	/* move sheet */
	/* new position - sheet No.1 - 0; sheet No.2 - 1, etc */
	/* success -- return true, failed -- return false */
	moveSheet: function(sheetName, newPosition){
		this.getSheet(sheetName).index = newPosition;
	},
	
	/* rename sheet */
	/* success -- return true, failed -- return false */
	renameSheet: function(sheetName, newSheetName,notSend) {
		this.clearRowUpdate();
		this.updateRanges(2, sheetName, newSheetName);
		var sheet = this.sheets[sheetName];
		delete this.sheets[sheetName];
		this.sheets[newSheetName] = sheet;
		return this.sheets[newSheetName].renameSheet(newSheetName,notSend);
		
	},
		
	/* get cell show Value*/
	getCellShowValue: function(sheetName, row, column) {
		return this.getSheet(sheetName).getCellShowValue(row, column);
	},
		
	/* get cell raw value */
	getCellRawValue: function(sheetName, row, column) {
		return this.getSheet(sheetName).getCellRawValue(row, column);
	},

	/* set cell raw value */
	setCellRawValue: function(sheetName, row, column, value,notSend){
		var sheet = this.getSheet(sheetName);
		return sheet.setCellRawValue(row, column, value,notSend);
	},
	/* set cell comments id*/
	createCellComments:function(sheetName, row, column, commentsId){
		var sheet = this.getSheet(sheetName);
		this.setCommentsCellIndex(sheetName,commentsId,row,column);
		sheet.createCellComments(row, column, commentsId);
	},
	/* delete cell comments by id*/
	delCellComments:function(sheetName,row, column, commentsId){
		var sheet = this.getSheet(sheetName);
		delete this.commentsList[commentsId];
		sheet.delCellComments(row, column, commentsId);
	},
	/*return the mapping cell for commentsId*/
	getCommentsCellIndex:function(commentsId){
		for(id in this.commentsList){
			if(id==commentsId){
				return this.commentsList[id];				
			}
		}
	},
	setCommentsCellIndex:function(sheetName,commentsId,row,column){
		if(commentsId!=null){
			if(undefined==this.commentsList||null==commentsId){
			this.commentsList = {};	
			}
			var index = new Array();
			index.push(sheetName);
			index.push(row);
			index.push(column);
			this.commentsList[commentsId] = index;
		}
	},
	
	/* get cell object,if isGeneral param is true,then when the cell is null,it will also return a cell object*/
	getCell: function(sheetName, row, column, isGeneral){
		var cell = null;
		try{
			cell = this.getSheet(sheetName).getCell(row,column,isGeneral);
		}catch(e){
			throw e.message || e.toString() +  "." + column + row;
		}
		if(this.status){
			if(cell.calculateStatus){
				var obj = {};
				obj.self = this;
				obj.end = false;
//				dojo.forEach(cell.getReferencedList(),function(token){
//					dojo.forEach(token.getCalcuteValue().getCells(),function(ROA,index){
//						cell = ROA.getCell();
//						if(!this.end && !this.self.addMemory(cell)){
//							dojo.forEach(this.self.getMemory().toArray(),function(item){
//								item.setShowValue('###Cyclic Reference###');
//								item.dependenceAutoUpdate(true);
//							});
//							this.self.end = true;
//							this.self.clearMemory();
//							this.self.clearRecycle();
//							throw "Cyclic Reference";
//						}
//					},this);
//				},obj);
			}else{
//				if(!this.addMemory(cell)){
//					dojo.forEach(this.getMemory().toArray(),function(item){
//						item.setShowValue('###Cyclic Reference###');
//						item.dependenceAutoUpdate(true);
//					});
//					throw "Cyclic Reference";
//					this.clearMemory();
//					this.clearRecycle();
//				}else{
//					cell._parse();
//				}
			}
		}
		return cell;
	},
	
	//get referenced cell value
	getCellRef:function(sheetName, row, column,ref,hasSheetRef){
		this.getSheet(sheetName);
		column = column.toUpperCase();
		var rang = new websheet.model.RangeModel(ref);
		var cell = this.getCell(sheetName, row, column,true);
		var cellValue = cell.getShowValue();
		var ROA = new websheet.model.ROAModel(cell,sheetName,row,column,websheet.model.ROAType.BOTHRELATIVE,hasSheetRef);
		rang.getRows().push(cellValue);
		rang.getCols().push(cellValue);
		rang.pushCell(ROA);
		rang.sheetName = sheetName;
		return rang;
	},
	
	//get cell range value
	getCellRangeRef: function(sheetName, startColumn, startRow, endColumn, endRow, ref,hasSheetRef) {
		this.getSheet(sheetName);
		startColumn = startColumn.toUpperCase();
		endColumn = endColumn.toUpperCase();
		var startColNumber = websheet.ColumnHelper.translateFromColCharacters(startColumn);
		var endColNumber = websheet.ColumnHelper.translateFromColCharacters(endColumn);
		if (startColNumber > endColNumber) {
			var tColNumber = endColNumber;
			endColNumber = startColNumber;
			startColNumber = tColNumber;
		}
		if (startRow > endRow) {
			var tRow = endRow;
			endRow = startRow;
			startRow = tRow;
		}
		var address = ref.split(":");
		var rang = new websheet.model.RangeModel(address[0],address[1]);
		rang.hasSemicolon = true;
		rang.sheetName = sheetName;
		var cellObj = null;
		var ROA = null;
		for (var iRow = startRow; iRow <= endRow; iRow++) {
			var row = new Array();
			for (var iColNum = startColNumber; iColNum <= endColNumber; iColNum++) {
				var iCol = websheet.ColumnHelper.translateFromColNum(iColNum);
				cellObj = this.getCell(sheetName, iRow, iCol,true);
				ROA = new websheet.model.ROAModel(cellObj,sheetName,iRow,iCol,websheet.model.ROAType.BOTHRELATIVE,hasSheetRef);
				var cellValue = cellObj.getShowValue();
				rang.pushCell(ROA);
				row.push(cellValue);
				var colLen = rang.getCols().length;
				var colIndex = iColNum - startColNumber;
				if(colLen >= colIndex + 1){
					rang.getCols()[colIndex].push(cellValue);
				}else{
					var cell = new Array();
					cell.push(cellValue);
					rang.getCols().push(cell);
					delete cell;
				}
			}
			rang.getRows().push(row);
			delete row;
		}
		
		return rang;
	},
	
	//return RangeModel instance by range address
	getCellRangeFromAddress: function(rangeAddress) {
		var address = rangeAddress.split(":");
		var range = new websheet.model.RangeModel(address[0],address[1], this);
		range.getCellsFromAddress();
		return range;
	},
	
	/* apply cell style */
	applyCellStyle: function(sheetName, row, column, styleIndex) {
	},

	/* apply cell format */
	applyCellFormat: function(sheetName, row, column, formatIndex) {
	},
		
	/* todo later */
	hideSheet: function(sheetName) {
	},
	
	/* todo later */
	showSheet: function(sheetName) {
	},
	
	/* todo later */
	getSheetCount: function() {
		var sheetCount = 0;
		for(var sheet in this.sheets){
			sheetCount += 1;
		}
		return sheetCount;
	},
		
	/* todo later */
	copySheet: function(sheetname) {
	},
	
	/* insert a row with the row index, the first row with index 0 */
	/* the cell reference should be adjusted accordingly */
	/* row index start from 1 */
	insertRow: function(sheetName,rowIndex,notSend) {
		this.getSheet(sheetName).insertRow(rowIndex,notSend);
		//deleteType = 0
		this.updateRanges(0,sheetName);
	},	
	/* delete a row with the row index, the first row with index 0 */
	/* the cell reference should be adjusted accordingly */
	/* row index start from 1 */
	deleteRow: function(sheetName,rowIndex,notSend) {
		this.getSheet(sheetName).deleteRow(rowIndex,notSend);
		this.updateRanges(0,sheetName);
	},
	
	/* insert a column with the column index, the first column with index 0 */
	/* the cell reference should be adjusted accordingly */
	/* column index start from 1 */
	insertColumn: function(sheetName,colIndex,notSend) {
		this.getSheet(sheetName).insertColumn(colIndex,notSend);
		this.updateRanges(1,sheetName);
	},	
	/* delete a column with the column index, the first column with index 0 */
	/* the cell reference should be adjusted accordingly */
	/* column index start from 1 */
	deleteColumn: function(sheetName,colIndex,notSend) {
		this.getSheet(sheetName).deleteColumn(colIndex,notSend);
		this.updateRanges(1,sheetName);
	},
	moveSheet:function(sheet,move2Index){
		var moveIndex = Number(move2Index);
		if(sheet.getIndex() ==  moveIndex){return false;}
//		dojo.forEach(this.sheets,function(item){
//			if(item.getIndex() > sheet.getIndex() && item.getIndex() <= moveIndex){
//				item.setIndex(parseInt(item.getIndex()) - 1);
//			}
//		},this);
		var curIndex = sheet.getIndex();
		var sheets = this.sheets;
	    if (curIndex < moveIndex) 
	    {
	        for (var shname in sheets) 
	        {
	           var tmpSheet = sheets[shname];
	           var tmpSheetIndex = tmpSheet.index;
	           if (tmpSheetIndex > curIndex && tmpSheetIndex <= moveIndex) 
	           {
	              tmpSheet.index = tmpSheetIndex - 1;
	           }
	        }
	    }
	    else 
	    {
	        for (var shname in sheets) 
	        {
	        	var tmpSheet = sheets[shname];
	        	var tmpSheetIndex = tmpSheet.index;
	            if (tmpSheetIndex >= moveIndex && tmpSheetIndex < curIndex)
	            {
	                tmpSheet.index = tmpSheetIndex + 1;
	            }
	         }
	    }
		sheet.setIndex(moveIndex);
		return true;
	},
	getStyleList:function(){
		return this.styleList;
	},
	
	getStyle:function(styleId){
		var styleList = this.getStyleList();
		if(styleList == null){return null;}
		return styleList.generateStyleCodebyID(styleId);
	},
	
	getNameRangList:function(){
		if(this.nameRangList == null){
			this.nameRangList = {};
		}
		return this.nameRangList;
	},
	
	getUnNameRangeList:function(){
		if(this.unnameRangList == null){
			this.unnameRangList = {};
		}
		return this.unnameRangList;
	},
	
	getMemory:function(){
		if(this._memory == null || this._memory == undefined){
			this._memory = new dojox.collections.Stack();
		}
		return this._memory;
	},
	
	addMemory:function(cell){
		if(!this.getMemory().contains(cell)){
			this.getMemory().push(cell);
			return true;
		}
		return false;
	},
	
	clearMemory: function() {
		if(this._memory == null || this._memory == undefined){return;}
		this.getMemory().clear();
	},
	
	
	getRecycle:function(){
		if(this._recycle == null || this._recycle == undefined){
			this._recycle = new dojox.collections.Stack();
		}
		return this._recycle;
	},
	
	addRecycle:function(cell){
		if(!this.getRecycle().contains(cell)){
			this.getRecycle().push(cell);
			return true;
		}
		return false;
	},
	
	clearRecycle: function() {
		if(this._recycle == null || this._recycle == undefined){return;}
		this.getRecycle().clear();
	},
	
	
	getRowUpdate:function(){
		if(this._rowUpdate == null || this._rowUpdate == undefined){
			this._rowUpdate = new dojox.collections.Stack();
		}
		return this._rowUpdate;
	},
	
	addRowUpdate:function(rowIndex){
		if(!this.getRowUpdate().contains(rowIndex)){
			this.getRowUpdate().push(rowIndex);
		}
	},
	
	clearRowUpdate:function(){
		if(this._rowUpdate == null || this._rowUpdate == undefined){return;}
		this.getRowUpdate().clear();
	},
	
	updateRanges:function(deleteType, sheetName, newSheetName){
		var unnameRanges = this.getUnNameRangeList();
		for(var rangeId in unnameRanges){
			var range = unnameRanges[rangeId];
			if(range){
				if(range.sheetName == sheetName){
					range.updateRangeAddress(deleteType, newSheetName);
				}
			}
		}
	}
});