dojo.provide("websheet.Helper");

//dojo.declare('websheet.Helper',[],{});
dojo.require('websheet.ColumnHelper');
dojo.require("websheet.event.eventFactory");

websheet.Helper = {

	/**
	 * Regular expressions for matching textual reference strings
	 */
	rgx: {
		simpleSheet: "([\\w]+)",
		sheet: "([\\w]+|[\\w\\s]+)",
		doc: "([^\\\/\:\*\?\"\<\>\|]+)",
		column: "([a-zA-Z]{1}|[a-hA-H]{1}[a-zA-Z]?|I[a-vA-V]?)",
		row: "([1-9]+[0-9]*)"
	}	/**
	 * The compiled regular expression for matching textual ref strings
	 */
	,
	refMatch: null,
	
	newGuid:function(){
		var guid = "";
		for(var i = 1;i <= 32;i++){
			var n = Math.floor(Math.random()*16.0).toString(16);
			guid += n;
			if((i == 8)||(i == 12)||(i == 16)||(i == 20)){
				guid += "_";
			}
		}
		return guid;
	},
	
	// checks to see if the passed value is a numeric value using a regular expression
	isNumeric: function(columnVal)
	{
		if(columnVal == null || columnVal == undefined){return false;}
		columnVal = columnVal.toString();
		var numberRegEx = /^[1-9][0-9]*(\.[0-9]*)?$/;
		//performing a trim
		columnVal = columnVal.replace(/^(\s|&nbsp;)+|(\s|&nbsp;)+$/g,"");
		
		if(columnVal.search(numberRegEx) < 0)
			return false;
		else
			return true;
				 
	},  
	trim:function(str){
		return str.replace(/(^\s*)|(\s*$)/g, "");
	},
	
	handleRefValue: function(refValue, refType)
    {
        var newValue = refValue;
        switch(refType)
        {
            case websheet.event.eventFactory.EVENT_SCOPE_ROW:
            	newValue = refValue.replace(".",".A");
            break;
            case websheet.event.eventFactory.EVENT_SCOPE_COLUMN:
            	newValue = refValue + "1";
            break;
            //Co-editing_Conflict
            //case this.SCOPE_SHEET:  //this line may be copied form other place
            case websheet.event.eventFactory.EVENT_SCOPE_SHEET:
            	newValue = refValue.split("|")[0];
            	break;
        }
        return newValue;   
    },
	
	/**
	 * parses a reference string into a parsed reference (object)
	 *
	 * @param {String} ref
	 * @return {Object} parsed reference
	 */
	parseRef: function(ref) {
		
		if (!ref) {
			return null;
		}
		
		var matches = websheet.Helper.refMatch.exec(ref);
		//        console.debug(matches);
		if (matches) {
			//            console.debug('match!');
			if (matches[6]) {
				
				return websheet.Helper.__createParsedRef(matches[6], matches[8], matches[9], matches[11], matches[12]);
			} else {
				return websheet.Helper.__createParsedRef(null, matches[1], matches[2], matches[4], matches[5]);
			}
			
		} else {
			return null;
		}
	},
	
	/**
	 * creates a parsed reference object
	 *
	 *
	 * @param {String} sheet
	 * @param {String} column
	 * @param {int} row
	 * @param {String} endColumn
	 * @param {int} endRow
	 * @return {Object} parsed reference
	 */
	__createParsedRef: function(sheet, column, row, endColumn, endRow) {
		return {
			sheet: sheet,
			column: column,
			row: parseInt(row, 10) || row,
			endColumn: endColumn != 'undefined' ? endColumn : null,
			endRow: endRow != undefined ? (parseInt(endRow, 10) || endRow) : 0,
			isRange: function() {
				return this.sheet && this.column && this.row && this.endColumn && this.endRow;
			},
			isCell: function() {
				return this.sheet && this.column && this.row && !this.endColumn && !this.endRow;
			},
			isSheet: function() {
				return this.sheet && !this.column && !this.row && !this.endColumn && !this.endRow;
			},
			isAbsolute: function() {
				return !!this.sheet;
			},
			isRelative: function() {
				return !this.sheet;
			},
			getColumnNumber: function() {
				return websheet.ColumnHelper.translateFromColCharacters(this.column);
			},
			getEndColumnNumber: function() {
				return websheet.ColumnHelper.translateFromColCharacters(this.endColumn);
			},
			toString: function() {
				var ret = "";
				if (this.sheet) {
					ret += this.sheet;
					if (!this.isSheet()) {
						ret += '.';
					}
				}
				if (this.isCell() || this.isRange()) {
					ret += this.column + this.row;
				}
				if (this.isRange()) {
					ret += ':' + this.endColumn + this.endRow;
				}
				return ret;
			},
			addressType:function(){
				if(this.sheet){
					return 1;
				}else{
					return 0;
				}
			}
		};
	},
	normalizeParsedRef: function(ref) {
		var fColNumber = websheet.ColumnHelper.translateFromColCharacters(ref.column);
		var eColNumber = websheet.ColumnHelper.translateFromColCharacters(ref.endColumn);
		if (fColNumber > eColNumber) {
			var tColNumber = eColNumber;
			eColNumber = fColNumber;
			fColNumber = tColNumber;
		}
		var fRow = ref.row;
		var eRow = ref.endRow;
		if (fRow > eRow) {
			fRow = ref.endRow;
			eRow = ref.row;
		}
		
		ref.column = websheet.ColumnHelper.translateFromColNum(fColNumber);
		ref.endColumn = websheet.ColumnHelper.translateFromColNum(eColNumber);
		
		ref.row = fRow;
		ref.endRow = eRow;
		
		return ref;
	}	
	
	/**
	 * copied from dojo 0.4.3 (dojo.string.escapeXml)
	 *
	 * @param {Object} str
	 * @param {Object} noSingleQuotes
	 */
	,
	escapeXml: function(str, noSingleQuotes) {
		//summary:
		//	Adds escape sequences for special characters in XML: &<>"'
		//  Optionally skips escapes for single quotes
		
		str = str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
		if (!noSingleQuotes) {
			str = str.replace(/'/gm, "&#39;");
		}
		return str; // string
	},
	
	normalizeSheetName: function(sheetName) {
		if (dojo.isString(sheetName) && dojo.trim(sheetName).length) {
			if (sheetName.indexOf(' ') != -1 && // there is a space
			sheetName.substr(0, 1) != "'" && // and it starts _not_ with '
			sheetName.substr(-1) != "'" // and it does _not_ end with '
) {
				sheetName = "'" + sheetName + "'";
			} else if (sheetName.indexOf(' ') == -1 && // does not contain a space
			sheetName.substr(0, 1) == "'" && // but it starts with '
			sheetName.substr(-1) == "'" // and it ends with '				
) {
				sheetName = sheetName.substring(1, sheetName.length - 1);
			}
		}
		return sheetName;
	},
	
	isValidSheetName: function(sheetName) {
		var isValid = false;
		if (dojo.isString(sheetName) && dojo.trim(sheetName).length) {
			isValid = new RegExp('^' + websheet.Helper.rgx.sheet + '$').test(sheetName);
		}
		return isValid;
	},
	
	isValidDocName: function(docName) { 
		var isValid = false;
		if (dojo.isString(docName) && dojo.trim(docName).length) {
			isValid = new RegExp('^' + websheet.Helper.rgx.doc + '$').test(docName);
		}
		return isValid;
	},
	
	//get the value of attribute which is defined in the css of node
	getStyle: function(node,attribute){
		if(dojo.isIE){
			return node.currentStyle?node.currentStyle[attribute]:'';
		}
		return document.defaultView.getComputedStyle(node,null)[attribute];   
	}  


};

websheet.Helper.rgx.cell = websheet.Helper.rgx.column + websheet.Helper.rgx.row;
websheet.Helper.rgx.relRef = websheet.Helper.rgx.cell + "(:" + websheet.Helper.rgx.cell + ")?";
websheet.Helper.rgx.absRef = websheet.Helper.rgx.sheet + "(." + websheet.Helper.rgx.relRef + ")?";
websheet.Helper.rgx.anyRef = websheet.Helper.rgx.relRef + "|" + websheet.Helper.rgx.absRef;
websheet.Helper.refMatch = new RegExp("^" + websheet.Helper.rgx.anyRef + "$", "i");
