/********************************************************/
/* created by Rui SuYing, ruisuy@cn.ibm.com				*/
/*  2010/6/28											*/
/* for range object in a spreadsheet document			*/
/********************************************************/

dojo.provide("websheet.model.RangeModel");

//dojo.require("websheet.model.DocumentModel");
dojo.require("websheet.model.SheetModel");
dojo.require("websheet.model.CellModel");
dojo.require("websheet.Helper");
dojo.declare("websheet.model.RangeModel",null,{
	TASK_RANGE: "TASK",
	COMMENT_RANGE: "COMMENT",
	NAME_RANGE: "NAMES",
	
	sheetId: null,
	sheetName:null,
	rangeId: null,
	rangeName: null,
	startAddress: null,
	endAddress: null,
	_dependencedList:null,		//cells list which is dependent on this range 
	_rows:null,
	_cols:null,
	cells:null,
	document:null,
	isNegative:false,
	usage:null, //can be TASK|COMMENT|NAMES
	hasSemicolon:false,			//if it is true,the range will like A1:A1 or A1:B2
	
	constructor: function(startAddress, endAddress,document) {
		this.startAddress = startAddress;
		this.endAddress = endAddress;
		this.document = document;
	},

	/**
	 * transform the sheet/row/column id to address.
	 * Return true if transform is done, otherwise return false when the id is invalid.
	 */
	transform:function(sheetId, startRowId,startColId,endRowId,endColId) {
		try{
			this.sheetID = sheetId;
			var sheet = this.document.getSheetById(sheetId);
			this.sheetName = sheet.getSheetName();
			this.startAddress = this.sheetName + "." + sheet.getColumnById(startColId).index + sheet.getRowById(startRowId).index;
			this.endAddress =sheet.getColumnById(endColId).index + sheet.getRowById(endRowId).index;
		}catch(e){
			return false;
		}
		return true;
	},
		
	/* BOOLEAN, returns whether the cell specified by the  	*/
	/* "CellAddress" depends on this range 					*/
	/* check _dependencedList to get the result 			*/
	checkDependency: function( CellAddress ) {
	},
	
	/* return value: 										*/
	/*		0 -- outside of the range						*/
	/*		1 -- inside of the range						*/
	checkCellRelationship: function( CellAddress ) {
	},
	
	/* return value: 										*/
	/*		0 -- outside of the range						*/
	/*		1 -- inside of the range						*/
	/*		2 -- at the edge but inside the range			*/
	checkRowRelationship: function( RowID ) {
	},
	
	/* return value: 										*/
	/*		0 -- outside of the range						*/
	/*		1 -- inside of the range						*/
	/*		2 -- at the edge but inside the range			*/
	checkColumnRelationship: function( ColNum ) {
	},	
	
	/* Range scope extension when a row insertion operation	*/
	RangeExtensionbyInsertRows: function( startRow, endRow ) {
	},
	
	/* Range scope extension when a row insertion operation	*/
	RangeExtensionbyInsertColumns: function( startColumn, endColumn ) {
	},
		
	SetRangeName: function( rangeName )  {
		this.rangeName = rangeName;
	},
		
	GetRangeName: function() {
		return this.rangeName;
	},
		
	/**
	 *
	 * @param {Array} paramRows
	 */
	setRows:function(paramRows){
		this._rows = paramRows;
	},
	
	getRows:function(){
		if(this._rows == null){
			this._rows = new Array();
		}
		return this._rows;
	},
	
	/**
	 *
	 * @param {Array} paramCols
	 */
	setCols:function(paramCols){
		this._cols = paramCols;
	},
	getCols:function(){
		if(this._cols == null){
			this._cols = new Array();
		}
		return this._cols;
	},
	
	getStartAddress:function(){
		return this.startAddress;
	},
	
	getEndAddress:function(){
		return this.endAddress;
	},
	
	/**
	 *
	 * @param {String} paramShowValue
	 */
	setShowValue:function(paramShowValue){
		this._showValue = paramShowValue;
	},
	getShowValue:function(){
		return this._showValue;
	},
	
	getCells:function(){
		if(this.cells == null){
			this.cells = new Array();
		}
		return this.cells;
	},
	
	pushCell:function(cell){
		this.getCells().push(cell);
	},
	
	setUsage:function(use){
		this.usage = use;
	},
	
	getUsage:function(){
		if(this.usage == null)
			this.usage = this.NAME_RANGE;
		return this.usage;
	},
	
	//push row, col, cells to this range model which is initialized by range address
	init:function(){
		//TODO get code from DocumentMode.getRangeFromAddress

	},
	
	updateRangeAddress : function(deleteType, newSheetName){
		var existCells = new Array();
		var oriCells = this.getCells();
		var token = "";
		//delete all cells in deleted column or deleted row
		dojo.forEach(this.getCells(),function(item,index){
			if(item){
				if(item.cell.getParent().isDelete || item.cell.getColumn().isDelete || item.cell.getParent().getParent().isDelete){
					item.cell.clearReference();
				}else{
					this.push(item);
				}
			}
		},existCells);
		//regenerate the cell address
		cells = existCells.sort();
		if(cells.length == 0){
			//the first cell
			var startCell = oriCells[0].getCellVieType();
			//the last cell
			var endCell = oriCells[oriCells.length-1].getCellVieType();
			if(oriCells[0].hasSheetRef ){
				if( (deleteType == 2) && newSheetName ){
					this.sheetName = newSheetName;
					this.startAddress = newSheetName + ".";
				}else if(deleteType != 2)
					this.startAddress = startCell.getSheetName() + ".";
			}else
				this.startAddress = "";
			if(deleteType == 0){
				//delete row
				this.startAddress += startCell.getCol() + "#REF!";
				this.endAddress = endCell.getCol() + "#REF!";
				
//				token += startCell.getCol() + "#REF!" + ":" + endCell.getCol() + "#REF!";
			}else if(deleteType == 1){
				//delete column
				this.startAddress += "#REF!" + startCell.getRow();
				this.endAddress = "#REF!" + endCell.getRow();
//				token += "#REF!" + startCell.getRow() + ":" + "#REF!" + endCell.getRow();
			}else{
				//delete sheet
				this.startAddress = "#REF!." + startCell.getCol() + startCell.getRow();
				this.endAddress = endCell.getCol() + endCell.getRow();
//				token += "#REF!." + startCell.getCol() + startCell.getRow() + ":" + endCell.getCol() + endCell.getRow();
			}
			isError = true;
		}else{
			//the first cell
			var startCell = cells[0].getCellVieType();
			//the last cell
			var endCell = cells[cells.length-1].getCellVieType();
			token = "";
			if(cells[0].hasSheetRef){
//				token = startCell.getSheetName() + "." ;
				if( (deleteType == 2) && newSheetName ){
					this.sheetName = newSheetName;
					this.startAddress = newSheetName + ".";
				}else if(deleteType != 2)
					this.startAddress = startCell.getSheetName() + ".";
			}else
				this.startAddress = "";
			
			this.startAddress += startCell.getCol() + startCell.getRow();
			this.endAddress = endCell.getCol() + endCell.getRow();
//			token += startCell.getCol() + startCell.getRow() + ":" + endCell.getCol() + endCell.getRow();
		}
		token = this.startAddress + ":" + this.endAddress;
		this.cells = cells;
		console.log("regenerate range address is :" + token)
	},
	
	//compare the relationship between two range addresses
	//return 0 if they have no intersection
	//return 1 if they have
	//return 2 if one range contains or is contained by another range
	//return -1 when error , for example, one range contains sheetName ,while another does not have
	compare : function(rangeAddress){
		var currentRangeAddress = this.startAddress + ":" + this.endAddress;
		var parsedRef1 = websheet.Helper.parseRef(currentRangeAddress);
		var parsedRef2 = websheet.Helper.parseRef(rangeAddress);
		if(parsedRef1 && parsedRef2){
		var sheetName1 = parsedRef1.sheet;
		var startColumn1 = parsedRef1.column;
		var endColumn1 = (parsedRef1.endColumn == undefined)? startColumn1 : parsedRef1.endColumn;;
		var startRow1 = parsedRef1.row;
		var endRow1 = (parsedRef1.endRow < 1)? startRow1 : parsedRef1.endRow;;
		var hasSheetRef1 = parsedRef1.addressType();
		
		var sheetName2 = parsedRef2.sheet;
		var startColumn2 = parsedRef2.column;
		var endColumn2 = (parsedRef2.endColumn == undefined || parsedRef2.endColumn == "")? startColumn2 : parsedRef2.endColumn;
		var startRow2 = parsedRef2.row;
		var endRow2 = ( parsedRef2.endRow < 1 || parsedRef2.endRow == "")?startRow2 : parsedRef2.endRow;
		var hasSheetRef2 = parsedRef2.addressType();
		var startCompare = false;
		if(hasSheetRef1 && hasSheetRef2){
			if(sheetName1 == sheetName2){
				startCompare = true;
			}else
				return 0;
		}else if(!hasSheetRef1 && !hasSheetRef2){
			startCompare = true;
		}
		if(startCompare){
			var startColNumber1 = websheet.ColumnHelper.translateFromColCharacters(startColumn1);
			var endColNumber1 = websheet.ColumnHelper.translateFromColCharacters(endColumn1);
			if (startColNumber1 > endColNumber1) {
				var tColNumber1 = endColNumber1;
				endColNumber1 = startColNumber1;
				startColNumber1 = tColNumber1;
			}
			if (startRow1 > endRow1) {
				var tRow1 = endRow1;
				endRow1 = startRow1;
				startRow1 = tRow1;
			}
			var startColNumber2 = websheet.ColumnHelper.translateFromColCharacters(startColumn2);
			var endColNumber2 = websheet.ColumnHelper.translateFromColCharacters(endColumn2);
			if (startColNumber2 > endColNumber2) {
				var tColNumber2 = endColNumber2;
				endColNumber2 = startColNumber2;
				startColNumber2 = tColNumber2;
			}
			if (startRow2 > endRow2) {
				var tRow2 = endRow2;
				endRow2 = startRow2;
				startRow2 = tRow2;
			}
			if(((startRow1 >= startRow2) && (endRow1 <= endRow2) && (startColNumber1 >= startColNumber2) && (endColNumber1 <= endColNumber2))
					|| ((startRow1 <= startRow2) && (endRow1 >= endRow2) && (startColNumber1 <= startColNumber2) && (endColNumber1 >= endColNumber2)))
				return 2;
			var rowHasIntersect = !((endRow1 < startRow2) || (endRow2 < startRow1));
			var colHasIntersect = !((endColNumber1 < startColNumber2) || (endColNumber2 < startColNumber1));
			if(rowHasIntersect && colHasIntersect){
				return 1;
			}else
				return 0;
		}
		}
		return -1;
	},
	
	//return RangeModel instance by range address
	getCellsFromAddress: function() {
		var rangeAddress = this.startAddress + ":" + this.endAddress;
		var parsedRef = websheet.Helper.parseRef(rangeAddress);
		var sheetName = parsedRef.sheet;
		this.sheetName = sheetName;
		var startColumn = parsedRef.column;
		var endColumn = parsedRef.endColumn;
		var startRow = parsedRef.row;
		var endRow = parsedRef.endRow;
		var hasSheetRef = parsedRef.addressType();
		this.document.getSheet(sheetName);
		var startColNumber = websheet.ColumnHelper.translateFromColCharacters(startColumn);
		var endColNumber = websheet.ColumnHelper.translateFromColCharacters(endColumn);
		if (startColNumber > endColNumber) {
			var tColNumber = endColNumber;
			endColNumber = startColNumber;
			startColNumber = tColNumber;
		}
		if (startRow > endRow) {
			var tRow = endRow;
			endRow = startRow;
			startRow = tRow;
		}
		var cellObj = null;
		var ROA = null;
		for (var iRow = startRow; iRow <= endRow; iRow++) {
			var row = new Array();
			for (var iColNum = startColNumber; iColNum <= endColNumber; iColNum++) {
				var iCol = websheet.ColumnHelper.translateFromColNum(iColNum);
				cellObj = this.document.getCell(sheetName, iRow, iCol,true);
				ROA = new websheet.model.ROAModel(cellObj,sheetName,iRow,iCol,websheet.model.ROAType.BOTHRELATIVE,hasSheetRef);
				var cellValue = cellObj.getShowValue();
				this.pushCell(ROA);
				row.push(cellValue);
				var colLen = this.getCols().length;
				var colIndex = iColNum - startColNumber;
				if(colLen >= colIndex + 1){
					this.getCols()[colIndex].push(cellValue);
				}else{
					var cell = new Array();
					cell.push(cellValue);
					this.getCols().push(cell);
					delete cell;
				}
			}
			this.getRows().push(row);
			delete row;
		}
	}
});