dojo.require("websheet.model.CellModel");
dojo.require("websheet.Helper");

dojo.provide("websheet.model.RowModel");

dojo.declare("websheet.model.RowModel",null,{
	_id : null,
	
	index: null,					//show index in document
	 
	_parent: null,
	
	_document:null,
	
	cells: null,
	
	count: null,					//row count
	
	property: null, 				// sheet property, now for hide or show or filtered,
	
	height:null,
	
	_dependencedList:null,			//cells list which is dependent on cells in this sheet. 
	
	isDelete:null,
	
	repeatednum:null,
	
	styleCells:null,

	constructor: function(rowName,sheet,rowId,height,property,repeatednum) {
		this._id = rowId;
		this.index = rowName;
		this._parent = sheet;
		this._document = sheet._parent;
		this.cells = {};
		if(undefined != property)
		    this.property = property;
		this.count = 0;
		this._dependencedList = null;
		if(undefined != repeatednum)
			this.repeatednum = repeatednum;
		if(undefined != height)
			this.height = height;
	},
	
	oldLoad:function(row){
		if(row == undefined || row == null)return null;
		for(var column in row){
			if(this.cells[column]){
				this.cells[column].setRawValue(row[column].value,true);
			}else{
				this.createCell(column,row[column].value);
			}
			if(column != undefined && column != "_id"){
				this.count += 1;
			}
		}
	},
	
	loadFromEventJson: function(rowJson)
	{
		if(rowJson == undefined || rowJson == null)return null;
		this.count = 0;
		for(var colIndex in rowJson)
		{
			var cell = this.createCell(colIndex,rowJson[colIndex].value,rowJson[colIndex].showvalue,rowJson[colIndex].repeatednum);
			this.cells[colIndex] = cell;
			if( undefined != rowJson[colIndex].style)
			{
				var styleCode = new websheet.model.StyleCode("FROMJSON", rowJson[colIndex].style);
				cell.setStyle(styleCode);
			}
			this.count++;
		}
		if(this.styleCells)
		{
			this.getParent().addStyleRow(this);
		}
	},
	//load sheet json data to generate sheet object.
	loadRow: function(row){
		if(row == undefined || row == null)return null;
		var column = null;
		var cell = null;
		for(var colId in row){
			column = this.getParent().getColumnById(colId);
			if(column != undefined && column != null){
				cell = this.createCell(column.index,row[colId].value,row[colId].showvalue,row[colId].repeatednum,row[colId].commentsId,row[colId].colspan,row[colId].rowspan,row[colId].calculatedvalue,row[colId].text_align);
				this.cells[column.index] = cell;
				if(this._document.styleList && row[colId].styleid){
					var cellStyle = this._document.getStyle(row[colId].styleid);
					cell.setStyle(cellStyle);
					if(cellStyle){
						this.addStyleCell(cell);
					}
				}
			}
		}
		if(this.styleCells){
			this.getParent().addStyleRow(this);
		}
	}, 
	/*
	 * construct row json for event 
	 * null : if this row is empty
	 */
	toEventJson: function()
	{
	   var row = {};
	   var flag = 0;

	   if(this.height!= null)
	   {
	       row.height = this.height;
	       flag++;
	   }
	   if(this.property!= null)
	   {
	       row.property = this.property;
	       flag++;
	   }
	   if(this.repeatednum != null)
	   {
	       row.repeatednum = this.repeatednum;
	       flag++;
	   }
	   if(this.cells!= null)
	   {       
	       var cells = this.cells;
	       var iExistCell = false;
	       row.cells = {};
	       for(var cellIndex in cells)
	       {
	    	   var cellModel = cells[cellIndex];
	    	   var jsonCell = cellModel.toEventJson();
	    	   if( null != jsonCell)
	    	   {
	    		   row.cells[cellIndex] = jsonCell;
	    		   iExistCell = true;
	    	   }
	       }
	       if( iExistCell == false )
	       {
	    	   delete row.cells;
	       }
	       else
	       {
	    	   flag++;
	       }
	   }
	   if(flag > 0) return row;
	   else return null;
	},
	
	//if cell has formula,then parse it.
	_parse: function(){
		var doc = this.getParent().getParent();
		for(var column in this.cells){
			doc.clearMemory();
			doc.clearRecycle();
			doc.addMemory(this.cells[column]);
			if(!this.cells[column].calculateStatus){
				//this.cells[column]._parse();
				this.cells[column]._parse2();
			}
		}
	},
	
	//create a new cell in sheet.
	createCell: function(column, value,showValue,repeatednum,commentsId,colspan,rowspan,calculatedvalue,text_align) {
		if(value == undefined){value = "";}
		var cell = new websheet.model.CellModel(value,this.getParent().getColumn(column,true),this,showValue,repeatednum,commentsId,colspan,rowspan,calculatedvalue,text_align);
		this.cells[column] = cell;
		this.count += 1;
		return cell;		
	},
	
	//set parent node
	setParent:function(parent){
		this._parent = parent;
	},
	
	//get parent node
	getParent:function(){
		return this._parent;
	},
	
	setIndex:function(index){
		this.index = index;
	},
	
	getIndex:function(){
		return this.index;
	},
	
	/* todo later */
	setRowProperty: function(sheetProperty) {
	},
		
	addDependenceList:function(cell){
		var isLoad = false;
		dojo.forEach(this.getDependencedList(),function(item){
			if(item == cell){
				isLoad = true;
				return;
			}
		});
		if(!isLoad){
			this.getDependencedList().push(cell);
		}
	},
	
	getStyleCells:function(){
		if(this.styleCells == null){
			this.styleCells = {};
		}
		return this.styleCells;
	},
	
	getStyleCell:function(colIndex){
		var cCol = colIndex;
		if(websheet.Helper.isNumeric(colIndex)){
			cCol = websheet.ColumnHelper.translateFromColNum(colIndex);
		}
		return this.getStyleCells()[cCol];
	},
	
	resetStyleCell:function(cell){
		this.getStyleCells()[cell.getCol()] = cell;
		this.getParent().resetStyleRow(this);
	},
	
	addStyleCell:function(cell){
		if(!this.getStyleCells()[cell.getCol()]){
			this.getStyleCells()[cell.getCol()] = cell;
		}
		this.getParent().addStyleRow(this);
	},
	getRepeatedCell:function(colIndex){
		var num = colIndex;
		if(!websheet.Helper.isNumeric(colIndex)){
			num = websheet.ColumnHelper.translateFromColCharacters(colIndex);
		}
		var index = num;
		var temp;
		var cCell;
		cCell = this.getCell(colIndex);
		if(cCell && cCell.getStyle() != null){
			return cCell;
		}
		cCell = null;
		for(index;index>0;index--){
			temp = this.getStyleCell(websheet.ColumnHelper.translateFromColNum(index));
			if(index <= num){
				if(cCell){
					if(websheet.ColumnHelper.translateFromColCharacters(cCell.getCol()) < index){continue;}
				}
				var repeate = 0;
				if(temp){
					repeate = temp.repeatednum;
					if(!repeate){repeate = 0;}
				}
				if(temp && parseInt(index) + parseInt(repeate) >=num){
					cCell = temp;
					break;
				}
			}
		}
		return cCell;
	},
	
	clearStyleCell:function(){
		this.styleCells = null;
		delete this.styleCells;
	},
	
	resortStyleCell:function(){
		var styleCells = {};
		var cell;
		var temp;
		var repeate = 0;
		for(var attr in this.styleCells){
			cell = this.styleCells[attr];
			if(!cell.getColumn().isDelete){
				styleCells[cell.getCol()] = cell;
			}else{
				temp = this.styleCells[attr];
				repeate = temp.repeatednum;
				if(repeate != undefined &&  repeate>=0){
					var nCell = this.getCell(attr, true);
					nCell.repeatednum = repeate;
					nCell.setStyle(temp.getStyle());
					styleCells[attr] = nCell;
				}
				delete this.styleCells[attr];
			}
		}
		delete this.styleCells;
		this.styleCells = styleCells;
	},
	
	copy2Next:function(rowModel){
		if(!rowModel){return;}
		var cell;
		var temp;
		var repeate = 0;
		rowModel.repeatednum = this.repeatednum;
		for(var attr in this.styleCells){
			temp = this.styleCells[attr];
			if(temp){
				repeate = temp.repeatednum;
				if(!repeate){
					repeate = 0;
				}
			}
			cell = rowModel.getCell(attr,true);
			if(repeate != 0){
				cell.repeatednum = repeate;
			}
			if(temp.getStyle() != undefined){
				cell.setStyle(temp.getStyle());
				rowModel.addStyleCell(cell);
			}
		}
	},
	
	is2RowCanMerge: function(pRow, lRow)
	{
		if(null == pRow || null == lRow)
		{
			return false;
		}
		var pRowIndex = pRow.index;
		var pRepeateNum = (pRow.repeatednum == null || pRow.repeatednum == undefined) ? 0 : parseInt(pRow.repeatednum);
		var lRowIndex = lRow.index;
		if(pRowIndex + pRepeateNum + 1 != lRowIndex)
		{
			return false;
		}
		if(pRow.height == undefined)
		{
			pRow.height = null;
		}
		if(lRow.height == undefined)
		{
			lRow.height = null;
		}
		if(pRow.height != lRow.height)
		{
			return false;
		}
		if(pRow.isRowContainCellValue() || lRow.isRowContainCellValue())
		{
			return false;
		}
		return pRow.isEqual(lRow) ;
	},
	
	/*
	 * compare the cells in row, not including row height
	 * only compare the style cells
	 */
	isEqual: function(otherRow)
	{
		if(otherRow == null)
		{
			return false;
		}
		var pCells =  this.getStyleCells();//this.cells;
		var lCells = otherRow.getStyleCells() ;//otherRow.cells;
		pCells = ( pCells == undefined)? null : pCells;
		lCells = ( lCells == undefined)? null : lCells;
		if(pCells == null && lCells == null)
		{
			return true;
		}
		if( null == pCells || null == lCells)
		{
			return false;
		}
		var pColIndexs = new Array();
		for(var pColIndex in pCells)
		{
			pColIndexs.push(pColIndex);
		}
		var lColIndexs = new Array();
		for(var lColIndex in lCells)
		{
			lColIndexs.push(lColIndex);
		}
		if(pColIndexs.length != lColIndexs.length)
		{
			return false;
		}
		for(var ind in pColIndexs)
		{
			var colIndex = pColIndexs[ind];
			var pCell = pCells[colIndex];
			var lCell = lCells[colIndex];
			if(lCell == undefined || lCell == null)
			{
				return false;
			}
			var flag = pCell.isEqual(lCell);
			if(flag == false)
			{
				return false;
			}
		}
		return true;
	},
	
	isStyleRow: function()
	{
		if(null == this.cells || undefined == this.cells)
		{
			return false;
		}
		var isContainValue = false;
		var isContainStyle = false;
		var cells = this.cells;
		for(var colIndex in cells)
		{
			var cell = cells[colIndex];
			if( (cell._showValue != undefined && cell._showValue != null && cell._showValue.length > 0) 
			|| (cell._rawValue != undefined && cell._rawValue != null && cell._rawValue.length > 0))
			{
				isContainValue = true;
				return false;
			}
			if(cell.styleCode != null && cell.styleCode != undefined)
			{
				isContainStyle = true;
			}
		}
		return isContainStyle;
	},
	
	isRowContainCellValue: function()
	{
		if(null == this.cells || undefined == this.cells)
		{
			return false;
		}
		var cells = this.cells;
		for(var colIndex in cells)
		{
			var cell = cells[colIndex];
			if( (cell._showValue != undefined && cell._showValue != null && cell._showValue.length > 0) 
			|| (cell._rawValue != undefined && cell._rawValue != null && cell._rawValue.length > 0))
			{
				return true;
			}
		}
	},
	deleteStyleCell:function(cell){
		delete this.getStyleCells()[cell.getCol()];
	},
	
	getDependencedList:function(){
		if(!this._dependencedList){
			this._dependencedList = new Array();
		}
		return this._dependencedList;	
	},
	
	getCellRawValue: function(column) {
		column = column.toUpperCase();
		try{
			if(this.cells[column]){
				return this.cells[column].getRawValue();
			}
		}catch(e){
			return "";
		}
		return "";
	},
	
	clearCellStyle:function(){
		var styleCells = this.getStyleCells();
		for(var attr in this.getStyleCells()){
			styleCells[attr].setDefaultStyle(null);
			this.deleteStyleCell(cell);
		}
	},
	/* set cell raw value then will call the reference cells update,return the update rows*/
	setCellRawValue: function(column, value,notSend){
		if((!value || (dojo.isString(value) && value === '')) 
				&& (this.cells[column] && !this.cells[column].getStyle()) 
				&& (!this.cells[column]._dependencedList 
						||(this.cells[column]._dependencedList && this.cells[column]._dependencedList.length == 0))){
			return this.deleteCell(column,notSend);
		}
		if(value == null || value == undefined){value = "";}
		var rowUpdate;
		var c = this.cells[column];
		if(!c){
			c = this.createCell(column);
			//Modified by cdllibin@cn.ibm.com. To make "notSend" take effect.
			c.setRawValue(value,notSend);
			c._parse();
			websheet.RepeatHelper.splitRow(this.getParent(),this,c.getCol());
		}else{
			if(c.getRawValue() == value){
				return null;
			}else{
				var doc = this.getParent().getParent();
				doc.clearMemory();
				rowUpdate = c.setRawValue(value,notSend);
				websheet.RepeatHelper.splitRow(this.getParent(),this,c.getCol());
				if(value.length == 0)
				{
					websheet.RepeatHelper.mergeCell(this,c.getCol());
//					websheet.RepeatHelper.mergeRow(this.getParent(),this,c.getCol());
				}
				doc.clearMemory();
			}
		}
		return rowUpdate;
	},
	/* set cell comments id*/
	createCellComments:function( column, commentsId){
		var c = this.cells[column];
		if(!c){
			c = this.createCell(column, undefined);
			c.createCellComments(commentsId);
		}else{
			var doc = this.getParent().getParent();
			doc.clearMemory();
			c.createCellComments(commentsId);
			doc.clearMemory();
		}
	},
	/* delete cell comments by id*/
	delCellComments:function( column){
		var c = this.cells[column];
		if(c){
			c.delCellComments();
		}
	},
	/* set cell show value then will call the reference cells update,return the update rows*/
	setCellShowValue: function(column, value,notSend){
		if(value == null || value == undefined){value = "";}
		var c = this.cells[column];
		if(!c){
			c = this.createCell(column, value);
			c._parse();
		}else{
			c.setShowValue(value,notSend);
		}
	},
	
	getCellShowValue: function(column) {
		column = column.toUpperCase();
		if(this.cells[column]){
			return this.cells[column].getShowValue();
		}
		return "";
	},
	
	/* when the cell is empty,then delete it*/
	deleteCell:function(column,isCompel,notsend){
		var update = 0;
		var cell = this.cells[column];
		if(cell != null && cell != undefined){
			update = cell.setRawValue("",notsend);
			if(cell.getDependencedList().length == 0 || isCompel){
				var ref = cell.makeReference();
				delete this.cells[column];
				/*if(!notsend){
					websheet.event.eventFactory.addEventCell(ref,{});
					websheet.event.eventFactory.publishEvent(dojo.hitch(pe.base.getConnector(),"publishDeleteEvent"),dojo.hitch(websheet.event.eventFactory,"generateDeleteCellMsg"),dojo.hitch(websheet.event.eventFactory,"generateSetCellMsg"));
				}*/
			}
		}
		return update;
	},
	
	/* get cell object */
	getCell: function(column,isGeneral){
		column = column.toUpperCase();
		var cell = this.cells[column];
		if(cell == undefined || cell == null){
			if(isGeneral){this.createCell(column, "");}
			else{
				//throw "cell is null!!!";
				}
		}
		return this.cells[column];
	},

	getCellCount: function()
	{
		var count = 0;
		for(var colIndex in this.cells)
		{
			count++;
		}
		return count;
	},
	
	sortCell: function(cellA, cellB)
	{
		var char2Num = dojo.hitch(websheet.ColumnHelper,"translateFromColCharacters");
		var strIndexA = cellA.getColumn().getIndex();
		var strIndexB = cellB.getColumn().getIndex();
		var indexA = char2Num(strIndexA);
		var indexB = char2Num(strIndexB);
		return (indexA - indexB);
	},
	
	/*
	 * startIndex&endIndex: int
	 * return the cells between startIndex and endIndex in order
	 */
	getCellsInOrder: function(startIndex, endIndex)
	{
		var char2Num = dojo.hitch(websheet.ColumnHelper,"translateFromColCharacters");
		var num2Char = dojo.hitch(websheet.ColumnHelper,"translateFromColNum");
		var cellArray = new Array();
		for(var strColIndex in this.cells)
		{
			var colIndex = char2Num(strColIndex)
			if(colIndex >= startIndex && colIndex <= endIndex)
			{
				cellArray.push(this.cells[strColIndex]);
			}
		}
		cellArray.sort(this.sortCell);
		return cellArray;
	},
	
	setRowStyle:function(style){
		var char2Num = dojo.hitch(websheet.ColumnHelper,"translateFromColCharacters");
		var num2Char = dojo.hitch(websheet.ColumnHelper,"translateFromColNum");
		var cells;
		var len;
		var index = 1;
		var styleColumns = this.getParent().getStyleColumns();
		//apply style of column to cell in current row
		for(var iColumn in styleColumns)
		{
			var iColObj = styleColumns[iColumn];
			if(iColObj.getStyle())
			{
				var repeatednum = (!iColObj.repeatednum)? 0 : parseInt(iColObj.repeatednum);
				var startColIndex = char2Num(iColumn);
				var endColIndex = startColIndex + repeatednum;
				var cellsArray = this.getCellsInOrder(startColIndex,endColIndex);
				for(var ind = 0 ; ind < cellsArray.length; ind++ )
				{
					var cell = cellsArray[ind];
					var cellIndex = char2Num(cell.getColumn().getIndex());
					var cellRepeateNum = (!cell.repeatednum)? 0 : parseInt(cell.repeatednum);
					if(startColIndex < cellIndex)
					{
						var strStartColIndex = num2Char(startColIndex);
						var newCell = this.createCell(strStartColIndex,"","",cellIndex-startColIndex-1);
						this.cells[strStartColIndex] = cell; 
					}
					if(!cell.styleCode)
					{
						cell.setStyle(iColObj.getStyle());
					}
					startColIndex = cellIndex + cellRepeateNum + 1;
				}
				if(startColIndex < endColIndex)
				{
					var strStartColIndex = num2Char(startColIndex);
					var newCell = this.createCell(strStartColIndex,"","",endColIndex-startColIndex-1);
					this.cells[strStartColIndex] = cell; 
				}
			}
		}
/*		for(var iColumn in styleColumns)
		{
			var iColObj = styleColumns[iColumn];
			var cell = this.getCell(iColObj.getIndex());
			if(!cell)
			{
				cell = this.getRepeatedCell(iColObj.getIndex());
			}
			if(!cell || !cell.styleCode)
			{
				cell = this.getCell(iColObj.getIndex(), true);
				var repeatednum = (!iColObj.repeatednum)? 0 : parseInt(iColObj.repeatednum);
				cell.setStyle(iColObj.getStyle());
				var startColIndex = char2Num(iColumn);
				var endColIndex = startColIndex + repeatednum;
				while(startColIndex <= endColIndex)
				{
					var strTmpColIndex = num2Char(startColIndex);
					var tmpCell = this.getCell(strTmpColIndex);
				}
			}
//			var cell = this.getCell(iColObj.getIndex(), true);
//			cell.repeatednum = iColObj.repeatednum;
//			var curStyle = cell.getStyle();
//			if(curStyle){
//				curStyle.changeStyle(iColObj.getStyle())
//			}else{
//				cell.setStyle(iColObj.getStyle());
//			}
		}*/
		len = this.getCellCount();//this.count;
		cells = this.cells;
		//in the case row doesn't have cells
		if(len == 0){
			var cell = this.getCell(websheet.ColumnHelper.translateFromColNum(index),true);
			cell.setStyle(style);
			cell.repeatednum = 1000;
			this.addStyleCell(cell);
		//in the case row has cells that have style
		}else{
			var temp;
			while(len != 0){
				var col = num2Char(index);
				var cell = this.getCell(col);
//				len -= 1;
				if(cell){
					var curStyle = cell.getStyle();
					if(curStyle){
						curStyle.changeStyle(style);
					}else{
						cell.setStyle(style);
					}
					len -= 1;
					temp = cell;
					var repeate = cell.repeatednum;
					if(repeate && repeate > 0){
						index += repeate + 1;
					}else{
						delete cell.repeatednum;
						index++;
					}
					temp = null;
				}else{
					if(temp){
						var repeate = temp.repeatednum;
						if(!repeate){temp.repeatednum = 1;}
						else{temp.repeatednum += 1;}
					}else{
						cell = this.getCell(col, true);
						cell.setStyle(style);
						temp = cell;
						this.addStyleCell(cell);
					}
					index++;
				}
			}
			if(!temp){
				temp = this.getCell(num2Char(index), true);
			}
			var curStyle = temp.getStyle();
			if(curStyle == style){
				temp.repeatednum = 1000;
			}else{
				cell = this.getCell(num2Char(index), true);
				cell.setStyle(style);
				cell.repeatednum = 1000;
			}
		}
	},
	/* call the dependence cell to update */
	dependenceAutoUpdate: function(){
		dojo.forEach(this.getDependencedList(),function(cell){
			cell.referenceAutoUpdate(this.index,1);
		},this); 
	},
	
	reGenerateFormula:function(deleteType,notSend){
		dojo.forEach(this.getDependencedList(),function(item){
			item.reGenerateFormula(deleteType,notSend);
		},this);
	},
	
	getCellsValueInDependencyList: function()
	{
		var arraylist = this.getDependencedList();
		if(arraylist.length == 0)
		{
			return null;
		}
		var cells = {};
		for(var index in arraylist)
		{
			var cell = arraylist[index];
			var cellObj = {};
			var cellRef = cell.getCellRef();
			cellObj.value = cell._rawValue;
			cellObj.showvalue = cell._showValue;
			cells[cellRef] = cellObj; 
		}
		return cells;
	},
	
	deleteDependence:function(cell){
		var dependenceList = this.getDependencedList(); 
		if(dependenceList.length != 0){
			dojo.every(dependenceList,function(item,index){
				if(item == cell){
					dependenceList.splice(index,1);
					return;
				}
			},this);
		}	
	}
});
