dojo.require("websheet.model.RowModel");
dojo.require("websheet.Helper");
dojo.require("websheet.model.ColumnModel");
dojo.provide("websheet.model.SheetModel");

dojo.declare("websheet.model.SheetModel",null,{
	_id : null,
	
	_name: null,
	
	index: null,					//show index in document
	
	_parent: null,
	
	rows: null,
	
	columns: null,
	
	count: null,					//row count
	
	property: null, 				// sheet property, now for hide and show
	
	_dependencedList:null,			//cells list which is dependent on cells in this sheet. 
	
	styleColumns:null,				//column has style,but row don't have
	
	styleRows:null,
	
	isDelete:null,
	
	constructor: function(sheetId,sheetName,document) {
		this._id = sheetId;
		this._name = sheetName;
		this._parent = document;
		this.rows = {};
		this.columns = {};
		this.property = null;
	},
	
	oldLoad:function(sheet){
		if(sheet == undefined || sheet == null)return null;
		for(var row in sheet.rows){
			if(this.rows[row]){
				this.rows[row].loadRow(sheet.rows[row]);
			}else{
				this.createRow(row).loadRow(sheet.rows[row]);
			}
			if(row != undefined && row != "_id"){
				this.count += 1;
			}
		}
	},
	
	loadFromEventJson: function(sheetJson)
	{
		if(sheetJson == undefined || sheetJson == null)return null;
		for(var rowIndex in sheetJson.rows)
		{
			var rowJson = sheetJson.rows[rowIndex];
			var row = this.createRow(rowIndex,null,rowJson.height, rowJson.property, rowJson.repeatednum);
			row.loadFromEventJson(rowJson.cells);
			this.rows[rowIndex] = row;
		}
		for(var colIndex in sheetJson.columns)
		{
			var colJson = sheetJson.columns[colIndex];
			var column = this.createColumn(colIndex, null, colJson.width, colJson.property, colJson.repeatednum);
			if( undefined != colJson.style)
			{
				var styleCode = new websheet.model.StyleCode("FROMJSON", colJson.style);
				column.setStyle(styleCode);
			}
		}
	},
	
	//load sheet json data to generate sheet object.
	loadSheet: function(sheet,doc){
		if(sheet == undefined || sheet == null)return null;
		var sheetData = doc.content.sheets[this._id];
		var obj = {};
		obj.self = this;
		obj.document = doc;
		obj.sheet = sheetData;
		//load column data
		dojo.forEach(sheet.columnsIdArray,function(colId,index){
				if(colId && dojo.string.trim(colId) != ""){
					var colData;
					if (this.document.meta.columns != undefined)
					{
						colData = this.document.meta.columns[colId];
					}
					if(colData){
						var column = this.self.createColumn(parseInt(index) + 1, colId, colData.width, colData.property, colData.repeatednum);
						column.setStyle(this.self.getParent().getStyle(colData.styleid));
					}else{
						this.self.createColumn(parseInt(index) + 1, colId);
					}
				}
		},obj);
		
		//load row data
		dojo.forEach(sheet.rowsIdArray,function(rowId,index){
				if(rowId && dojo.string.trim(rowId) != ""){
					var rows = this.document.meta.rows;
					var row;
					if(this.sheet){
						if(rows && this.sheet.rows){
							var rowData = rows[rowId];
							if(rowData){
								row = this.self.createRow(parseInt(index) + 1, rowId, rowData.height, rowData.property, rowData.repeatednum);
								if(rowData.styleid){
									row.setStyle(this.self.getParent().getStyle(rowData.styleid));
								}
							}else{
								row = this.self.createRow(parseInt(index) + 1, rowId);
							}
						}else{
							row = this.self.createRow(parseInt(index) + 1, rowId);
						}
						row.loadRow(this.sheet.rows[rowId]);
						this.self.rows[parseInt(index) + 1] = row;
					}
				}
		},obj);
		
	},
	/*
	 * construct sheet json for event 
	 */
	toEventJson: function()
	{
	   var sheetObj = {};
	   if(this._name != null)
	   {
	      sheetObj.sheetname = this._name;
	   }
	   if(this.index != null)
	   {
	      sheetObj.sheetindex = this.index;
	   }
	   if(this.property != null)
	   {
	      sheetObj.property = this.property;
	   }
	   if(this.rows != null)
	   {
	      var rows = this.rows;
	      sheetObj.rows={};
	      var isExistRow = false;
	      for(var rowIndex in rows)
	      {
	    	  var rowModel = rows[rowIndex];
	    	  var rowJson = rowModel.toEventJson();
	    	  if(rowJson != null )
	    	  {
	    		  sheetObj.rows[rowIndex] = rowJson;
	    		  isExistRow = true;
	    	  }
	      }
	      if( isExistRow == false )
	      {
	    	  delete sheetObj.rows;
	      }
	   }
	   
	   if(this.columns != null)
	   {
	   	  var columns = this.columns;
	      sheetObj.columns = {};
	   	  var isExistColumn = false;
	   	  for(var colIndex in columns)
	   	  {
	   		 var colModel = columns[colIndex];
	   		 var colJson = colModel.toEventJson();
	   		 if(colJson != null)
	   		 {
	   			sheetObj.columns[colIndex] = colJson;
	   			isExistColumn = true;
	   		 } 
	   	  }
	   	  if( isExistColumn == false)
	   	  {
	   		  delete sheetObj.columns;
	   	  }
	   }
	   return sheetObj;
	},
	
	//if cell has formula,then parse it.
	_parse: function(){
		for(var row in this.rows){
			this.rows[row]._parse();
		}
	},
	
	//create a new row in sheet.
	createRow: function(rowNum,rowId,height,property,repeatednum) {
		var row = new websheet.model.RowModel(rowNum,this,rowId,height,property,repeatednum);
		this.rows[rowNum] = row;
		this.count +=1;
		return row;		
	},
	
	//create a new column in sheet.
	createColumn: function(colIndex,colId,width,property,repeatednum) {
		var colCharacter = colIndex;
		if(websheet.Helper.isNumeric(colIndex)){
			colCharacter = websheet.ColumnHelper.translateFromColNum(colIndex);
		}
		var column = new websheet.model.ColumnModel(colCharacter,this,colId,width,property,repeatednum);
		this.columns[colCharacter] = column;
		return column;		
	},
	
	getId:function(){return this._id;},
	//set sheet name
	setSheetName:function(name){
		this._name = name;
	},
	
	//get sheet name
	getSheetName:function(){
		return this._name;
	},
	
	//set parent node
	setParent:function(parent){
		this._parent = parent;
	},
	
	//get parent node
	getParent:function(){
		return this._parent;
	},
	
	/* todo later */
	setSheetProperty: function(sheetProperty) {
	},
		
	addDependenceList:function(cell){
		var isLoad = false;
		dojo.forEach(this.getDependencedList(),function(item){
			if(item == cell){
				isLoad = true;
				return;
			}
		});
		if(!isLoad){
			this.getDependencedList().push(cell);
		}
	},
	getDependencedList:function(){
		if(!this._dependencedList){
			this._dependencedList = new Array();
		}
		return this._dependencedList;	
	},
	
	getCellRawValue: function(row, column) {
		try{
			if(this.rows[row]){
				return this.rows[row].getCellRawValue(column);
			}
		}catch(e){
			return "";
		}
		return "";
	},
	
	/* set cell raw value then will call the reference cells update,return the update rows*/
	setCellRawValue: function(row, column, value,notSend){
		var r = this.rows[row];
		if(!r){
			r = this.createRow(row);
		}
		var updateRow =  r.setCellRawValue(column,value,notSend);
		if(value.length == 0)
		{
			websheet.RepeatHelper.mergeRow(this,r);
		}
		return this._parent.getRowUpdate();
	},
	/* set cell comments id*/
	createCellComments:function( row, column, commentsId){
		var r = this.rows[row];
		if(!r){
			r = this.createRow(row);
		}
		r.createCellComments( column, commentsId);
	},
	/* delete cell comments by id*/
	delCellComments:function( row, column, commentsId){
		var r = this.rows[row];
		r.delCellComments( column);
	},
	
	/* set cell show value then will call the reference cells update,return the update rows*/
	setCellShowValue: function(row, column, value,notSend){
		var r = this.rows[row];
		if(!r){
			r = this.createRow(row);
		}
		return r.setCellShowValue(column,value,notSend);
	},
	
	getCellShowValue: function(row, column) {
		try{
			if(this.rows[row]){
				return this.rows[row].getCellShowValue(column);
			}
		}catch(e){
			return "";
		}
		return "";
	},
	
	/* when the cell is empty,then delete it*/
	deleteCell:function(row, column){
		for(i in this.getParent().commentsList){
			var index = this.getParent().commentsList[i];
			if(index[0]==row&&index[0]==column){
				this.getParent().commentsList[index] = undefined;
				this.delCellComments(row, column, index);
			}
				
		}
		var r = this.rows[row];
		if(r){
			return r.deleteCell(column);
		}
		return 0;
	},
	
	/* get cell object */
	getCell: function(row, column,isGeneral){
		var rowObj = this.rows[row];
		if(rowObj == undefined || rowObj == null){
			if(isGeneral){rowObj = this.createRow(row);}
			else{
				return null;
				}
		}
		return rowObj.getCell(column,isGeneral);
	},
	
	/* rename sheet */
	/* success -- return true, failed -- return false */
	renameSheet: function(newSheetName,notSend) {
		this._name = newSheetName;
		return this.dependenceAutoUpdate(notSend);
	},
	insertRowWithContent: function(rowIndex, rowJson)
	{
		//step 1: split the row
		var rowModel = this.getRow(rowIndex,true);
		websheet.RepeatHelper.splitRow(this,rowModel);
		//step 2: move the rowindex
	    var rows = {};
		for(var row in this.rows)
		{
			if(row >= rowIndex)
			{
				this.rows[row].index = parseInt(row) + 1;
				rows[parseInt(row) + 1] = this.rows[row];
			}else
			{
				rows[row] = this.rows[row];
			}
			delete this.rows[row];
		}
		delete this.rows;
		this.rows = rows;
		//step 3: create the insert row
		var curRowModel = this.createRow(rowIndex,null,rowJson.height, rowJson.property, rowJson.repeatednum);
		curRowModel.loadFromEventJson(rowJson.cells);
		this.rows[rowIndex] = curRowModel;
		
		var otherCells = rowJson.otherCells;

		if(undefined != otherCells && null != otherCells)
		{
			for(var cellRef in otherCells)
			{
				var cellObj = otherCells[cellRef];
				var parsedRef = websheet.Helper.parseRef(cellRef);
				var cell = this.getCell(parsedRef.row,parsedRef.column,true);
				cell.setRawValue(cellObj.value);
				cell.setShowValue(cellObj.value);
			}
		}

	    this.count += 1;
		var regenerateCells = new dojox.collections.Stack();
		for(var row in this.rows){
			if(row >= rowIndex){
				dojo.forEach(this.rows[row].getDependencedList(),function(item){
					if(!regenerateCells.contains(item)){
						regenerateCells.push(item);
					}
				},regenerateCells);
			}
		}
		dojo.forEach(regenerateCells.toArray(),function(item){
			item.reGenerateFormula(0);
		});
		
		this.resortStyleRow();
		
		//step 4: merge row
		if(curRowModel.isStyleRow())
		{
			this.addStyleRow(curRowModel);
			websheet.RepeatHelper.mergeRow(this,curRowModel);
		}
	},
	
	/* insert a row with the row index, the first row with index 1 */
	/* the cell reference should be adjusted accordingly */
	/* row index start from 1 */
	insertRow: function(rowIndex,notSend) {
		var repeatRow = this.getRepeatedRow(rowIndex-1);
		
				//insert a row,adjust rows index
		var rows = {};
		for(var row in this.rows){
			if(row >= rowIndex){
				this.rows[row].index = parseInt(row) + 1;
				rows[parseInt(row) + 1] = this.rows[row];
			}else{
				rows[row] = this.rows[row];
			}
			delete this.rows[row];
		}
		delete this.rows;
		this.rows = rows;
		this.resortStyleRow();
		
		if(repeatRow){
			var repeate =  repeatRow.repeatednum ;
			if(repeate == undefined){
				repeate = 0;
			}
			if(repeatRow.isRowContainCellValue())
			{
				var nextRow = this.getRow(rowIndex ,true);
				repeatRow.copy2Next(nextRow);
				delete repeatRow.repeatednum;
				delete nextRow.repeatednum;
				this.addStyleRow(nextRow);
			}
			else
			{
				repeatRow.repeatednum = parseInt(repeate) + 1;
			}
		}

		this.count += 1;
		var regenerateCells = new dojox.collections.Stack();
		for(var row in this.rows){
			if(row >= rowIndex){
				dojo.forEach(this.rows[row].getDependencedList(),function(item){
					if(!regenerateCells.contains(item)){
						regenerateCells.push(item);
					}
				},regenerateCells);
			}
		}
		dojo.forEach(regenerateCells.toArray(),function(item){
			item.reGenerateFormula(0,notSend);
		});
		
	},	
	
	/* delete a row with the row index, the first row with index 0 */
	/* the cell reference should be adjusted accordingly */
	/* row index start from 1 */
	deleteRow: function(rowIndex,notSend,notReGenerate) {
//		for(i in this.getParent().commentsList){
//			var index = this.getParent().commentsList[i];
//			if(index[1]==rowIndex)
//				this.getParent().commentsList[i] = undefined;
//				this.delCellComments(row, index[2], index);
//		}
		var repeatRow = this.getRepeatedRow(rowIndex);
		if(repeatRow && repeatRow.repeatednum != undefined){
			repeatRow.repeatednum -= 1;
		}
		//delete a row,adjust rows index
		var rows = {};
		var temp;
		for(var row in this.rows){
			if(row == rowIndex){
				this.rows[row].isDelete = true;
				temp = this.rows[row];
			}
			else if(row > rowIndex){
				this.rows[row].index = parseInt(row) - 1;
				rows[parseInt(row) - 1] = this.rows[row];
			}
			else{
				rows[row] = this.rows[row];
			}
			delete this.rows[row];
		}
		delete this.rows;
		this.rows = rows;
		this.count -= 1;
		if(!notReGenerate){
			if(temp){
				temp.reGenerateFormula(0);
			}
			var regenerateCells = new dojox.collections.Stack();
			for(var row in this.rows){
				if(row >= rowIndex){
					dojo.forEach(this.rows[row].getDependencedList(),function(item){
						if(!regenerateCells.contains(item)){
							regenerateCells.push(item);
						}
					},regenerateCells);
				}
			}
			dojo.forEach(regenerateCells.toArray(),function(item){
				item.reGenerateFormula(0,notSend);
				if(item.isIgnoreRecursive){
					item.referenceAutoUpdate(item.getCellRef(),0,false);
				}
			});
		}
		this.resortStyleRow();
//		var pRowIndex = rowIndex -1;
//		if(pRowIndex > 0)
//		{
//			var pRowModel = this.getRowModel(pRowIndex);
//			if(null != pRowModel && pRowModel.isStyleRow())
//			{
//				websheet.RepeatHelper.mergeRow(this,pRowModel);
//			}
//			
//		}
	},
	
	insertColumnWithContent: function(colIndex,colJson)
	{
		//insert a column,adjust column index
		var colChar = colIndex;
		var colNum = colIndex;
		var num2Char = dojo.hitch(websheet.ColumnHelper,"translateFromColNum");
		var char2Num = dojo.hitch(websheet.ColumnHelper,"translateFromColCharacters");
		if(websheet.Helper.isNumeric(colIndex)){
			colChar = num2Char(colIndex);
		}else
		{
			colNum = char2Num(colIndex);
		}
		//step 1 : split col
		var colModel = this.getColumnModel(colIndex);
//		websheet.RepeatHelper.splitColumn(this,colModel,colIndex);
		
		var oldStyleRows = {};
		for(var rowIndex in this.styleRows)
		{
			oldStyleRows[rowIndex] = this.styleRows[rowIndex];
		}
		//step 2 : change the cell position in row
		for(var rowIndex in this.rows)
		{
			//update the repeate number of repeate cell 
			var rowModel = this.rows[rowIndex];
			var repeateCell = rowModel.getRepeatedCell(colIndex);
			if(repeateCell)
			{
				var repeate =  repeateCell.repeatednum ;
				if(repeate)
				{
//					var tmpCellModel = this.getCell(rowIndex,colChar,true);
//					websheet.RepeatHelper.splitCell(rowModel,tmpCellModel);
                    websheet.RepeatHelper.divideCell(rowModel,colChar);
				}
			}
			//update the model	
			var cells = {};
			for(var col in rowModel.cells)
			{
				var curColNum = char2Num(col);
				if(curColNum >= colNum)
				{
					var nextColChar = num2Char(parseInt(curColNum) + 1);
					cells[nextColChar] = rowModel.cells[col];
				}else
				{
					cells[col] = rowModel.cells[col];
				}
				delete rowModel.cells[col];
			}
			rowModel.cells = cells;
//			rowModel.resortStyleCell();
		}
		delete  this.styleRows;
        this.styleRows = oldStyleRows;
        
		var columns = {};
		//split col and change the column model's position
		for(var col in this.columns)
		{
			websheet.RepeatHelper.divideColumn(this,col);
			var curColNum = char2Num(col);
			var nextColChar = num2Char((parseInt(curColNum) + 1));
			if(curColNum >= colNum)
			{
				this.columns[col].index = nextColChar;
				columns[nextColChar] = this.columns[col];
			}else
			{
				columns[col] = this.columns[col];
			}
		}
		this.columns = columns;
		this.resortStyleColumn();
		// step 4:  insert the curColumn model
		var curColModel = this.createColumn(colIndex, null, colJson.width, colJson.property);
		if(undefined != colJson.style)
		{
			var styleCode = new websheet.model.StyleCode("FROMJSON", colJson.style);
			curColModel.setStyle(styleCode);
		}
		this.columns[colIndex] = curColModel;
		//step 5: insert the cells in this column
		if(undefined != colJson.cells)
		{
			for(var rowIndex in colJson.cells)
			{
				var cellJson = colJson.cells[rowIndex];
				var rowModel = this.getRow(rowIndex,true);
				
				websheet.RepeatHelper.splitRow(this,rowModel,colIndex);
//				websheet.RepeatHelper.splitRow(this,rowModel);
				var cellModel = rowModel.createCell(colIndex,cellJson.value,cellJson.showvalue);
//				websheet.RepeatHelper.splitCell(rowModel,cellModel);
				rowModel.resortStyleCell();
			    if( undefined != cellJson.style)
			    {
					var styleCode = new websheet.model.StyleCode("FROMJSON", cellJson.style);
					cellModel.setStyle(styleCode);
					rowModel.resetStyleCell(cellModel);
			    }
				rowModel.cells[colIndex] = cellModel;
				if(rowModel.styleCells)
				{
					this.addStyleRow(rowModel);
				}
				if(undefined == cellJson.value && undefined == cellJson.showvalue)
				{
					websheet.RepeatHelper.mergeCell(rowModel,colIndex);
					websheet.RepeatHelper.mergeRow(this,rowModel);
				}
			}
		}
		//Step 6: other cells which depend on the insert column
		var otherCells = colJson.otherCells;
		if(undefined != otherCells && null != otherCells)
		{
			for(var cellRef in otherCells)
			{
				var cellObj = otherCells[cellRef];
				var parsedRef = websheet.Helper.parseRef(cellRef);
				var cell = this.getCell(parsedRef.row,parsedRef.column,true);
				cell.setRawValue(cellObj.value);
				cell.setShowValue(cellObj.value);
			}
		}
		//update the cell reference
		var regenerateCells = new dojox.collections.Stack();
		for(var col in this.columns){
			if( websheet.ColumnHelper.translateFromColCharacters(col) >= colNum){
				dojo.forEach(this.columns[col].getDependencedList(),function(item){
					if(!regenerateCells.contains(item)){
						regenerateCells.push(item);
					}
				},regenerateCells);
			}
		}
		dojo.forEach(regenerateCells.toArray(),function(item){
			item.reGenerateFormula(1);
			if(item.isIgnoreRecursive){
				item.referenceAutoUpdate(item.getCellRef(),1,false);
			}
		});
//		this.resortStyleColumn();
	},
	/* insert a column with the column index, the first column with index 0 */
	/* the cell reference should be adjusted accordingly */
	/* column index start from 1 */
	insertColumn: function(colIndex,notSend) {
		//insert a column,adjust column index
		var colChar = colIndex;
		var colNum = colIndex;
		var num2Char = dojo.hitch(websheet.ColumnHelper,"translateFromColNum");
		var char2Num = dojo.hitch(websheet.ColumnHelper,"translateFromColCharacters");
		if(websheet.Helper.isNumeric(colIndex)){
			colChar = num2Char(colIndex);
		}else{
			colNum = char2Num(colIndex);
		}
		var repeatCol = this.getRepeatedColumn(colNum - 1);
		if(repeatCol){
			var repeate =  repeatCol.repeatednum ;
			if(repeate == undefined || repeate == 0){
				repeate = 0;
			}
			repeatCol.repeatednum = parseInt(repeate) + 1;
		}
		//change the cell position in row
		for(var row in this.rows)
		{
			//update the repeate number of repeate cell 
			var repeateCell = this.getRow(row).getRepeatedCell(num2Char(parseInt(colNum) -1));
			//update the model	
			var cells = {};
			for(var col in this.rows[row].cells){
				var curColNum = char2Num(col);
				if(curColNum >= colNum){
					var nextColChar = num2Char(parseInt(curColNum) + 1);
					cells[nextColChar] = this.rows[row].cells[col];
				}else{
					cells[col] = this.rows[row].cells[col];
				}
				delete this.rows[row].cells[col];
			}
			this.rows[row].cells = cells;
			
			if(repeateCell)
			{
				if(repeateCell._showValue !="" || repeateCell._rawValue != "")
				{
					delete  repeateCell.repeatednum;
					var nextCell = this.getCell(row, colChar,true);
					repeateCell.copy2Next(nextCell);
					this.rows[row].addStyleCell(nextCell);
				}
				else
				{
					var repeate =  repeateCell.repeatednum ;
					if(repeate == undefined)
					{
						repeate = 0;
					}
					repeateCell.repeatednum = parseInt(repeate) + 1;
				}
			}
		}
		var columns = {};
		//change the column model's position
		for(var col in this.columns){
			var curColNum = char2Num(col);
			var nextColChar = num2Char((parseInt(curColNum) + 1));
			if(curColNum >= colNum){
				this.columns[col].index = nextColChar;
				columns[nextColChar] = this.columns[col];
			}else{
				columns[col] = this.columns[col];
			}
		}
		this.columns = columns;
		//update the cell reference
		var regenerateCells = new dojox.collections.Stack();
		for(var col in this.columns){
			if( websheet.ColumnHelper.translateFromColCharacters(col) >= colNum){
				dojo.forEach(this.columns[col].getDependencedList(),function(item){
					if(!regenerateCells.contains(item)){
						regenerateCells.push(item);
					}
				},regenerateCells);
			}
		}
		dojo.forEach(regenerateCells.toArray(),function(item){
			item.reGenerateFormula(1,notSend);
			if(item.isIgnoreRecursive){
				item.referenceAutoUpdate(item.getCellRef(),1,false);
			}
		});
		this.resortStyleColumn();
	},	
	
	/* delete a column with the column index, the first column with index 0 */
	/* the cell reference should be adjusted accordingly */
	/* column index start from 1 */
	deleteColumn: function(colIndex,notSend) {
		//insert a column,adjust column index
		var colChar = colIndex;
		var colNum = colIndex;
		var num2Char = dojo.hitch(websheet.ColumnHelper,"translateFromColNum");
		var char2Num = dojo.hitch(websheet.ColumnHelper,"translateFromColCharacters");
//		for(i in this.getParent().commentsList){
//			var index = this.getParent().commentsList[i];
//			if(index[2]==colChar)
//				this.getParent().commentsList[i] = undefined;
//				this.delCellComments(index[1], index[2], index);
//		}
		if(websheet.Helper.isNumeric(colIndex)){
			colChar = num2Char(colIndex);
		}else{
			colNum = char2Num(colIndex);
		}
		//whether the deleted column is in the repeat column
		var repeatCol = this.getRepeatedColumn(colChar);
		var repeate = 0;
		if(repeatCol){
			repeate =  repeatCol.repeatednum ;
			if(repeate != undefined){
				repeatCol.repeatednum -= 1;
			}
		}
		//change the cell position in row
		var temp = new Array();
		var repeateCell;
		for(var row in this.rows){
			//update the repeate number of repeate cell 
			repeateCell = this.getRow(row).getRepeatedCell(colChar);
			if(repeateCell){
				repeate =  repeateCell.repeatednum ;
				if(repeate != undefined){
					repeateCell.repeatednum -= 1;
				}
			}
			//update the model
			var cells = {};
			for(var col in this.rows[row].cells){
				var curColNum = char2Num(col);
				if(curColNum > colNum){
					var preColChar = num2Char(parseInt(curColNum) - 1);
					cells[preColChar] = this.rows[row].cells[col];
				}else if(curColNum ==  colNum){
					delete this.rows[row].cells[col];
				}else{
					cells[col] = this.rows[row].cells[col];
				}
				delete this.rows[row].cells[col];
			}
			this.rows[row].cells = cells;
		}
		var deleteCol;
		var columns = {};
		//change the column model's position
		for(var col in this.columns){
			var curColNum = char2Num(col);
			var preColChar = num2Char((parseInt(curColNum) - 1));
			if(curColNum > colNum){
				this.columns[col].index = preColChar;
				columns[preColChar] = this.columns[col];
			}else if(curColNum == colNum){
				this.columns[col].isDelete = true;
				deleteCol = this.columns[col];
				delete this.columns[col];
			}else{
				columns[col] = this.columns[col];
			}
		}
		this.columns = columns;
		if(deleteCol){
			deleteCol.reGenerateFormula(1,notSend);
		}
		//update the cell reference
		var regenerateCells = new dojox.collections.Stack();
		for(var col in this.columns){
			if( char2Num(col) >= colNum){
				dojo.forEach(this.columns[col].getDependencedList(),function(item){
					if(!regenerateCells.contains(item)){
						regenerateCells.push(item);
					}
				},regenerateCells);
			}
		}
		dojo.forEach(regenerateCells.toArray(),function(item){
			item.reGenerateFormula(1,notSend);
		});
		this.resortStyleColumn();
	},
	
	getColumn:function(colIndex,isGenerate){
		var colCharacter = colIndex;
		if(websheet.Helper.isNumeric(colIndex)){
			colCharacter = websheet.ColumnHelper.translateFromColNum(colIndex);
		}
		if(!this.columns[colCharacter] && isGenerate){
			this.createColumn(colCharacter);
		}
		return this.columns[colCharacter];
	},
	
	getColumnById:function(colId){
		for(var column in this.columns){
			if(this.columns[column]._id == colId){
				return this.columns[column];
			}
		}
	},
	
	/*
	 * if the rowIndex in the repeated number, return the node
	 * if the row independently exisiting, return the row
	 * null if not exist
	 */
	getRowModel: function(rowIndex)
	{
		var rowModel = null;
		rowModel = this.getRepeatedRow(rowIndex);
		if(null == rowModel || undefined == rowModel)
		{
			rowModel = this.getRow(rowIndex);
		}
		rowModel = (undefined == rowModel)? null: rowModel;
		return rowModel;
	},
	
	/*
	 * if the colIndex in the repeated number, return the node
	 * if the column independently exisiting, return the column
	 * null if not exist
	 */
	getColumnModel: function(colIndex)
	{
		var colModel = null;
		colModel = this.getRepeatedColumn(colIndex);
		if(null == colModel || undefined == colModel)
		{
			colModel = this.getColumn(colIndex);
		}
		colModel = (undefined == colModel)? null: colModel;
		return colModel;
	},
	/* get row by index,row index start from 1 */
	getRow:function(rowIndex,isGenerate){
		var row =  this.rows[rowIndex];
		if(!row && isGenerate){
			this.createRow(rowIndex);
		}
		return this.rows[rowIndex];
	},
	
	getRowById:function(rowId){
		for(var row in this.rows){
			if(this.rows[row]._id == rowId){
				return this.rows[row];
			}
		}
	},
	
	getIndex:function(){
		return this.index;
	},
		
	setIndex:function(index){
		this.index = index;
	},	
	/* auto update all the row*/
	_autoUpdate:function(){
		
	},	

	/* apply cell style */
	applyCellStyle: function(row, column, styleIndex) {
	},

	/* apply cell format */
	applyCellFormat: function(row, column, formatIndex) {
	},
	
	getStyleColumns:function(){
		if(this.styleColumns == null){
			this.styleColumns = {};
		}
		return this.styleColumns;
	},
	
	getStyleColumn:function(colIndex){
		var cCol = colIndex;
		if(websheet.Helper.isNumeric(colIndex)){
			cCol = websheet.ColumnHelper.translateFromColNum(colIndex);
		}

		return this.getStyleColumns()[cCol];
	},
	getRepeatedColumn:function(colIndex){
		var num = colIndex;
		if(!websheet.Helper.isNumeric(colIndex)){
			num = websheet.ColumnHelper.translateFromColCharacters(colIndex);
		}
		var column;
		var index = num;
		var temp;
		var cCol;
		var repeate = 0;
		for(index;index>0;index--){
			temp = this.getStyleColumn(index);
			if(index <= num){
				if(column){
					if(websheet.ColumnHelper.translateFromColCharacters(column.getIndex()) < index){continue;}
				}
				if(temp){
					repeate = temp.repeatednum;
					if(!repeate){repeate = 0;}
				}
				if(temp && parseInt(index) + parseInt(repeate) >=num){
					column = temp;
					break;
				}
			}
		}
		return column;
	},
	
	addStyleColumn:function(column){
		if(!this.getStyleColumns()[column.getIndex()]){
			this.getStyleColumns()[column.getIndex()] = column;
		}
	},
	
	clearStyleColumn:function(){
		this.styleColumns = null;
		delete this.styleColumns;
	},
	
	resortStyleColumnOnly: function()
	{
		var styleColumn = {};
		var column;
		var row;
		var temp;
		var deleteIndex;
		for(var attr in this.styleColumns)
		{
			column = this.getStyleColumn(attr);
			if(!column.isDelete)
			{
				styleColumn[column.getIndex()] = column;
			}else
			{
				deleteIndex = column.getIndex();
				if(column.repeatednum >= 0)
				{
					temp = this.getColumn(deleteIndex,true);
					temp.repeatednum = column.repeatednum;
					temp.setStyle(column.getStyle());
					styleColumn[column.getIndex()] = temp;
				}
			}
		}
		delete this.styleColumns;
		this.styleColumns = styleColumn;
	},
	resortStyleColumn:function(){
		var styleColumn = {};
		var column;
		var row;
		var temp;
		var deleteIndex;
		for(var attr in this.styleColumns){
			column = this.getStyleColumn(attr);
			if(!column.isDelete){
				styleColumn[column.getIndex()] = column;
			}else{
				deleteIndex = column.getIndex();
				if(column.repeatednum >= 0){
					temp = this.getColumn(deleteIndex,true);
					temp.repeatednum = column.repeatednum;
					if(column.repeatednum != undefined && column.repeatednum != null && column.repeatednum >= 0){
						temp.setStyle(column.getStyle());
					}
					styleColumn[column.getIndex()] = temp;
				}
			}
		}
		delete this.styleColumns;
		this.styleColumns = styleColumn;
		for(var attr in this.styleRows){
			row = this.styleRows[attr];
			row.resortStyleCell();
		}
	},
	
	deleteStyleColumn:function(column){
		delete this.getStyleColumns()[column.getIndex()]
	},
	
	getStyleRows:function(){
		if(this.styleRows == null){
			this.styleRows = {};
		}
		return this.styleRows;
	},
	
	resortStyleRow:function(){
		var styleRow = {};
		var row;
		var temp;
		for(var attr in this.styleRows){
			row = this.styleRows[attr];
			if(!row.isDelete){
				styleRow[row.getIndex()] = row;
			}else{
				var repeate = row.repeatednum;
				if(repeate != undefined &&  repeate>=0){
					temp = this.getRow(row.getIndex(),true);
					row.copy2Next(temp);
					styleRow[row.getIndex()] = temp;
				}
			}
		}
		delete this.styleRows;
		this.styleRows = styleRow;
	},
	
	getStyleRow:function(rowIndex){
		return this.getStyleRows()[rowIndex];
	},
	
	getRepeatedRow:function(rowIndex){
		var row;
		var index = rowIndex;
		var temp;
		var repeate = 0;
		for(index;index>0;index--){
			temp = this.getStyleRow(index);
			if(temp){
				repeate = temp.repeatednum;
				if(!repeate){repeate = 0;}
			}
			if(temp && parseInt(temp.getIndex()) + parseInt(repeate) >=rowIndex){
				if(row){
					if(temp.getIndex() > row.getIndex()){
						row = temp;
					}
				}else{
					row = temp;
					break;
				}
			}
		}
		return row;
	},
	
	resetStyleRow: function(row)
	{
		var index = row.getIndex();
		this.getStyleRows()[index] = row;
	},
	
	addStyleRow:function(row){
		var index = row.getIndex();
		if(!this.getStyleRows()[index]){
			this.getStyleRows()[index] = row;
		}
	},
	
	clearStyleRow:function(){
		this.styleRows = null;
		delete this.styleRows;
	},
	
	deleteStyleRow:function(row){
		delete this.getStyleRows()[row.getIndex()];
	},
	deleteDependence:function(cell){
		var dependenceList = this.getDependencedList(); 
		if(dependenceList.length != 0){
			dojo.every(dependenceList,function(item,index){
				if(item == cell){
					dependenceList.splice(index,1);
					return;
				}
			},this);
		}	
	},
	
	getCellsValueInDependencyList: function()
	{
		var arraylist = this.getDependencedList();
		var cells = {};
		for(var index in arraylist)
		{
			var cell = arraylist[index];
			var cellObj = {};
			var cellRef = cell.getCellRef();
			cellObj.value = cell._rawValue;
			cellObj.showvalue = cell._showValue;
			cells[cellRef] = cellObj; 
		}
		return cells;
	},
	/* call the dependence cell to update */
	dependenceAutoUpdate: function(notSend){
		dojo.forEach(this.getDependencedList(),function(cell){
			if(cell){
				cell.referenceAutoUpdate(this._ref,1);
				cell.reGenerateFormula(2,notSend);
				this.getParent().addRowUpdate(cell.getRow());
			}
		},this); 
		return this.getParent().getRowUpdate();
	}
});