/************************************************************************************/
/* author ruisuy@cn.ibm.com															*/
/* Spreadsheet Undo manager(single user mode only, in TP2							*/
/************************************************************************************/
dojo.provide("websheet.model.SheetUndoManager");

dojo.require("websheet.CometConnector");
dojo.require("websheet.event.eventFactory");

dojo.declare("websheet.model.SheetUndoManager",null, {
	_current: -1,
	_undoredoList:null,
	_MaxUndoSize: 10,
	_isON:true,
	
	_applier: null,
	_base: null,
	
	UNDO:1,
	REDO:2,
	
	constructor: function(_connector,base){
		this._current = -1;
		this._undoredoList = [];
		this._applier = _connector;
		this._base = base;
	},
	
	AddUndoAction: function(undoAction) {
		// for single user mode use only
		if ((LB_COM.autosaveEnable == false)||(this._isON == false))
			return false;
		
		if (this._undoredoList == null)
		{
			// init the undo/redo list
			this._undoredoList = [];
		}
		if ((undoAction != undefined)&&(undoAction != null))
		{
			//Drop event later than _current;
			var  listLenth = this._undoredoList.length;
			for (var index=this._current+1; index<listLenth; index++)
			{
				var tmpEvent = this._undoredoList.pop(index+1);
				delete tmpEvent;
			}
			
			// Meet max size, drop earlist event
			if (this._undoredoList.length == this._MaxUndoSize)
			{
				var tmpEvent = this._undoredoList.shift();
				delete tmpEvent;
				
				this._current--;
			}
			
			// push the new event into the task
			this._undoredoList.push(undoAction);
			this._current++;
			
			return true;
		}
	},

	Undo: function() {
		if (this.hasUndo())
		{
			var undoEvent = websheet.event.eventFactory.copyMsg(this._undoredoList[this._current]);
			this._current--;
			
			// apply event to model
			this._applyEvent(undoEvent, this.UNDO);
			if(undoEvent.reverse.events.length == 1 && undoEvent.reverse.events[0].reference.refType == "cell")
			{
				this.FocusCell(undoEvent.reverse.events[0].reference.refValue);
			}
			delete undoEvent;
		}
	},
	
	Redo: function() {
		if ( this.hasRedo()&&this._isON )
		{
			var redoEvent = websheet.event.eventFactory.copyMsg(this._undoredoList[this._current+1]);
			this._current++;
			
			// apply event to model
			this._applyEvent(redoEvent, this.REDO);
			if(redoEvent.events.length == 1 && redoEvent.events[0].reference.refType == "cell")
			{
				this.FocusCell(redoEvent.events[0].reference.refValue);
			}
		}
	},
	
	FocusCell: function(refValue)
	{
		if(this._applier!=null&&this._isON)
		{
			this._applier.handleFocusCell(refValue);
		}
	},
	// apply event to model, send out undo/redo action message
	_applyEvent: function (event, type) {
		if ( this._applier!=null&&this._isON )
		{
			switch (type)
			{
				case this.UNDO:
//					for (var index = 0; index< event.reverse.events.length; index++)
//					{
//						this._applier.applyEvent(event.reverse.events[index], "UNDO", true);
//					}
					this._applier.applyUndoEvents(event.reverse.events);
					break;
				case this.REDO:
					for (var index = 0; index< event.events.length; index++)
					{
						this._applier.applyEvent(event.events[index], "REDO", true);
					}
					break;
			}
			var sentEvent = null;
			if (type == this.UNDO)
			{
				sentEvent = websheet.event.eventFactory.copyMsgwithReverseasEvent(event);
			}
			else if (type == this.REDO)
			{
				sentEvent = websheet.event.eventFactory.copyMsg(event);
			}
			this._applier._publishbyMessageSender(sentEvent);
		}
	},
	
	hasUndo: function() {
		if ((this._current!=-1)&&(this._undoredoList[this._current]!=undefined)
				&&(this._undoredoList[this._current]!=null))
			return true;
		else return false;
	},
	
	hasRedo: function() {
		if ((this._undoredoList[this._current+1]!=undefined)
						&&(this._undoredoList[this._current+1]!=null))
			return true;
		else return false;
	},
	
	Reset: function (){ 
		var tmpEvent = null;
		var length = this._undoredoList.length;
		for (var index = 0; index<length; index++)
		{
			tmpEvent = this._undoredoList.pop();
			delete tmpEvent;
		}
		this._current = -1;
	},

	turnOff: function(){
		this._isON = false;
	},
	
	turnOn: function(){
		this._isON = true;
	}
});


