dojo.provide("websheet.parse.parseHelper");
dojo.require("websheet.ColumnHelper");
//dojo.require("websheet.model.RangeModel");
dojo.require("dojox.collections.Dictionary");
websheet.parse.parseHelper = {
	/**
	 * parse formula in cell
	 * @param {websheet.model.CellModel}cell	cell model
	 * @return {String}ret.value 				cell show value
	 */	
		
	NAERROR: 0,
	UNSUPPORTEDFORMULA: 100,
	DEFAULTERROR:500,
	currentParserSheetName : null,
	
	
	_parse: function(cell) {
		var ret = null;
		var showValue = {};
		try {
//			cell._rawValue = "=sum(Sheet1.A2:D2)+sum(Sheet1.C3:E3)";
			cell.getDocument().clearMemory();
			var tokenList = cell.getTokenList();
			var input = new org.antlr.runtime.ANTLRStringStream(cell.getRawValue()), lexer = new JSExprLexer(input), tokens = new org.antlr.runtime.CommonTokenStream(lexer), parser = new JSExprParser(tokens);
			var sheet = cell.getParent().getParent();
			parser.sheetName = sheet.getSheetName();
			currentParserSheetName = parser.sheetName;
			parser.document = sheet.getParent();
			parser.cell = cell;
			parser.tokenList = tokenList;
			ret = parser.prog();
//			websheet.ColumnHelper.serialize2String(cell,1);
			
		} 
		catch (e) {
			showValue.isError = true;
			if ((e.errorType != undefined)&&(e.errorType != null))
				showValue.errorType = e.errorType;
			else
				showValue.errorType = this.DEFAULTERROR;
            if(typeof e == "string")
            {
            	showValue.result = e;
            }
            else
            {
            	switch (e.errorType)
    			{
    				case this.NAERROR:
    					showValue.result = e.message;
    					break;
    				case this.UNSUPPORTEDFORMULA:
    					showValue.result = e.message;
    					break;
    				default:
    					showValue.result = "###Syntax Error###";
    					break;
    			}
            }

		}
		if (ret && ret.value!=undefined && ret.value!=null) {
			var result = ret.value.getShowValue();
			if(result instanceof websheet.model.RangeModel){
				if(result.getRows().length == 1){
					showValue.result =  result.getRows()[0];
				}else{
					showValue.result = "#VALUE!";
				}
			}else{
				showValue.result = result;
			}
			if(showValue.result == "NaN"){
				showValue.result = "#VALUE!";
			}
		}
		return showValue;
	},
	/**
	 * if reference cell has changed,will call this method to recalculate the influenced single part
	 * @param {object}token				  cell of token tree 
	 * @param {String}recalculativeValue  store the re-calculative value
	 * @return {String}recalculativeValue
	 */
	recalculateByPart:function(token,recalculateValue,doc,isError){
		try{
			if(token.getTokenType() == websheet.parse.tokenType.ARRAYFORMULA_TOKEN){
				return recalculateValue;
			}
			//if only a single cell,then return the cell value
			if(token.getParent() == null && token.getTokenType() != websheet.parse.tokenType.FUNCTION_TOKEN && token.getTokenType() != websheet.parse.tokenType.OPERATOR_TOKEN && token.getTokenType() != websheet.parse.tokenType.NEGATIVE_TOKEN){
				if(token.getTokenType() == websheet.parse.tokenType.CELLREF_TOKEN || token.getTokenType() == websheet.parse.tokenType.RANGEREF_TOKEN){
					var cell = token.getCalcuteValue().getCells()[0].getCellVieType();
					if(cell.isError){isError = true;}
					recalculateValue = cell.getShowValue();
				}else{
					recalculateValue = token.getCalcuteValue();
				}
			}
			//if it has parent,then find the parent recalculate its value.
			else if(token.getTokenType() == websheet.parse.tokenType.FUNCTION_TOKEN || token.getTokenType() == websheet.parse.tokenType.OPERATOR_TOKEN || token.getTokenType() == websheet.parse.tokenType.NEGATIVE_TOKEN){
				var ignoreError = this.isIgnoreErrorFormula(token.getName());
				var calculateValue = new Array();
				var list = null;
				var isNegative = false;
				if(token.getTokenType() == websheet.parse.tokenType.NEGATIVE_TOKEN){isNegative = true;}
				if(token.getTokenType() == websheet.parse.tokenType.FUNCTION_TOKEN){
					list = token.getTokenList().getCalcuteValue();
				}else{
					list = token.getTokenList();
				} 
				dojo.forEach(list,function(item){
					if(item.getTokenType() == websheet.parse.tokenType.RANGEREF_TOKEN){ 
						var cellRange = new websheet.model.RangeModel(item.getCalcuteValue().getStartAddress(),item.getCalcuteValue().getEndAddress());
						cellRange.isNegative = isNegative;
						var row = new dojox.collections.Dictionary();
						var col = new dojox.collections.Dictionary();
						var curRow = null;
						var cell = null;
						var cells = item.getCalcuteValue().getCells();
						dojo.forEach(cells,function(ROA,index){
							cell = ROA.getCellVieType();
							if(cell.isError && !ignoreError){
								throw cell.getShowValue();
							}
							var value;
							if(row.item(cell.getRow()) == undefined){
								row.add(cell.getRow(),new Array());
							}
							value = cell.getShowValue();
							if(isNegative){value = websheet.ColumnHelper.transForm2Negative(value);}
							if(col.item(cell.getCol()) == undefined){
								col.add(cell.getCol(),new Array());
							}
							row.item(cell.getRow()).push(value);
							col.item(cell.getCol()).push(value);
							cellRange.pushCell(ROA);
						},this);
						cellRange.setRows(row.getValueList());
						cellRange.setCols(col.getValueList());
						calculateValue.push(cellRange);
					}else{
						var showValue;
						if(item.getTokenType() == websheet.parse.tokenType.CELLREF_TOKEN){
							var nCell = item.getCalcuteValue().getCells()[0].getCellVieType();
							if(nCell.isError && !ignoreError){throw nCell.getShowValue();}
//							showValue = nCell.getShowValue();
							var rang = new websheet.model.RangeModel(nCell.getCol() + nCell.getRow());
							var cellValue = nCell.getShowValue();
							var ROA = new websheet.model.ROAModel(nCell,nCell.getSheetName(),nCell.getRow(),nCell.getCol(),websheet.model.ROAType.BOTHRELATIVE,false);
							rang.getRows().push(cellValue);
							rang.getCols().push(cellValue);
							rang.pushCell(ROA);
							showValue = rang;
						}else{
							if(item instanceof websheet.parse.tokenList || item instanceof websheet.parse.token){
								showValue = item.getCalcuteValue();
							}else{
								showValue = item;
							}
						}
						if(isNegative){showValue = websheet.ColumnHelper.transForm2Negative(showValue);}
						calculateValue.push(showValue);
					}
				});
				if(token.getTokenType() == websheet.parse.tokenType.FUNCTION_TOKEN){
					recalculateValue = websheet.functions.Complete.__get(token.getName())(calculateValue);
					token.setCalcuteValue(recalculateValue);
				}else if(token.getTokenType() == websheet.parse.tokenType.OPERATOR_TOKEN){
					recalculateValue = websheet.functions.Complete.MATH(token.getName(),websheet.ColumnHelper.analyzeResult(calculateValue[0]),websheet.ColumnHelper.analyzeResult(calculateValue[1]));
					token.setCalcuteValue(recalculateValue);
				}else if(token.getTokenType() == websheet.parse.tokenType.NEGATIVE_TOKEN){
					token.setCalcuteValue(calculateValue);
					if(dojo.isArray(calculateValue)){
						recalculateValue = calculateValue[0];
					}else{
						recalculateValue = calculateValue;
					}
				}
				if(token.getParent()){
					recalculateValue = dojo.hitch(this, "recalculateByPart")(token.getParent(),recalculateValue,doc,isError);
				}
				return recalculateValue;
			}else{
				var parent = token.getParent();
				if(parent.getTokenType() == websheet.parse.tokenType.FUNCTIONPARMAS_TOKEN){
					recalculateValue = dojo.hitch(this, "recalculateByPart")(parent.getParent(),recalculateValue,doc,isError);
				}else{
					recalculateValue = dojo.hitch(this, "recalculateByPart")(parent,recalculateValue,doc,isError);
				}
			}
			return recalculateValue;
		}catch(e){
			throw (e.message || e.toString() || "Error");
		}
	},
	
	getCurrentParseSheetName:function(){
		return currentParserSheetName;
	},
	
	isIgnoreErrorFormula:function(functName){
		functName = functName.toLowerCase();
		var formulaStr = new Array();
		var isIgnore = false;
		formulaStr.push("isnumber");
		formulaStr.push("isblank");
		formulaStr.push("countBlank");
		formulaStr.push("vlookup");
		formulaStr.push("hlookup");
		formulaStr.push("sumif");
		formulaStr.push("row");
		formulaStr.push("col");
		formulaStr.push("iserror");
		dojo.forEach(formulaStr,function(item){
			if(item == functName){
				isIgnore = true;
			}
		},this);
		return isIgnore;
	},
	
	isIgnoreRecursiveFormula:function(functName){
		functName = functName.toLowerCase();
		var formulaStr = new Array();
		var isIgnoreRecursive = false;
		formulaStr.push("row");
		formulaStr.push("col");
		formulaStr.push("iserror");
		dojo.forEach(formulaStr,function(item){
			if(item == functName){
				isIgnoreRecursive = true;
			}
		},this);
		return isIgnoreRecursive;
	}
};