// define the map between number and character
L2N = {
	"A" : 1,
	"B" : 2,
	"C" : 3,
	"D" : 4,
	"E" : 5,
	"F" : 6,
	"G" : 7,
	"H" : 8,
	"I" : 9,
	"J" : 10,
	"K" : 11,
	"L" : 12,
	"M" : 13,
	"N" : 14,
	"O" : 15,
	"P" : 16,
	"Q" : 17,
	"R" : 18,
	"S" : 19,
	"T" : 20,
	"U" : 21,
	"V" : 22,
	"W" : 23,
	"X" : 24,
	"Y" : 25,
	"Z" : 26
};
N2L = [ "", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M",
	   "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z" ];

DeleteRows = new Array();
DeleteColumns = new Array();
dColumn = 0;

// This function is to translate string like "AA" to 27
function string2Number(string) {
	var number = 0;
	var array = string.split("");
	for (i = 0; i < array.length; i++) {
		number = number * 26;
		number += L2N[array[i]];
	}
	return number;
}

// This function is to translate number like 27 to string "AA"
function number2String(number) {
	var quotient = number;
	var remain;
	var string = '';
	while (quotient > 0) {
		quotient = Math.floor(number / 26);
		remain = number % 26;
		if (remain == 0) {
			string = "Z" + string;
			quotient = quotient - 1;
		} else
			string = N2L[remain] + string;
		number = quotient;
	}
	return string;
}

// This function is to translate label like "A1" to row-column pair like [1,1];
function label2RowColumnPair(label) {
	// label should be a string like "A1"
	var rowString = label.match(/\d+/);
	var columnString = label.match(/[A-Z]+/).join();
	return [ Number(rowString), string2Number(columnString) ];
}

// This function is to translate row-column pair like [1,1] to label "A1"
function rowColumnPair2Label(pair) {
	// pair should be an array with row number and column number
	return number2String(pair[1]) + pair[0];
}

/*
 * Definition of Class Table This class is mainly responsible for showing and
 * moving the sheet in a specified vision area
 */
// page is the page on which to draw the table
// doc is the whole document
function Table(doc, row, column) {
	this.doc=doc;
	var item;
	for (item in doc.sheets)
		if(doc.sheets[item].index==1)
		{
			this.currentSheet=doc.sheets[item];
			this.index=1;
			break;
		}
	$("title").text(this.currentSheet.getSheetName());
	this.firstRowNumber = 1;
	this.firstColumnNumber = 1;
	this.visibleRows = row;
	this.visibleColumns = column;
	this.inEditMode=false;
	this.currentCellId="";
	this.rowArray=new Array(100);
}
Table.prototype.getFirstRowNumber=function(){
	return this.firstRowNumber;
}
Table.prototype.getFirstColumnNumber=function(){
	return this.firstColumnNumber;
}
Table.prototype.getVisibleRows = function() {
	return this.visibleRows;
};
Table.prototype.getVisibleColumns = function() {
	return this.visibleColumns;
};
Table.prototype.getVisibleRowsColumns = function() {
	return [ this.visibleRows, this.visibleColumns ];
};

Table.prototype.setVisibleRows = function(rows) {
	this.visibleRows = rows;
	this.drawTableArea();
	this.updateContent();
};
Table.prototype.setVisibleColumns = function(columns) {
	this.visibleColumns = columns;
	this.drawTableArea();
	this.updateContent();
};
// rowsColumns is an array like [row,column]
Table.prototype.setVisibleRowsColumns = function(rowsColumns) {
	if(this.visibleRows == rowsColumns[0] &&
	   this.visibleColumns == rowsColumns[1])
		return;
	this.visibleRows = rowsColumns[0];
	this.visibleColumns = rowsColumns[1];
	this.drawTableArea();
	this.updateContent();
};

// drawTable just draw the lines but does not load content
// Use updateContent to load the content for the table
Table.prototype.drawTableArea = function() {
	var table;
	var tr, th, td;
	var i, j;
	var id;
	
	var tableArea=$("#table_area");
	tableArea.empty();
	/* Start to draw table */
	table = $("<table></table>");
	table.attr("id", "table");
	
	// First draw the horizon heading
	tr = $('<tr id="tr0"></tr>');
	this.rowArray[0]=tr;
	th = $('<th id="00"></th>');
	
	tr.append(th);
	for (j = 0; j < this.visibleColumns; j++) {
		id = j + 1;
		id = "c" + id;
		th = $('<th id="' + id + '"></th>');
		tr.append(th);
	}
//	th = $('<th class="more_column"><input type="button" value="..."</input></th>');
//	tr.append(th);
	table.append(tr);
	
	// Then draw the vertical heading and cells
	for (i = 0; i < this.visibleRows; i++) {
		tr = $('<tr id="tr'+Number(i+1)+'"></tr>');
		this.rowArray[i+1]=tr;
		id = i + 1;
		id = "r" + id;
		th = $('<th id="' + id + '"></th>');
		tr.append(th);
		for (j = 0; j < this.visibleColumns; j++) {
			id = i + 1;
			var temp = j + 1;
			id = id + "_" + temp;
			td = $('<td id="' + id + '"></td>');
			tr.append(td);
		}
		
	//	td = $('<td class="more_column"></td>');
	//	tr.append(td);
		table.append(tr);
	}
	
//	tr=$('<tr id="tr'+Number(this.visibleRows+1)+'"></tr>');
//	th = $('<th class="more_row"><input type="button" value="..."></input></th>');
//	tr.append(th);
//	for(i=0;i<this.visibleColumns+1;i++){
//		td = $('<td class="more_row"></td>');
//		tr.append(td);
//	}
//	table.append(tr);
	
	tableArea.append(table);
	
	//set div #table_area heigth and autoscroll
	//	tableArea.css({"width":width,"height":height,"overflow-x":"auto","overflow-y":"auto"});
	//Add event handler for tds
	this.addHandler();
};

Table.prototype.drawPage = function() {
	this.drawTableArea();	
};

Table.prototype._editCell=function(self,target){
	var id, rowId, columnId;
	var tableRow, tableColumn, sheetRow, sheetColumn;
	
	$(".blue_bg").removeClass("blue_bg");
	target.addClass("blue_bg");
	id = target.attr("id");
	self.currentCellId=id;
	
	tableRow = Number(id.split("_")[0]);
	sheetRow = tableRow + self.firstRowNumber - 1;
	
	tableColumn = Number(id.split("_")[1]);
	sheetColumn = tableColumn + self.firstColumnNumber - 1;
	
	rowId = "r" + tableRow;
	$("#" + rowId).addClass("blue_bg");
	columnId = "c" + tableColumn;
	$("#" + columnId).addClass("blue_bg");
	
	var indexLabel=rowColumnPair2Label( [ sheetRow, sheetColumn ]);
	//	$("#index").text(rowColumnPair2Label( [ sheetRow, sheetColumn ]));
	var rawValue=self.currentSheet.getCellRawValue(sheetRow,number2String(sheetColumn));
	if(rawValue==null || rawValue==undefined)
		rawValue="";
	//TODO  add code to deal with android edit function.
	//window.location.href="js-call:startEdit:"+indexLabel+":"+rawValue+"0";//add a "0" to save the last " "(space)
	//	$("#value").val(rawValue);
	
	//Now set the "#value" focused.
	//	$("#value").focus();
}

Table.prototype._editCell=function(self,target){
	var id, rowId, columnId;
	var tableRow, tableColumn, sheetRow, sheetColumn;
	
	$(".blue_bg").removeClass("blue_bg");
	target.addClass("blue_bg");
	id = target.attr("id");
	self.currentCellId=id;
	
	tableRow = Number(id.split("_")[0]);
	sheetRow = tableRow + self.firstRowNumber - 1;
	
	tableColumn = Number(id.split("_")[1]);
	sheetColumn = tableColumn + self.firstColumnNumber - 1;
	
	rowId = "r" + tableRow;
	$("#" + rowId).addClass("blue_bg");
	columnId = "c" + tableColumn;
	$("#" + columnId).addClass("blue_bg");
	
	var indexLabel=rowColumnPair2Label( [ sheetRow, sheetColumn ]);
	//	$("#index").text(rowColumnPair2Label( [ sheetRow, sheetColumn ]));
	var rawValue=self.currentSheet.getCellRawValue(sheetRow,number2String(sheetColumn));
	if(rawValue==null || rawValue==undefined)
		rawValue="";
	nativeObj.startEditUI(indexLabel,rawValue);
}

// Add click handler for cells in the table
Table.prototype.addHandler = function() {
	var self=this;

	var editCell=function(event) {
		//alert("long press");
		var target=$(event.currentTarget);
		self._editCell(self,target);
	}
	
	var tap=function(event){
		//alert("tap");
		nativeObj.toggleToolbar();
	}
	
	$("#table td", self.page).not(".more_row").not(".more_column").longpress(editCell,tap);
	//add handler for viewing more rows.
//	$(".more_row").click(function(event){
//						 self.setVisibleRows(self.getVisibleRows()+5);
//						 self.drawTableArea();
//						 self.updateContent();
//						 });
//	$(".more_column").click(function(event){
//							self.setVisibleColumns(self.getVisibleColumns()+5);
//							self.drawTableArea();
//							self.updateContent();
//							});
};

Table.prototype.updateCellContent=function(index,rawValue){
	//if rawValue is a formula, then change all character to upper case
	if(rawValue.indexOf("=")==0 || rawValue.indexOf("{=")==0)
		rawValue=rawValue.toUpperCase();
	var pair = label2RowColumnPair(index);
	
	//get the cell raw value and decide whether to  make changes
	var originalRawValue=this.currentSheet.getCellRawValue(pair[0],number2String(pair[1]));
	if(originalRawValue==undefined || originalRawValue==null)
		originalRawValue="";
	
	var id = "#"+Number(pair[0] - this.firstRowNumber + 1) + "_"
	+ Number(pair[1] - this.firstColumnNumber + 1);
	
	if(originalRawValue==rawValue)
		return;
	
	//Associated with Huang Qun's work
	nativeObj.beginEdit();
	
	this.currentSheet.setCellRawValue(pair[0],number2String(pair[1]),rawValue,false);
	var cell = this.currentSheet.getCell(pair[0], number2String(pair[1]));
	var showValue=cell.getShowValue();
	$(id).text(showValue);
	//Associated with Huang Qun's work
	var firstParameter=this.currentSheet.getIndex()+","+pair[0]+","+pair[1];
	if(cell.isFormula)
	{
		nativeObj.update(firstParameter+",calculatedvalue",showValue);
		nativeObj.update(firstParameter+",formularawvalue",rawValue);
	}
	else
		nativeObj.update(firstParameter+",rawvalue",rawValue);
	
	nativeObj.update(firstParameter+",showvalue",showValue);
	
	
	var dependentedCells=cell.getDependencedList();
	var item;
	if(dependentedCells.length!=0)
		for (item in dependentedCells)
		{
			cell=dependentedCells[item];
			if(cell!=null)
			{
				var column =string2Number(cell.getColumn().index);
				var row=cell.getRow();
				this.updateDependentCellContent(cell,row,column);
				showValue=cell.getShowValue();
				firstParameter=this.currentSheet.getIndex()+","+row+","+column;
				nativeObj.update(firstParameter+",showvalue",showValue);
				if(cell.isFormula)
					nativeObj.update(firstParameter+",calculatedvalue",showValue);
			}			
		}
	
	nativeObj.endEdit();
	//	$(id).focus();
};

 Table.prototype.addValueBlurHandler = function() {
 // TODO what if the row and column of the cell is totally new?
 var value = $("#value");
 var index = $("#index");
 var self = this;
 value.blur(function() {
 if(index.text()=="")
 return;
 var rawValue = $(this).val();
 
 //if rawValue is a formula, then change all character to upper case
 if(rawValue.indexOf("=")==0 || rawValue.indexOf("{=")==0)
 rawValue=rawValue.toUpperCase();
 var pair = label2RowColumnPair(index.text());
 
 //get the cell raw value and decide whether to  make changes
 var originalRawValue=self.currentSheet.getCellRawValue(pair[0],number2String(pair[1]));
 if(originalRawValue==undefined || originalRawValue==null)
 originalRawValue="";
 
 var id = "#"+Number(pair[0] - self.firstRowNumber + 1) + "_"
 + Number(pair[1] - self.firstColumnNumber + 1);
 
 if(originalRawValue==rawValue)
 {
 $(id).focus();
 return;
 }
 
 //Associated with Huang Qun's work
 nativeObj.beginEdit();
 
 self.currentSheet.setCellRawValue(pair[0],number2String(pair[1]),rawValue,false);
 var cell = self.currentSheet.getCell(pair[0], number2String(pair[1]));
 var showValue=cell.getShowValue();
 $(id).text(showValue);
 //Associated with Huang Qun's work
 var firstParameter=self.currentSheet.getIndex()+","+pair[0]+","+pair[1];
 if(cell.isFormula)
 {
 nativeObj.update(firstParameter+",calculatedvalue",showValue);
 nativeObj.update(firstParameter+",formularawvalue",rawValue);
 }
 else
 nativeObj.update(firstParameter+",rawvalue",rawValue);
 
 nativeObj.update(firstParameter+",showvalue",showValue);
 
 
 var dependentedCells=cell.getDependencedList();
 var item;
 if(dependentedCells.length!=0)
 for (item in dependentedCells)
 {
 cell=dependentedCells[item];
 if(cell!=null)
 {
 var column =string2Number(cell.getColumn().index);
 var row=cell.getRow();
 self.updateDependentCellContent(cell,row,column);
 showValue=cell.getShowValue();
 firstParameter=self.currentSheet.getIndex()+","+row+","+column;
 nativeObj.update(firstParameter+",showvalue",showValue);
 if(cell.isFormula)
 nativeObj.update(firstParameter+",calculatedvalue",showValue);
 }
 
 }
 
 nativeObj.endEdit();
 $(id).focus();
 });
 };
Table.prototype.updateDependentCellContent=function(cell,row,column){
	//if the cell is in the visible area, update the td content
	if(row>=this.firstRowNumber 
	   && row<this.firstRowNumber+this.visibleRows
	   && column>=this.firstColumnNumber 
	   && column<this.firstRowNumber+this.visibleColumns){
		var id="#"+Number(row-this.firstRowNumber+1)+"_"+
		Number(column-this.firstColumnNumber+1);
		var showValue=cell.getShowValue();
		$(id).text(showValue);
	}
};

var flag = false;

Table.prototype.updateRange=function(startRow,startColumn,endRow,endColumn){
	var defaultHeight=content.styles.defaultrowstyle.height;
	var defaultWidth=content.styles.defaultcolumnstyle.width;
	//set the width of columns
	for(j=startColumn;j<=endColumn;j++)
	{
		var width;
		var column=this.currentSheet.getColumn(j);
		if(column==null || column==undefined)
			break;
		width=column.width;
		if(width==null || width == undefined)
			width=defaultWidth;
		$("#c"+j).css("width",toPx(width));
	}
	
	var i,j;
	var doc=this.currentSheet.getParent();
	for(i=startRow;i<=endRow;i++)
	{
		//Add style for the row
		var height;
		var row=this.currentSheet.getRow(i);
		if(row==null || row ==undefined)
			break;
		height=row.height;
		if(height ==null || height == undefined)
			height=defaultHeight;
		$("#tr"+i).css("height",toPx(height));
		
		//update content of the row
		for(j=startColumn;j<=endColumn;j++)
		{
			//get the column
			var column=this.currentSheet.getColumn(j);
			var columnStyleCode;
			if(column==null || column ==undefined)
				break;
			columnStyleCode=column.getStyle();
			
			//get the cell
			var cell=this.currentSheet.getCell(i,number2String(j));
			if(cell==null || cell==undefined)
			{
				var tdId="#"+Number(i-this.firstRowNumber+1)+"_"+Number(j-this.firstColumnNumber+1);
				var td=$(tdId);
				//set the style of the cell to default column style
				if(columnStyleCode!=null &&columnStyleCode!=undefined)
					this.setCellStyle(td,columnStyleCode);
				td.text("");
				continue;
			}
			
			//set show value and style
			var repeatNum=cell.repeatednum;
			if(repeatNum==null || repeatNum==undefined)
				repeatNum=0;
			var k;
			//TODO There is a bug in the concord code.This is a temporary solution
			if(cell.isFormula)
				cell.setRawValue(cell.getRawValue());
			
			var showValue=cell.getShowValue();
			if(showValue==null || showValue==undefined)
				showValue="";
			var styleCode=cell.styleCode;
			
			var tdId="#"+Number(i-this.firstRowNumber+1)+"_"+Number(j-this.firstColumnNumber+1);
			var td=$(tdId);
			//set show value
			td.text(showValue);
			
			//set span
			if(cell.colspan != null){
				td.attr("colspan", cell.colspan);
			}

			if(cell.rowspan != null){
				td.attr("rowspan", cell.rowspan);
			}
			
			if(cell.text_align != null){
				td.attr("align",cell.text_align);
			}
			
			//set style
			if(styleCode==null || styleCode == undefined)
				styleCode=columnStyleCode;
			if(styleCode!=null && styleCode != undefined)
				this.setCellStyle(td,styleCode);
			
			
			for(k=0;k<repeatNum;k++)
			{
				tdId="#"+Number(i-this.firstRowNumber+1)+"_"+Number(j-this.firstColumnNumber+2+k);
				td=$(tdId);
				//set show value
				td.text(showValue);
				//set style
				if(styleCode!=null && styleCode != undefined)
					this.setCellStyle(td,styleCode);
			}
			
			j=j+repeatNum;
		}
	}
	
	//Remove the redundant rows from the former pages
	for(i=startRow;i<=endRow;i++)
	{
		if(DeleteColumns[i] == 0 || DeleteColumns[i] == null || DeleteColumns[i] == undefined)
			continue;
		//if(DeleteColumns[i]<this.visibleColumns)
		if(DeleteColumns[i]<dColumn)
		{
			for(var k = 0; k<DeleteColumns[i];k++)
			{
				var del_td_2 = "#"+Number(i-this.firstRowNumber+1)+"_"+Number(startColumn + k + 1 - this.firstColumnNumber);
				var td_2 = $(del_td_2);
				td_2.remove();
			}
			DeleteColumns[i] = 0;
		}
		else
		{
			//for(var k = 0;k<visibleColumns-1;k++)
			for(var k=0;k<dColumn;k++)
			{
				var del_td_2 = "#"+Number(i-this.firstRowNumber+1)+"_"+Number(startColumn + k + 1 - this.firstColumnNumber);
				var td_2 = $(del_td_2);
				td_2.remove();
			}
			//DeleteColumns[i] = DeleteColumns[i] - dColumn;
			DeleteColumns[i] = DeleteColumns[i] - dColumn;
		}
	}
	
	//Remove the redundant cell if there are merged cells at first time
	for(i=startRow;i<=endRow;i++) {
		var row=this.currentSheet.getRow(i);
		if(row==null || row ==undefined)
			break;
		for(j=startColumn;j<=endColumn;j++){
			var column=this.currentSheet.getColumn(j);
			if(column==null || column ==undefined)
				break;
			//get the cell
			var cell=this.currentSheet.getCell(i,number2String(j));
			if(cell!=null && cell!=undefined)
			{
				if(cell.colspan != null){
					if(cell.rowspan == null || cell.rowspan==1){
						//can be cleaned once
						//if(cell.colspan < this.visibleColumns){
						//if could be drawed at once
						if(cell.colspan < this.visibleColumns){
							for (var k = 0; k < cell.colspan-1; k++) {
								var del_td ="#"+Number(i-this.firstRowNumber+1)+"_"+Number(j-this.firstColumnNumber+1+1+k); 
								var td = $(del_td);
								td.remove();
							}
						}
						else//cannot be cleaned once
						{
							//calculate the difference and record it
							DeleteColumns[i] = cell.colspan - this.visibleColumns + j - this.firstColumnNumber;
							//remove the redundant cells in this screen
							for (var k = 0; k < cell.colspan-1; k++) {
								var del_td ="#"+Number(i-this.firstRowNumber+1)+"_"+Number(j-this.firstColumnNumber+1+1+k); 
								var td = $(del_td);
								td.remove();
							}
						}
					}
					else//cell.rowspan>1
					{
						if(cell.colspan < this.visibleColumns){
							//clean each row
							for(var m = 0; m<cell.rowspan;m++){
								if(m==0){
									for (var k = 0; k < cell.colspan-1; k++) {
										var del_td ="#"+Number(i-this.firstRowNumber+1)+"_"+Number(j-this.firstColumnNumber+1+1+k); 
										var td = $(del_td);
										td.remove();
									}
								}
								else{
									for(var k = 0; k < cell.colspan;k++){
										var del_td = "#"+Number(i-this.firstRowNumber+1+m)+"_"+Number(j-this.firstColumnNumber+1+k);
										var td = $(del_td);
										td.remove();
									}
								}
							}
						}
						else{
						//cannot be cleaned at once
							for(var m=0;m<cell.rowspan;m++){
								if(m==0){
									DeleteColumns[i] = cell.colspan - this.visibleColumns + j - this.firstColumnNumber;
									for (var k = 0; k < cell.colspan-1; k++) {
										var del_td ="#"+Number(i-this.firstRowNumber+1)+"_"+Number(j-this.firstColumnNumber+1+1+k); 
										var td = $(del_td);
										td.remove();
									}
								}
								else{//following rows
									DeleteColumns[i+m] = cell.colspan - this.visibleColumns + j - this.firstColumnNumber;
									for (var k = 0; k < cell.colspan-1; k++) {
										var del_td ="#"+Number(i-this.firstRowNumber+1+m)+"_"+Number(j-this.firstColumnNumber+1+k); 
										var td = $(del_td);
										td.remove();
									}
								}
							}
						}
							
					}
							
				}
				//the page switched. new column need to be deleted.

				if(cell.rowspan != null){
					for (var k = 0; k < cell.rowspan-1; k++) {
						var del_td ="#"+Number(i-this.firstRowNumber+1+1+k)+"_"+Number(j-this.firstColumnNumber+1); 
						var td = $(del_td);
						td.remove();
					}
				}
			}
		}
	}
	
};
Table.prototype.updateContent = function() {
	var self = this;

	// First update the headings
	var updateTh = function(index) {
		var id;
		id = $(this).attr("id");
		if (id.charAt(0) == "r") {
			var rowNumber = self.firstRowNumber - 1
			+ Number(id.substring(1, id.length));
			$(this).text(rowNumber);
		} else if (id.charAt(0) == "c") {
			var columnNumber = Number(id.substring(1, id.length))
			+ self.firstColumnNumber - 1;
			$(this).text(number2String(columnNumber));
		}
	};
	$("#table").find("th").each(updateTh);
	
	// Then update the cell content
	var row1 = this.firstRowNumber;
	var column1 = this.firstColumnNumber;
	var row2 = row1 + this.visibleRows - 1;
	var column2 = column1 + this.visibleColumns - 1;
	this.updateRange(row1,column1,row2,column2);
};

//TODO
function toPx(origin)
{
	if(origin=="thin"||origin=="thick"||origin=="medium"||
	   origin.indexOf("px")!=-1)
		return origin;
	var index=origin.indexOf("in");
	if(index!=-1)
	{
		var inch=Number(origin.substring(0,index));
		return Math.ceil(inch*72)+"px";
	}
	index=origin.indexOf("cm");
	if(index!=-1)
	{
		var cm=Number(origin.substring(0,index));
		return Math.ceil(cm*28.35)+"px";
	}
	return "1px";
}
Table.prototype.setCellStyle=function(cell,styleCode){
	if(styleCode==null || styleCode==undefined)
		return;
	var styleMap="";
	var fontName=styleCode.getFontname();
	var fontSize=styleCode.getFontsize();
	var fontColor=styleCode.getFontcolor();
	var isItalic=styleCode.isItalic();
	var isUnderline=styleCode.isUnderline();
	var isStrikethrough=styleCode.isStrikethrough();
	var isBold=styleCode.isBold();
	var borderLeft=styleCode.getBorderLeft();
	var borderRight=styleCode.getBorderRight();
	var borderTop=styleCode.getBorderTop();
	var borderBottom=styleCode.getBorderBottom();
	var borderLeftColor=styleCode.getBorderLeftColor();
	var borderRightColor=styleCode.getBorderRightColor();
	var borderTopColor=styleCode.getBorderTopColor();
	var borderBottomColor=styleCode.getBorderBottomColor();
	var alignment=styleCode.getAlignment();
	var backgroundColor=styleCode.getbgColor();
	var indent=styleCode.getIndent();
	var autoWrap=styleCode.getAutoWrap();
	
	var setBorderColor=false;
	
	if(fontName!=null)
		cell.css('font-family',fontName);
	if(fontSize!=null)
		cell.css('font-size',fontSize);
	if(fontColor!=null)
		cell.css('color',fontColor);
	if(isItalic !=null && isItalic)
		cell.css('font-style','italic');
	if(isUnderline!=null && isUnderline)
		cell.css('text-decoration','underline');
	if(isStrikethrough!=null && isStrikethrough)
		cell.css('text-decoration','line-through');
	if(isBold !=null && isBold)
		cell.css('font-weight','bold');
	
	if(borderLeft!=null)
	{
		cell.css({'border-left-width':toPx(borderLeft),'border-left-color':'black'});
	}
	if(borderRight!=null)
		cell.css({'border-right-width':toPx(borderRight),'border-right-color':'black'});
	if(borderTop!=null)
		cell.css({'border-top-width':toPx(borderTop),'border-top-color':'black'});
	if(borderBottom!=null)
		cell.css({'border-bottom-width':toPx(borderBottom),'border-bottom-color':'black'});
	
	if(borderLeftColor!=null)
	{
		cell.css('border-left-color',borderLeftColor);
	}
	if(borderRightColor!=null)
	{
		cell.css('border-right-color',borderRightColor);
	}
	if(borderTopColor!=null)
	{
		cell.css('border-top-color',borderTopColor);
	}
	if(borderBottomColor!=null)
	{
		cell.css('border-bottom-color',borderBottomColor);
	}
	
	
	if(alignment!=null){
		//cell.css('text-align',alignment);
		if(alignment=="start")
			cell.css('text-align','left');
		if(alignment=="center")
			cell.css('text-align','center');
		if(alignment=="end")
			cell.css('text-align','right');
	}
	
//	if(alignment!=null)
//		cell.css('text-align',alignment);
	if(backgroundColor!=null)
		cell.css('background-color',backgroundColor);
	if(indent!=null)
		cell.css('text-indent',indent);
};
// move the table so that the left corner cell is cell(row,column)
Table.prototype.moveTo = function(row, column) {
	//alert("row: "+row+" , column: "+column);
	if(row<=0)
		row=1;
	if(column<=0)
		column=1;
	if(row==this.firstRowNumber && column==this.firstColumnNumber)
		return;
	this.firstRowNumber = row;
	this.firstColumnNumber = column;
	this.drawTableArea();
	this.updateContent();
};

Table.prototype.showSheetByIndex=function(index){
	var sheets=this.doc.sheets;
	//in java code, index start from 0
	//while in SheetModel, index start from 1,so add index by 1.
	index++;
	if(index<=0||index>sheets.length||index==this.index)
	{
		nativeObj.setContentWidth($(document).width());
		nativeObj.setContentHeight($(document).height());
		nativeObj.finishLoading();
		return;
	}
	
	for (item in sheets)
		if(sheets[item].index==index)
		{
			this.currentSheet=sheets[item];
			this.index=index;
			break;
		}
	this.firstColumnNumber=1;
	this.firstRowNumber=1;
	this.visibleColumns=10;
	this.visibleRows=30;
	this.drawTableArea();
	this.updateContent();
	window.scrollTo(0,0);
	nativeObj.setContentWidth($(document).width());
	nativeObj.setContentHeight($(document).height());
	nativeObj.finishLoading();
}
Table.prototype.getAllSheetName=function(){
	var sheets=this.doc.sheets;
	var names=[];
	for (sheet in sheets){
		names.push(sheets[sheet].getSheetName());
	}
	return names.join(";");
}
Table.prototype.stopEdit=function(){
	$(".blue_bg").removeClass("blue_bg");
}
Table.prototype.editNextCell=function(){
	//	alert("editNextCell");
	var row,column;
	var rowColumnPair=this.currentCellId.split("_");
	row=Number(rowColumnPair[0]);
	column=Number(rowColumnPair[1]);
	row=row +1;
	if(row>this.visibleRows)
	{
		this.setVisibleRows(this.getVisibleRows()+1);
	}
	
	var nextCellId=[row,column].join("_");
	this.currentCellId=nextCellId;
	var target=$("#"+nextCellId);
	this._editCell(this,target);
}

Table.prototype.moveViewPort=function(x,y){
	x=Number(x);
	y=Number(y);
	if(x>0)//move to right view port
		this.moveTo(this.firstRowNumber,this.firstColumnNumber+Math.round(this.visibleColumns/3));
	else if(x<0)//move to left view port
		this.moveTo(this.firstRowNumber,this.firstColumnNumber-Math.round(this.visibleColumns/3));
	if(y>0)//move to downside port
		this.moveTo(this.firstRowNumber+Math.round(this.visibleRows/3),this.firstColumnNumber);
	else if(y<0)//move to upside port
		this.moveTo(this.firstRowNumber-Math.round(this.visibleRows/3),this.firstColumnNumber);
};


Table.prototype.sendSheetNames=function(){
	sheetNames=""
	for (item in this.doc.sheets)
	{
		//alert(this.doc.sheets[item].getSheetName());
		sheetNames += this.doc.sheets[item].getSheetName()+"_-_-";
	}
	//alert(sheetNames);
	sheetNames=sheetNames.substr(0,sheetNames.length-4);
	nativeObj.updateSheetNames(sheetNames);
};
Table.prototype.finishLoading=function(){
	nativeObj.finishLoading();
	nativeObj.setContentWidth($(document).width());
	nativeObj.setContentHeight($(document).height());
};

Table.prototype.addRowColumn=function(rows,columns){
        if(rows>0)
	        this.addRow(rows);
	if(columns>0)
        	this.addColumn(columns);
	nativeObj.setContentWidth($(document).width());
	nativeObj.setContentHeight($(document).height());
	nativeObj.finishLoading();
};
Table.prototype.addRow=function(rows){
	var self=this;

	var editCell=function(event) {
		//alert("long press");
		var target=$(event.currentTarget);
		self._editCell(self,target);
	};
	
	var tap=function(event){
		//alert("tap");
		nativeObj.toggleToolbar();
	};
	
	var table=$("table");
	//Draw the vertical heading and cells
	var i;
	for (i = 0; i < rows; i++) {
		var id = 1+this.visibleRows+i;
		tr = $('<tr id="tr'+id+'"></tr>');
		this.rowArray[id]=tr;
		
		th = $('<th id="r' + id + '"></th>');
		th.text(Number(id+this.firstRowNumber-1));
		tr.append(th);
		for (j = 0; j < this.visibleColumns; j++) {
			var temp = j + 1;
			td = $('<td id="' + id+'_' + temp + '"></td>');
			td.longpress(editCell,tap);
			tr.append(td);
		}
		table.append(tr);
	}

	var startRow=this.firstRowNumber+this.visibleRows;
	var endRow=startRow+rows-1;
	var startColumn=this.firstColumnNumber;
	var endColumn=this.firstColumnNumber+this.visibleColumns-1;
	this.updateRange(startRow,startColumn,endRow,endColumn);
	this.visibleRows+=rows;	
};
Table.prototype.addColumn=function(columns){

	var self=this;

	var editCell=function(event) {
		//alert("long press");
		var target=$(event.currentTarget);
		self._editCell(self,target);
	};
	
	var tap=function(event){
		//alert("tap");
		nativeObj.toggleToolbar();
	};
	//First add column th.
	var i,j;
	for(i=0;i<columns;i++){
		var thTr=this.rowArray[0];
		var temp=i+this.visibleColumns+1;
		var th=$('<th id="c'+temp+'"></th>');
		th.text(number2String(temp+this.firstColumnNumber-1));
		thTr.append(th);
	}
	for(j=1;j<=this.visibleRows;j++){
        	var tr=this.rowArray[j];
	        for(i=1;i<=columns;i++){
	                var td=$('<td id="'+ j +'_'+Number(this.visibleColumns+i)
	                        +'"></td>');
	                td.longpress(editCell,tap);
	                tr.append(td);    
	        }
	}
	var startRow=this.firstRowNumber;
	var endRow=this.firstRowNumber+this.visibleRows-1;
	var startColumn=this.firstColumnNumber+this.visibleColumns;
	var endColumn=startColumn+columns-1;
	dColumn = columns;
	this.updateRange(startRow,startColumn,endRow,endColumn);
	this.visibleColumns+=columns;
	//dColumn = columns;
};
/* END of Class Table */
